/*
 * @(#)AbstractActionListCell.java 1.2 02.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
//import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Map;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.JComboBox;



/** An abstract action for list cells.
 * 
 * @author sven.luzar
 * @version 2.0
 *
 */
public abstract class AbstractActionListCell extends AbstractActionList {

 /** A dummy jgraph for the menu components based
  *  on jgraph components. (For example the arrows.)
  * 
  */
 public static JGraph dummyGraph = new JGraph();
 
 /** Indicates whether an action has been reset (if it was just performed).
  * 
  */ 
 public static final String IDENTIFIER_RESET = " X";
 
 /** Indicates if an action is being selected.
  * 
  */
 public static final String IDENTIFIER_SELECT = "...";
 
 /** A component for holding the arrow starting points within cells.  Along
  * with the field <tt>arrowEndPoints</tt>, represent an arrow.
  * 
  */ 
 protected static java.util.List arrowPoints = new ArrayList();
 
 /** A component for holding the arrow ending points within cells.  Along with the field
  * <tt>arrowPoints</tt>, represent an arrow.
  * 
  */ 
 protected static java.util.List arrowEndPoints = new ArrayList();

 
 /**  Initializes arrows within cells.
  * 
  */ 
 static {
  arrowPoints.add(new Point(0, 6));
  arrowPoints.add(new Point(40, 6));
  arrowEndPoints.add(new Point(0, 6));
  arrowEndPoints.add(new Point(14, 6));
 }
 

 /**
  * Constructor for AbstractActionListCell.
  * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
  */
 public AbstractActionListCell(GPGraphpad graphpad) {
  super(graphpad);
 }

 /**
  * Constructor for AbstractActionListCell.
  * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
  * @param name This string is the name of the action, and is sent to super().
  */
 public AbstractActionListCell(GPGraphpad graphpad, String name) {
  super(graphpad, name);
 }

 /**
  * Constructor for AbstractActionListCell.
  * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
  * @param name This string is the name of the action, and is sent
  * to super().
  * @param icon  The icon for the action, and is sent to super().
  */
 public AbstractActionListCell(
  GPGraphpad graphpad,
  String name,
  Icon icon) {
  super(graphpad, name, icon);
 }

 /** Gets JComboBox for current cell within list.
  * @see com.jgraph.pad.actions.AbstractActionList#getToolBarComponent()
  * @return Returns a JComboBox.
  */
 protected JComboBox getToolBarComponent() {
  JComboBox combo = super.getToolBarComponent();
  combo.setRenderer(new GraphListCellRenderer(this));
  combo.setPreferredSize(new Dimension(45, 26));
  combo.setMaximumSize(combo.getPreferredSize());
  return combo;
 }

 /** Gets the items of the cell.
  * @see com.jgraph.pad.actions.AbstractActionList#getItems()
  * @return Returns an array of item Objects.
  */
 protected Object[] getItems() {
  Vector items = new Vector();

  items.add(IDENTIFIER_RESET);
  items.add(IDENTIFIER_SELECT);

  fillCustomItems(items);

  return items.toArray();
 }

 /** Performs an action, which is indicated by the parameter <tt>e</tt>.
  * @param e This holds the information for the current action to be performed.
  * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
  */
 public void actionPerformed(ActionEvent e) {
  Object source = e.getSource();
  Map map = GraphConstants.createMap();
  if (source instanceof JComboBox) {
   JComboBox box = (JComboBox) e.getSource();
   Object item = box.getSelectedItem();
   if (item instanceof CellView) {
    fillApplyMap((CellView) item, map);
   }
   if (item == IDENTIFIER_SELECT) {
    selectAndFillMap(map);
   } else if (item == IDENTIFIER_RESET) {
    fillResetMap(map);
   }

  }
  setSelectionAttributes(map);

 }
 
 /** Abstract method. 
  */ 
 protected abstract void fillCustomItems(Vector items);

 /** Abstract method. 
  */ 
 protected abstract void fillResetMap(Map target);

 /** Abstract method. 
  */ 
 protected abstract void fillApplyMap(CellView source, Map target);

 /** Abstract method. 
  */ 
 protected abstract void selectAndFillMap(Map target);
 
 /** The size of the cell.
  */
 protected Dimension size = new Dimension(14, 15);
 
 /** The origin of the cell (0,0).
  */ 
 protected Point point = new Point(0, 0);


}
