/*
 * @(#)AbstractActionRadioButton.java 1.2 01.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.Component;

//import javax.swing.AbstractButton;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JRadioButton;
import javax.swing.JRadioButtonMenuItem;


/**  An abstract action for buttons.
 * 
 * @author sven.luzar
 * @version 2.0
 *
 */
public abstract class AbstractActionRadioButton extends AbstractActionToggle {
  
  /** Contains the last Action Command.
   * 
   */
  public String lastActionCommand = null;
  
  /**
   * Constructor for AbstractActionRadioButton.
   * @param graphpad This is a reference to the graphpad for this action,
   * and is sent to super().
   */
  public AbstractActionRadioButton(GPGraphpad graphpad) {
    super(graphpad);
  }
  
  /**
   * Constructor for AbstractActionRadioButton.
   * @param graphpad This is a reference to the graphpad for this action,
   * and is sent to super().
   * @param name The name of the action to be performed, and is
   * sent to super().
   */
  public AbstractActionRadioButton(GPGraphpad graphpad, String name) {
    super(graphpad, name);
  }
  
  /**
   * Constructor for AbstractActionRadioButton.
   * @param graphpad This is a reference to the graphpad for this action,
   * and is sent to super().
   * @param name The name of the action to be performed, and is 
   * sent to super().
   * @param icon The icon for the action, and is sent to super().
   */
  public AbstractActionRadioButton(
                                   GPGraphpad graphpad,
                                   String name,
                                   Icon icon) {
    super(graphpad, name, icon);
  }
  
  /** Abstract method. */
  public abstract String[] getPossibleActionCommands();
  
  /** Returns an array of components, with links to this action.
   * @see com.jgraph.pad.actions.AbstractActionDefault#getMenuBarComponents()
   * @return Returns an array of Component objects.
   */
  public Component[] getMenuComponents() {
    String[] actionCommands = getPossibleActionCommands();
    
    Component[] components = new JComponent[actionCommands.length ];
    
    for (int i = 0; i < actionCommands.length ; i++){
      components[i] = getMenuComponent(actionCommands[i]);
    }
    
    return components;
  }
  
  /** Gets the tools for this action.
   * @see com.jgraph.pad.actions.AbstractActionDefault#getToolBarComponents()
   * @return Returns an array of Component objects.
   */
  public Component[] getToolComponents() {
    String[] actionCommands = getPossibleActionCommands();
    
    Component[] components = new JComponent[actionCommands.length ];
    
    for (int i = 0; i < actionCommands.length ; i++){
      components[i] = getToolComponent(actionCommands[i]);
    }
    
    return components;
  }
  
  
  /** Returns a JMenuItem with a link to this action.
   * @see com.jgraph.pad.actions.AbstractActionDefault#createMenuBarComponent()
   * @return Returns a button as a JMenuItem Object.
   */
  protected Component getMenuComponent(String actionCommand) {
    JMenuItem button = new JRadioButtonMenuItem(this);
    GPBarFactory.fillAbstractButton(button, getName()+actionCommand, actionCommand);
    abstractButtons.add(button);
    if (lastActionCommand.endsWith(actionCommand))
      button.setSelected(true);
    String presentationText = getPresentationText(actionCommand);
    if (presentationText != null)
      button.setText(presentationText);
    
    return button;
  }
  
  /** Returns a JMenuItem tool with a link to this action.
   * @see com.jgraph.pad.actions.AbstractActionDefault#createToolBarComponent()
   * @return Returns a button as a JMenuItem Object.
   */
  protected Component getToolComponent(String actionCommand) {
    JRadioButton button = new JRadioButton(this);
    GPBarFactory.fillAbstractButton(button, getName()+actionCommand, actionCommand, false);
    abstractButtons.add(button);
    if (lastActionCommand.endsWith(actionCommand))
      button.setSelected(true);
    return button;
  }
  
  /** Checks to see if the final action command was given.
   * @see com.jgraph.pad.actions.AbstractActionToggle#isSelected(String)
   * @return Returns <tt>true</tt> if it is the final action, <tt>false</tt> otherwise.
   */
  public boolean isSelected(String actionCommand) {
    return actionCommand.equals(lastActionCommand);
  }
}
