import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
//import java.io.PrintStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.TooManyListenersException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.CellRendererPane;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.TransferHandler;
import javax.swing.UIManager;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.MouseInputListener;
import javax.swing.plaf.ActionMapUIResource;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.UIResource;
import javax.swing.tree.DefaultMutableTreeNode;

// Referenced classes of package com.jgraph.plaf.basic:
//            BasicGraphDropTargetListener

/**
 * The public class BasicGraphUI that extends GraphUI. This class describes the user interface for the graphpad application.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 *
 */
public class BasicGraphUI extends GraphUI
    implements Serializable
{
    /** The class GraphTransferHandler that extends TransferHandler.
     */
    public class GraphTransferHandler extends TransferHandler
    {
        /**
         * The object sent out when transferring control.
         */
        protected Object out;

        /** The object that comes in when transferring control.
         */
        protected Object in;

        /** The number of object that have come in for this graph.
         */
        protected int inCount;

        /** Determines whether a component can be imported.
         *
         * @param jcomponent The JComponent with which you are working.
         * @param adataflavor This is a DataFlavor[].
         * @return Returns true always.
         */
        public boolean canImport(JComponent jcomponent, DataFlavor adataflavor[])
        {
            return true;
        }

        /** The public method createTransferable calls createTransferable(JComponent).
         *
         * @return  Returns the output of the method createTransferable(JComponent)*/
        public Transferable createTransferable()
        {
            return createTransferable(((JComponent) (graph)));
        }

        /** The protected final method createTransferable(JComponent)
         *
         * @param jcomponent This is the JComponent with which you are working.
         * @return Returns a Transferable object from the method create(JGraph, Object, ConnectionSet, Map, Rectangle) */
        protected final Transferable createTransferable(JComponent jcomponent)
        {
            if(jcomponent instanceof JGraph)
            {
                JGraph jgraph = (JGraph)jcomponent;
                Object aobj[] = graphLayoutCache.order(jgraph.getSelectionCells());
                if(aobj != null && aobj.length > 0)
                {
                    Object aobj1[] = jgraph.getDescendants(aobj);
                    ConnectionSet connectionset = ConnectionSet.create(graphModel, aobj1, false);
                    Map map = GraphConstants.createAttributes(aobj1, graphLayoutCache);
                    Rectangle rectangle = jgraph.getCellBounds(aobj);
                    out = ((Object) (aobj));
                    return create(jgraph, aobj, connectionset, map, rectangle);
                }
            }
            return null;
        }

        /** The protected method create
         * @param jgraph A JGraph
         * @param aobj An Object
         * @param connectionset A ConnectionSet
         * @param map A Map
         * @param rectangle A Rectangle
         * @return Returns a new GraphTransferable object created with aobj, connectionset, map, and rectangle. */
        protected GraphTransferable create(JGraph jgraph, Object aobj[], ConnectionSet connectionset, Map map, Rectangle rectangle)
        {
            return new GraphTransferable(aobj, connectionset, map, rectangle);
        }

        /** The protected method exportDone
         * @param jcomponent This is a JComponent and may be an instance of JGraph.
         * @param transferable This is a Transferable and may be an instance of GraphTransferable.
         * @param i An integer.  If i == 2, the descendants of the cells of transferrable will be removed from the GraphLayoutCache. */
        public void exportDone(JComponent jcomponent, Transferable transferable, int i)
        {
        	//System.out.println ("Called on Ctrl +C = ExportDone method" ) ;
        	//	(new Exception()).printStackTrace() ;
        	if((jcomponent instanceof JGraph) && (transferable instanceof GraphTransferable) && i == 2)
        	{
        		JGraph jgraph = (JGraph)jcomponent;
        		Object aobj[] = ((GraphTransferable)transferable).getCells();
        		jgraph.getGraphLayoutCache().remove(jgraph.getDescendants(aobj));
        	}
        	updateHandle();
        	setInsertionLocation(null);
        	
        	// If the action is copy, then copy to the clipboard. This check is necessary because this 
        	// method is also called on the cut action.
        	if (TransferHandler.COPY == i) 
        		EditCopy.Copy() ;
        }

        /** The method getSourceActions
         * @param jcomponent This is a JComponent
         * @return Returns 3 */
        public int getSourceActions(JComponent jcomponent)
        {
            return 3;
        }

// --> New JGraph importdata function

// NOTE: 1. We abuse return value to signal removal to the sender.
//       2. We always clone cells when transferred between two models
//          This is because they contain parts of the model's data.
//       3. Transfer is passed to importDataImpl for unsupported
//          dataflavors (becaue method may return false, see 1.)
/** The public method importData
 * @param comp This is a JComponent and may be an instance of JGraph
 * @param t This is a Transferable.
 * @return Returns true or false, and this signals removal to the sender. */
public boolean importData(JComponent comp, Transferable t) 
{
	try 
	{
		if (comp instanceof JGraph) 
		{
			JGraph graph = (JGraph) comp;
			
			//System.out.println ( "ImportData " + GraphTransferable.dataFlavor) ;
			
			if (t.isDataFlavorSupported(GraphTransferable.dataFlavor)) 
			{
				// May be null
				Point p = getInsertionLocation();
				
				// Get Local Machine Flavor
				Object obj = t.getTransferData(GraphTransferable.dataFlavor);
				GraphTransferable gt = (GraphTransferable) obj;
				
				// Get Transferred Cells
				Object[] cells = gt.getCells();
				
				// Check if all cells are in the model
				boolean allInModel = true;
				for (int i = 0; i < cells.length && allInModel; i++)
					allInModel = allInModel && graph.getModel().contains(cells[i]);
				
				// Count repetitive inserts
				if (in == cells)
					inCount++;
				else
					inCount = (allInModel) ? 1 : 0;
					in = cells;
					
					// Delegate to handle
					if (p != null && in == out && handle != null) {
						int mod =(dropAction == TransferHandler.COPY)? InputEvent.CTRL_MASK: 0;
						handle.mouseReleased(new MouseEvent(comp,0,0,mod,p.x,p.y,1,false));
						return false;
					}
					
					// Get more Transfer Data
					Rectangle bounds = gt.getBounds();
					Map nested = gt.getAttributeMap();
					ConnectionSet cs = gt.getConnectionSet();
					
					// Move across models or via clipboard always clones
					if (!allInModel|| p == null || dropAction == TransferHandler.COPY) 
					{
						
						// Create map of (cell, clone) pairs
						Map clones = graph.cloneCells(cells);
						
						// Replace keys in transfer data
						nested = GraphConstants.replaceKeys(clones, nested);
						cs = cs.clone(clones);
						
						// Translate cells
						int dx = 0;
						int dy = 0;
						
						// Cloned via Drag and Drop
						if (p != null) 
						{
							Point insert = graph.fromScreen(graph.snap(new Point(p)));
							dx = insert.x - bounds.x;
							dy = insert.y - bounds.y;
							
							// Cloned via Clipboard
						} else {
							dx = inCount * graph.getGridSize();
							dy = inCount * graph.getGridSize();
						}
						GraphConstants.translate(nested.values(), dx, dy);
						
						// Replace cells in order
						Object[] newCells = new Object[cells.length];
						for (int i = 0; i < cells.length; i++)
							newCells[i] = clones.get(cells[i]);
						
						// Insert and exit
						graph.getGraphLayoutCache().insert(newCells,nested,cs,null,null);
						
						// Signal sender to remove only if moved between different models
						return (dropAction == TransferHandler.MOVE&& !allInModel);
						
						// We are dealing with a move across multiple views of the same model
					} 
					else 
					{	
						// Moved via Drag and Drop
						if (p != null) {
							// Scale insertion location
							Point insert =graph.fromScreen(graph.snap(new Point(p)));
							
							// Compute translation vector
							int dx = insert.x - bounds.x;
							int dy = insert.y - bounds.y;
							
							// Translate attributes
							GraphConstants.translate(nested.values(),dx,dy);
							
							// Edit cells
							graph.getGraphLayoutCache().edit(nested,null,null,null);
						}
						
						// Select cells
						graph.setSelectionCells(cells);
						
						// Don't remove at sender
						return false;
					}
			} 
			else
				EditPaste.Paste() ; // This means that the format is Picture.
		}
	} 
	catch (Exception exception) 
	{
		System.err.println("BasicGraphUI : Cannot import: ");
		exception.printStackTrace() ;
	}
	return false;
}

//****************************************************************************

        /** The protected method importDataImpl
         * @param jcomponent This is a JComponent.
         * @param transferable This is a Transferable.
         * @return Returns false. */
        protected boolean importDataImpl(JComponent jcomponent, Transferable transferable)
        {
            return false;
        }

        /** This is the GraphTransferHandler() constructor.
         */
        public GraphTransferHandler()
        {
            inCount = 0;
        }
    }

    /** This class listens for events that are fired when a component is dropped to the graphpad.
     */
    public class GraphDropTargetListener extends BasicGraphDropTargetListener
        implements Serializable
    {

        /** This is the protected method saveComponentState
         * @param jcomponent This is a JComponent. */
        protected void saveComponentState(JComponent jcomponent)
        {
        }

        /** This is the protected method restoreComponentState and restores a mouse drag operation.
         * @param jcomponent This is a JComponent. */
        protected void restoreComponentState(JComponent jcomponent)
        {
            if(handle != null)
            {
                handle.mouseDragged(null);
            }
        }

        /** This is the protected method updateInsertionLocation and modifies a mouse drag operation.
         * @param jcomponent This is the JComponent being worked with.
         * @param point This holds the x and y values for the new MouseEvent */
        protected void updateInsertionLocation(JComponent jcomponent, Point point)
        {
            setInsertionLocation(point);
            if(handle != null)
            {
                byte byte0 = ((byte)(dropAction != 1 ? 0 : 2));
                handle.mouseDragged(new MouseEvent(jcomponent, 0, 0L, byte0, point.x, point.y, 1, false));
            }
        }

        /** This is the method dragEnter.
         * @param droptargetdragevent This parameter is sent to super.dragEnter(DropTargetDragEvent). */
        public void dragEnter(DropTargetDragEvent droptargetdragevent)
        {
            dropAction = droptargetdragevent.getDropAction();
            super.dragEnter(droptargetdragevent);
        }

        /** This is the dropActionChanged method.
         * @param droptargetdragevent This parameter is sent to super.dropActionChanged(DropTargetDragEvent). */
        public void dropActionChanged(DropTargetDragEvent droptargetdragevent)
        {
            dropAction = droptargetdragevent.getDropAction();
            super.dropActionChanged(droptargetdragevent);
        }

        /** This is the public GraphDropTargetListener constructor.
         */
        public GraphDropTargetListener()
        {
        }
    }

    /** The MouseInput handler class handles all the events associated with mouse moves and clicks.
     */
    public class MouseInputHandler
        implements MouseInputListener
    {

        /**
         * The component from which the event originates.
         */
        protected Component source;
        
        /** The component on which the mouse ended.
         */
        protected Component destination;

        /** The mouseClicked method.
         * @param mouseevent This method dispatches the mouse event from the source to the destination. */
        public void mouseClicked(MouseEvent mouseevent)
        {
            if(destination != null)
            {
                destination.dispatchEvent(SwingUtilities.convertMouseEvent(source, mouseevent, destination));
            }
        }

        /** The mousePressed method.
         * NOTE:This method is empty.
         * @param mouseevent This is a mouse event */
        public void mousePressed(MouseEvent mouseevent)
        {
        }

        /** The mouseReleased method is called when the mouse button is released.
         * @param mouseevent This MouseEvent is dispatched from the source to the destination. */
        public void mouseReleased(MouseEvent mouseevent)
        {
            if(destination != null)
            {
                destination.dispatchEvent(SwingUtilities.convertMouseEvent(source, mouseevent, destination));
            }
            removeFromSource();
        }

        /** The mouseEntered method
         * @param mouseevent If this MouseEvent is LeftMouseButton then the object is removed from the source. */
        public void mouseEntered(MouseEvent mouseevent)
        {
            if(!SwingUtilities.isLeftMouseButton(mouseevent))
            {
                removeFromSource();
            }
        }

        /** The mouseExited method
         * @param mouseevent If this MouseEvent is LeftMouseButton then the object is removed from the source.  */
        public void mouseExited(MouseEvent mouseevent)
        {
            if(!SwingUtilities.isLeftMouseButton(mouseevent))
            {
                removeFromSource();
            }
        }

        /** The mouseDragged method is fired when the mouse is dragged.
         * @param mouseevent This MouseEvent is dispatched from the source to the destination */
        public void mouseDragged(MouseEvent mouseevent)
        {
            if(destination != null)
            {
                destination.dispatchEvent(SwingUtilities.convertMouseEvent(source, mouseevent, destination));
            }
        }

        /** The mouseMoved method is fired when the mouse is moved.
         * @param mouseevent The object is removed from the source. */
        public void mouseMoved(MouseEvent mouseevent)
        {
            removeFromSource();
        }

        /** The removeFromSource method. After execution, source = destination = null.
         */
        protected void removeFromSource()
        {
            if(source != null)
            {
                source.removeMouseListener(this);
                source.removeMouseMotionListener(this);
            }
            source = destination = null;
        }

        /** The MouseInputHandler constructor
         * @param component This Component is the source.
         * @param component1 This Component is the destination.
         * @param mouseevent The MouseEvent is dispatched from the source to the destination. */
        public MouseInputHandler(Component component, Component component1, MouseEvent mouseevent)
        {
            source = component;
            destination = component1;
            source.addMouseListener(this);
            source.addMouseMotionListener(this);
            component1.dispatchEvent(SwingUtilities.convertMouseEvent(component, mouseevent, component1));
        }
    }

    /** The GraphSelectAllAction class extends the AbstractAction class and handles all events associated
     * with all the components being selected.
     */
    private class GraphSelectAllAction extends AbstractAction
    {

        /** 
         * Determines whether all the components on the graph were selected.
         */
        private boolean selectAll;

        /** The actionPerformed method.
         * Adds all the selected cells to the list of selectedcells stored by the application.
         * @param actionevent This is the ActionEvent that is performed (selectAll). */
        public void actionPerformed(ActionEvent actionevent)
        {
            if(graph != null)
            {
                if(selectAll)
                {
                    Object aobj[] = new Object[graphModel.getRootCount()];
                    for(int i = 0; i < aobj.length; i++)
                    {
                        aobj[i] = graphModel.getRootAt(i);
                    }

                    graph.addSelectionCells(aobj);
                } else
                {
                    graph.clearSelection();
                }
            }
        }

        /** The isEnabled method
         * @return Returns true if graph is not null and isEnabled() */
        public boolean isEnabled()
        {
            return graph != null && graph.isEnabled();
        }

        /** The GraphSelectAllAction constructor
         * @param s This is a string.
         * @param flag This is a boolean that sets the value of selectAll. True is all components are selected, false otherwise.*/
        public GraphSelectAllAction(String s, boolean flag)
        {
            selectAll = flag;
        }
    }

    /** This class listens for events fired when the user tries to edit a component.
     */
    private class GraphEditAction extends AbstractAction
    {

        /** The actionPerformed method sets the editing to begin at the focused cell if isEnabled() and the focused cell is a graphcell
         * @param actionevent This is an ActionEvent that is performed. */
        public void actionPerformed(ActionEvent actionevent)
        {
            if(isEnabled() && (getFocusedCell() instanceof GraphCell))
            {
                graph.startEditingAtCell(getFocusedCell());
            }
        }

        /** The isEnabled method
         * @return Returns true if graph is not null and isEnabled() */
        public boolean isEnabled()
        {
            return graph != null && graph.isEnabled();
        }

        /** The GraphEditAction constructor
         * @param s This is a string. */
        public GraphEditAction(String s)
        {
        }
    }

    /**
     * The GraphCancelEditingAction class listens for the event that is fired when the user cancels an edit
     * of a component.
     */
    private class GraphCancelEditingAction extends AbstractAction
    {

        /** The actionPerformed method
         * @param actionevent This is the ActionEvent that is performed to cancel the editing. */
        public void actionPerformed(ActionEvent actionevent)
        {
            if(graph != null)
            {
                cancelEditing(graph);
            }
        }

        /** The isEnabled method
         * @return Returns true if graph is not null and isEnabled() */
        public boolean isEnabled()
        {
            return graph != null && graph.isEnabled();
        }

        /** The GraphCancelEditingAction constructor
         * @param s This is a string. */
        public GraphCancelEditingAction(String s)
        {
        }
    }

    /** This class handles event fired when the graph is extended in any direction.i.e. the scrollbars are moved.
     */
    public class GraphIncrementAction extends AbstractAction
    {

        /** 
         * The direction in which the scrollbar was moved on the graph.
         * 1. Graph is moved up
         * 2. Graph is moved right
         * 3. Graph is moved down
         * 4. Graph is moved left.
         */
        protected int direction;

        /** The method is called when the graph is extended by the user. 
         * @param actionevent This is the ActionEvent that is performed (GraphIncrement). */
        public void actionPerformed(ActionEvent actionevent)
        {
            if(graph != null)
            {
                byte byte0 = 70;
                Rectangle rectangle = graph.getVisibleRect();
                if(direction == 1)
                {
                    rectangle.translate(0, -byte0);
                } else
                if(direction == 2)
                {
                    rectangle.translate(byte0, 0);
                } else
                if(direction == 3)
                {
                    rectangle.translate(0, byte0);
                } else
                if(direction == 4)
                {
                    rectangle.translate(-byte0, 0);
                }
                graph.scrollRectToVisible(rectangle);
            }
        }

        /** The isEnabled method
         * @return Returns true if graph is not null and isEnabled() */
        public boolean isEnabled()
        {
            return graph != null && graph.isEnabled();
        }

        /** The GraphIncrementAction constructor
         * @param i This is an integer that sets the direction.
         * @param s This is a String. NOTE: It is never used.
         */
        private GraphIncrementAction(int i, String s)
        {
            direction = i;
        }

    }

    /** The PropertyChangeHandler class handles all events raised when any property of the graph changes.
     */
    public class PropertyChangeHandler
        implements PropertyChangeListener, Serializable
    {

        /** The propertyChange method which is called whenever a property of the graph is changed/
         * @param propertychangeevent This is the PropertyChangeEvent that occurs. 
         */
        public void propertyChange(PropertyChangeEvent propertychangeevent)
        {
            if(propertychangeevent.getSource() == graph)
            {
                String s = propertychangeevent.getPropertyName();
                if(s.equals("minimumSize"))
                {
                    updateCachedPreferredSize();
                } else
                if(s.equals("model"))
                {
                    setModel((GraphModel)propertychangeevent.getNewValue());
                } else
                if(s.equals("view"))
                {
                    setGraphLayoutCache((GraphLayoutCache)propertychangeevent.getNewValue());
                    graph.repaint();
                } else
                if(s.equals("marquee"))
                {
                    setMarquee((BasicMarqueeHandler)propertychangeevent.getNewValue());
                } else
                if(s.equals("editable"))
                {
                    boolean flag = ((Boolean)propertychangeevent.getNewValue()).booleanValue();
                    if(!flag && isEditing(graph))
                    {
                        cancelEditing(graph);
                    }
                } else
                if(s.equals("selectionModel"))
                {
                    setSelectionModel(graph.getSelectionModel());
                } else
                if(s.equals("gridVisible") || s.equals("gridSize") || s.equals("portsVisible") || s.equals("antiAliased"))
                {
                    graph.repaint();
                } else
                if(s.equals("scale"))
                {
                    updateSize();
                } else
                if(s.equals("font"))
                {
                    completeEditing();
                    updateSize();
                } else
                if(s.equals("componentOrientation") && graph != null)
                {
                    graph.graphDidChange();
                }
            }
        }

        /** The PropertyChangeHandler constructor
         *  NOTE: This method is empty
         */
        public PropertyChangeHandler()
        {
        }
    }

    /** The RootHandle class is used to describe the handle for the application.
     */
    public class RootHandle
        implements CellHandle, Serializable
    {

        /** 
         * The change in the x co-ordinates of the mouse positions on a move.
         */
        protected transient int _mouseToViewDelta_x;
        
        /** The change in the y co-ordinates of the mouse positions on a move.
         */
        protected transient int _mouseToViewDelta_y;
        /** 
         * The image for the portion of the application not visible on the screen.
         */
        protected transient Image offscreen;
        /** 
         * A description of the graphics for the portion of the application not visible on the screen.
         */
        protected transient Graphics offgraphics;
        /** 
         * Determines whether this was the first time a drag was performed
         */
        protected transient boolean firstDrag;
        /**
         * An array of all the views in the application.
         */
        protected transient CellView views[];
        
        /** 
         * An array of all the contexts as view, each is a cellview object.
         */
        protected transient CellView contextViews[];
        /** 
         * An array of all the ports in the graphpad.
         */
        protected transient PortView portViews[];
        
        /** A rectangle that stores the bounds of the cell being edited.
         */
        protected transient Rectangle cachedBounds;
        
        /** 
         * An array of the handles of all cells.
         */
        protected transient CellHandle handles[];
        
        /** 
         * The point where the cell being edited starts.
         */
        protected transient Point start;
        
        /** The point where the cell being edited ends.
         */
        protected transient Point last;
        
        /** The point where the cell being snapped starts.
         */
        protected transient Point snapStart;
        
        /** The point where the cell being snapped ends.
         */
        protected transient Point snapLast;
        
        /** 
         * Determines whether the mouse is being moved.
         */
        protected boolean isMoving;
        
        /** 
         * Determines whether a component is being dragged with the mouse.
         */
        protected boolean isDragging;
        
        /** 
         * The handle for the cell that is currently being edited.
         */
        protected transient CellHandle activeHandle;
        
        /** 
         * The current context of the graph.
         */
        protected transient GraphContext context;
        
        /** 
         * Determines whether the current graphContext is visible on the graph.
         */
        protected boolean isContextVisible;
        
        /** 
         * Determines whether the graph has been painted after a change to it.
         */
        protected boolean blockPaint;
        
        /**
         * This is a connection object that describes which components are connected to each other.
         */
        protected transient ConnectionSet disconnect;

        /** The getContext method
         * @return Returns the GraphContext */
        public GraphContext getContext()
        {
            return context;
        }

        /** The paint method paints the graph.
         * @param g These Graphics are painted in all of the handles that are not null. */
        public void paint(Graphics g)
        {
            if(handles != null && handles.length < BasicGraphUI.MAXHANDLES)
            {
                for(int i = 0; i < handles.length; i++)
                {
                    if(handles[i] != null)
                    {
                        handles[i].paint(g);
                    }
                }

            }
            blockPaint = true;
        }

        /** The overlay method updates the views of the cells and repaints the graph.
         * @param g These Graphics are painted on cells or ports. */
        public void overlay(Graphics g)
        {
            if(isDragging && !BasicGraphUI.DNDPREVIEW)
            {
                return;
            }
            if(cachedBounds != null)
            {
                g.setColor(Color.black);
                g.drawRect(cachedBounds.x, cachedBounds.y, cachedBounds.width - 2, cachedBounds.height - 2);
            } else
            {
                Graphics2D graphics2d = (Graphics2D)g;
                java.awt.geom.AffineTransform affinetransform = graphics2d.getTransform();
                graphics2d.scale(graph.getScale(), graph.getScale());
                if(views != null)
                {
                    for(int i = 0; i < views.length; i++)
                    {
                        paintCell(g, views[i], views[i].getBounds(), true);
                    }

                }
                if(contextViews != null && isContextVisible)
                {
                    for(int j = 0; j < contextViews.length; j++)
                    {
                        paintCell(g, contextViews[j], contextViews[j].getBounds(), true);
                    }

                }
                graphics2d.setTransform(affinetransform);
                if(portViews != null && graph.isPortsVisible())
                {
                    paintPorts(g, portViews);
                }
            }
        }

        /** The mouseMoved method updates the handles for the cell when the mouse has been moved.
         * @param mouseevent This MouseEvent may be applied to the handles. */
        public void mouseMoved(MouseEvent mouseevent)
        {
            if(!mouseevent.isConsumed() && handles != null)
            {
                for(int i = handles.length - 1; i >= 0 && !mouseevent.isConsumed(); i--)
                {
                    if(handles[i] != null)
                    {
                        handles[i].mouseMoved(mouseevent);
                    }
                }

            }
        }

        /** The mousePressed method is invoked whenever any button in the mouse is pressed.
         * This modifies the cells based on where the mouse was clicked.
         * @param mouseevent This MouseEvent may be applied to the handles. */
        public void mousePressed(MouseEvent mouseevent)
        {
            if(!mouseevent.isConsumed() && graph.isMoveable())
            {
                if(handles != null)
                {
                    for(int i = handles.length - 1; i >= 0; i--)
                    {
                        if(handles[i] != null)
                        {
                            handles[i].mousePressed(mouseevent);
                            if(mouseevent.isConsumed())
                            {
                                activeHandle = handles[i];
                                return;
                            }
                        }
                    }

                }
                if(views != null)
                {
                    Point point = mouseevent.getPoint();
                    Point point1 = graph.fromScreen(new Point(point));
                    CellView cellview = findViewForPoint(point1);
                    if(cellview != null)
                    {
                        if(snapSelectedView)
                        {
                            Rectangle rectangle = cellview.getBounds();
                            start = graph.toScreen(new Point(rectangle.x, rectangle.y));
                            snapStart = graph.snap(new Point(start));
                            _mouseToViewDelta_x = point.x - start.x;
                            _mouseToViewDelta_y = point.y - start.y;
                        } else
                        {
                            snapStart = graph.snap(new Point(point));
                            _mouseToViewDelta_x = snapStart.x - point.x;
                            _mouseToViewDelta_y = snapStart.y - point.y;
                            start = new Point(snapStart);
                        }
                        last = new Point(start);
                        snapLast = new Point(snapStart);
                        isContextVisible = contextViews != null && contextViews.length < BasicGraphUI.MAXCELLS && (!mouseevent.isControlDown() || !graph.isCloneable());
                        mouseevent.consume();
                    }
                }
            }
        }

        /** The findViewForPoint method
         * @param point This is the Point that is looking for a CellView.
         * @return Returns a CellView for the Point or null */
        protected CellView findViewForPoint(Point point)
        {
            int i = graph.getTolerance();
            Rectangle rectangle = new Rectangle(point.x - i, point.y - i, 2 * i, 2 * i);
            for(int j = 0; j < views.length; j++)
            {
                if(views[j].intersects(graph.getGraphics(), rectangle))
                {
                    return views[j];
                }
            }

            return null;
        }

        /** The startDragging method is fired when components are dragged along the graph. 
         * It dragging is not enabled for the graph, it does nothing.
         * @param mouseevent This is the start dragging mouse event */
        protected void startDragging(MouseEvent mouseevent)
        {
            isDragging = true;
            if(graph.isDragEnabled())
            {
                byte byte0 = ((byte)(!mouseevent.isControlDown() || !graph.isCloneable() ? 2 : 1));
                TransferHandler transferhandler = graph.getTransferHandler();
                try
                {
                    transferhandler.exportAsDrag(graph, mouseevent, byte0);
                }
                catch(Exception exception) { }
            }
        }

        /** The method initOffscreen
         */
        protected void initOffscreen()
        {
            try
            {
                Rectangle rectangle = graph.getBounds();
                offscreen = new BufferedImage(rectangle.width, rectangle.height, 1);
                offgraphics = offscreen.getGraphics();
                offgraphics.setClip(0, 0, (int)rectangle.getWidth(), (int)rectangle.getHeight());
                offgraphics.setColor(graph.getBackground());
                offgraphics.fillRect(0, 0, (int)rectangle.getWidth(), (int)rectangle.getHeight());
                graph.getUI().paint(offgraphics, graph);
            }
            catch(Error error)
            {
                offscreen = null;
                offgraphics = null;
            }
        }

        /** The mouseDragged method
         * @param mouseevent The mouse is dragged */
        public void mouseDragged(MouseEvent mouseevent)
        {
            boolean flag = isConstrainedMoveEvent(mouseevent);
            Rectangle rectangle = null;
            if(firstDrag && graph.isDoubleBuffered() && cachedBounds == null)
            {
                initOffscreen();
                firstDrag = false;
            }
            if(mouseevent != null && !mouseevent.isConsumed())
            {
                if(activeHandle != null)
                {
                    activeHandle.mouseDragged(mouseevent);
                } else
                if(start != null)
                {
                    Graphics g = offgraphics == null ? graph.getGraphics() : offgraphics;
                    Point point = new Point(mouseevent.getPoint());
                    point.translate(-_mouseToViewDelta_x, -_mouseToViewDelta_y);
                    Point point1 = graph.snap(point);
                    Point point2 = point1;
                    int i = graph.getMinimumMove();
                    int j = point2.x - start.x;
                    int l = point2.y - start.y;
                    if(isMoving || Math.abs(j) > i || Math.abs(l) > i)
                    {
                        boolean flag1 = false;
                        isMoving = true;
                        if(disconnect == null && graph.isDisconnectOnMove())
                        {
                            disconnect = context.disconnect(graphLayoutCache.getAllDescendants(views));
                        }
                        int k;
                        int i1;
                        if(flag && cachedBounds == null)
                        {
                            int j1 = point2.x - start.x;
                            int k1 = point2.y - start.y;
                            if(Math.abs(j1) < Math.abs(k1))
                            {
                                k = 0;
                                i1 = k1;
                            } else
                            {
                                k = j1;
                                i1 = 0;
                            }
                        } else
                        {
                            k = point2.x - last.x;
                            i1 = point2.y - last.y;
                        }
                        double d = graph.getScale();
                        k = (int)((double)k / d);
                        i1 = (int)((double)i1 / d);
                        g.setColor(graph.getForeground());
                        g.setXORMode(graph.getBackground());
                        if(graph.isDragEnabled() && !isDragging)
                        {
                            startDragging(mouseevent);
                        }
                        if(k != 0 || i1 != 0)
                        {
                            if(!snapLast.equals(snapStart) && (offscreen != null || !blockPaint))
                            {
                                overlay(g);
                                flag1 = true;
                            }
                            isContextVisible = !mouseevent.isControlDown() || !graph.isCloneable();
                            blockPaint = false;
                            if(offscreen != null)
                            {
                                rectangle = graph.toScreen(AbstractCellView.getBounds(views));
                                Rectangle rectangle1 = graph.toScreen(AbstractCellView.getBounds(contextViews));
                                if(rectangle1 != null)
                                {
                                    rectangle.add(rectangle1);
                                }
                            }
                            if(flag && cachedBounds == null)
                            {
                                CellView acellview[] = graphLayoutCache.getAllDescendants(views);
                                for(int l1 = 0; l1 < acellview.length; l1++)
                                {
                                    CellView cellview = graphLayoutCache.getMapping(acellview[l1].getCell(), false);
                                    Map map = cellview.getAllAttributes();
                                    acellview[l1].setAttributes(GraphConstants.cloneMap(map));
                                    acellview[l1].refresh(false);
                                }

                            }
                            if(cachedBounds != null)
                            {
                                cachedBounds.translate((int)((double)k * d), (int)((double)i1 * d));
                            } else
                            {
                                GraphLayoutCache.translateViews(views, k, i1);
                                if(views != null)
                                {
                                    graphLayoutCache.update(views);
                                }
                                if(contextViews != null)
                                {
                                    graphLayoutCache.update(contextViews);
                                }
                            }
                            if(!point1.equals(snapStart))
                            {
                                overlay(g);
                                flag1 = true;
                            }
                            if(flag)
                            {
                                last = new Point(start);
                            }
                            last.translate((int)((double)k * d), (int)((double)i1 * d));
                            snapLast = point1;
                            if(flag1 && offscreen != null)
                            {
                                rectangle.add(graph.toScreen(AbstractCellView.getBounds(views)));
                                Rectangle rectangle2 = graph.toScreen(AbstractCellView.getBounds(contextViews));
                                if(rectangle2 != null)
                                {
                                    rectangle.add(rectangle2);
                                }
                                rectangle.grow(2, 2);
                                int i2 = Math.max(0, rectangle.x);
                                int j2 = Math.max(0, rectangle.y);
                                int k2 = i2 + rectangle.width;
                                int l2 = j2 + rectangle.height;
                                if(isDragging)
                                {
                                    return;
                                }
                                graph.getGraphics().drawImage(offscreen, i2, j2, k2, l2, i2, j2, k2, l2, graph);
                            }
                        }
                    }
                }
            } else
            if(mouseevent == null)
            {
                graph.repaint();
            }
        }

        /** The mouseReleased method is fired when a mouse button is released.
         * @param mouseevent The mouse is released. */
        public void mouseReleased(MouseEvent mouseevent)
        {
label0:
            {
                try
                {
                    if(mouseevent != null && !mouseevent.isConsumed())
                    {
                        if(activeHandle != null)
                        {
                            activeHandle.mouseReleased(mouseevent);
                            activeHandle = null;
                        } else
                        if(isMoving && !mouseevent.getPoint().equals(start))
                        {
                            if(cachedBounds != null)
                            {
                                int i = mouseevent.getX() - start.x;
                                int j = mouseevent.getY() - start.y;
                                Point point = graph.fromScreen(new Point(i, j));
                                GraphLayoutCache.translateViews(views, point.x, point.y);
                            }
                            CellView acellview[] = graphLayoutCache.getAllDescendants(views);
                            if(mouseevent.isControlDown() && graph.isCloneable())
                            {
                                Object aobj[] = graph.getDescendants(context.getCells());
                                ConnectionSet connectionset = ConnectionSet.create(graphModel, aobj, false);
                                connectionset.addConnections(acellview);
                                Map map1 = GraphConstants.createAttributes(acellview, null);
                                insertCells(context.getCells(), map1, connectionset, true, 0, 0);
                            } else
                            if(graph.isMoveable())
                            {
                                Map map = GraphConstants.createAttributes(acellview, null);
                                graph.getGraphLayoutCache().edit(map, disconnect, null, null);
                            }
                            mouseevent.consume();
                        }
                    }
                }
                catch(Exception exception)
                {
                    isDragging = false;
                    disconnect = null;
                    offscreen = null;
                    firstDrag = true;
                    start = null;
                    break label0;
                }
                finally
                {
                    isDragging = false;
                    disconnect = null;
                    offscreen = null;
                    firstDrag = true;
                    start = null;
                    //throw exception1;
                }
                isDragging = false;
                disconnect = null;
                offscreen = null;
                firstDrag = true;
                start = null;
                break label0;
            }
        }

    /**
     * Creates a root handle which contains handles for the given
     * cells. The root handle and all its children point to the
     * specified JGraph instance. The root handle is responsible
     * for dragging the selection.
         * @param graphcontext This is the GraphContext in which work is being done. */
        public RootHandle(GraphContext graphcontext)
        {
            _mouseToViewDelta_x = 0;
            _mouseToViewDelta_y = 0;
            firstDrag = true;
            start = null;
            isMoving = false;
            isDragging = false;
            activeHandle = null;
            isContextVisible = true;
            blockPaint = false;
            disconnect = null;
            context = graphcontext;
            if(!graphcontext.isEmpty())
            {
                views = graphcontext.createTemporaryCellViews();
                if(graphcontext.getDescendantCount() < BasicGraphUI.MAXCELLS)
                {
                    contextViews = graphcontext.createTemporaryContextViews();
                } else
                {
                    cachedBounds = graph.toScreen(graph.getCellBounds(graphcontext.getCells()));
                }
                Object aobj[] = graphcontext.getCells();
                if(aobj.length < BasicGraphUI.MAXHANDLES)
                {
                    handles = new CellHandle[views.length];
                    for(int i = 0; i < views.length; i++)
                    {
                        handles[i] = views[i].getHandle(graphcontext);
                    }

                    portViews = graphcontext.createTemporaryPortViews();
                }
            }
        }
    }

  /**
   * TreeMouseListener is responsible for updating the selection
   * based on mouse events.
   */
    public class MouseHandler extends MouseAdapter
        implements MouseMotionListener, Serializable
    {

        /** The cell under the mousepointer
         */
        protected CellView cell;
        
        /** The object than handles mouse operations
         */
        protected Object handler;
        
        /** The previous cursor
         */
        protected transient Cursor previousCursor;

        /** Invoked when a mouse button is pressed
         * @param mouseevent mousePressed */
        public void mousePressed(MouseEvent mouseevent)
        {
            handler = null;
            if(!mouseevent.isConsumed() && graph.isEnabled())
            {
                graph.requestFocus();
                int i = graph.getTolerance();
                Rectangle rectangle = graph.fromScreen(new Rectangle(mouseevent.getX() - i, mouseevent.getY() - i, 2 * i, 2 * i));
                Point point = graph.fromScreen(new Point(mouseevent.getPoint()));
                focus = focus == null || !focus.intersects(graph.getGraphics(), rectangle) ? null : focus;
                cell = graph.getNextViewAt(focus, point.x, point.y);
                if(focus == null)
                {
                    focus = cell;
                }
                completeEditing();
                if(!isForceMarqueeEvent(mouseevent))
                {
                    if(mouseevent.getClickCount() == graph.getEditClickCount() && focus != null && focus.isLeaf() && focus.getParentView() == null)
                    {
                        handleEditTrigger(focus.getCell());
                        mouseevent.consume();
                        cell = null;
                    } else
                    if(!isToggleSelectionEvent(mouseevent))
                    {
                        if(handle != null)
                        {
                            handle.mousePressed(mouseevent);
                            handler = handle;
                        }
                        if(!mouseevent.isConsumed() && cell != null && !graph.isCellSelected(cell))
                        {
                            selectCellForEvent(cell.getCell(), mouseevent);
                            focus = cell;
                            if(handle != null)
                            {
                                handle.mousePressed(mouseevent);
                                handler = handle;
                            }
                            cell = null;
                        }
                    }
                }
                if(!mouseevent.isConsumed() && (!isToggleSelectionEvent(mouseevent) || focus == null) && marquee != null)
                {
                    marquee.mousePressed(mouseevent);
                    handler = marquee;
                }
            }
        }

        /** This handles the edit functionality. It start the editing of the specified cell.
         * @param obj start editing at this cell */
        protected void handleEditTrigger(Object obj)
        {
            graph.startEditingAtCell(obj);
        }

        /** This is invoked when the mouse is dragged. It updates the scroll region of the graph.
         * @param mouseevent mouseDragged */
        public void mouseDragged(MouseEvent mouseevent)
        {
            try
            {
                BasicGraphUI.autoscroll(graph, mouseevent.getPoint());
                if(handler == marquee)
                {
                    marquee.mouseDragged(mouseevent);
                } else
                if(handler == null && !isEditing(graph) && focus != null)
                {
                    if(!graph.isCellSelected(focus.getCell()))
                    {
                        selectCellForEvent(focus.getCell(), mouseevent);
                        cell = null;
                    }
                    if(handle != null)
                    {
                        handle.mousePressed(mouseevent);
                    }
                    handler = handle;
                }
                if(handle != null && handler == handle)
                {
                    handle.mouseDragged(mouseevent);
                }
            }
            finally { }
        }

        /** This is invoked when the mouse is moved. It resets the cursor position.
         * @param mouseevent mouseMoved */
        public void mouseMoved(MouseEvent mouseevent)
        {
            if(previousCursor == null)
            {
                previousCursor = graph.getCursor();
            }
            if(graph != null && graph.isEnabled())
            {
                if(marquee != null)
                {
                    marquee.mouseMoved(mouseevent);
                }
                if(handle != null)
                {
                    handle.mouseMoved(mouseevent);
                }
                if(!mouseevent.isConsumed() && previousCursor != null)
                {
                    graph.setCursor(previousCursor);
                    previousCursor = null;
                }
            }
        }

        /** This is invoked when the mouse button is released. Its calls the appropriate event handlers
         * based on what was clicked by the mouse.
         * @param mouseevent mouseReleased */
        public void mouseReleased(MouseEvent mouseevent)
        {
            try
            {
                if(mouseevent != null && !mouseevent.isConsumed())
                {
                    if(handler == marquee)
                    {
                        marquee.mouseReleased(mouseevent);
                    } else
                    if(handler == handle && handle != null)
                    {
                        handle.mouseReleased(mouseevent);
                    }
                    if(isDescendant(cell, focus) && mouseevent.getModifiers() != 0)
                    {
                        cell = focus;
                    }
                    if(!mouseevent.isConsumed() && cell != null)
                    {
                        Object obj = cell.getCell();
                        boolean flag = graph.isCellSelected(obj);
                        selectCellForEvent(obj, mouseevent);
                        focus = cell;
                        if(flag && graph.isCellSelected(obj))
                        {
                            javax.swing.tree.TreeNode treenode = ((DefaultMutableTreeNode)obj).getRoot();
                            selectCellForEvent(treenode, mouseevent);
                            focus = graphLayoutCache.getMapping(treenode, false);
                        }
                    }
                }
            }
            finally
            {
                handler = null;
                cell = null;
            }
        }

        /** Determines whether a cell is a descendant of another.
         * 
         * @param cellview this is the parent view
         * @param cellview1 this is the child view
         * @return Returns false if parent or child is null */
        protected boolean isDescendant(CellView cellview, CellView cellview1)
        {
            if(cellview == null || cellview1 == null)
            {
                return false;
            }
            Object obj = cellview.getCell();
            Object obj1 = cellview1.getCell();
            Object obj2 = obj1;
            do
            {
                if(obj2 == obj)
                {
                    return true;
                }
            } while((obj2 = graphModel.getParent(obj2)) != null);
            return false;
        }

        /** The MouseHandler constructor. Sets the perviousCursor to null.
         */
        public MouseHandler()
        {
            previousCursor = null;
        }
    }

  /**
   * This is used to get mutliple key down events to appropriately generate
   * events.
   */
    public class KeyHandler extends KeyAdapter
        implements Serializable
    {

        /** Key code that is being generated for.
         */
        protected Action repeatKeyAction;
        
        /** Set to true while keyPressed is active.
         */
        protected boolean isKeyDown;

        /** The keyPressed method. Calls the appropriate handlers based on what was clicked by the button.
         * @param keyevent The event of the key being pressed */
        public void keyPressed(KeyEvent keyevent)
        {
            if(graph != null && graph.hasFocus() && graph.isEnabled())
            {
                KeyStroke keystroke = KeyStroke.getKeyStroke(keyevent.getKeyCode(), keyevent.getModifiers());
                if(graph.getConditionForKeyStroke(keystroke) == 0)
                {
                    ActionListener actionlistener = graph.getActionForKeyStroke(keystroke);
                    if(actionlistener instanceof Action)
                    {
                        repeatKeyAction = (Action)actionlistener;
                    } else
                    {
                        repeatKeyAction = null;
                    }
                } else
                {
                    repeatKeyAction = null;
                    if(keystroke.getKeyCode() == 27)
                    {
                        if(marquee != null)
                        {
                            marquee.mouseReleased(null);
                        }
                        if(mouseListener != null)
                        {
                            mouseListener.mouseReleased(null);
                        }
                        updateHandle();
                        graph.repaint();
                    }
                }
                if(isKeyDown && repeatKeyAction != null)
                {
                    repeatKeyAction.actionPerformed(new ActionEvent(graph, 1001, ""));
                    keyevent.consume();
                } else
                {
                    isKeyDown = true;
                }
            }
        }

        /** The keyReleased method. Sets the isKeyDown to false.
         * @param keyevent The event of the key being released */
        public void keyReleased(KeyEvent keyevent)
        {
            isKeyDown = false;
        }

        /** The KeyHandler constructor
         */
        public KeyHandler()
        {
        }
    }

  /**
   * Listener responsible for getting cell editing events and updating
   * the graph accordingly.
   */
    public class CellEditorHandler
        implements CellEditorListener, Serializable
    {

        /** The editingStopped method. Calls the completeEditing method.
         * @param changeevent This is the event where editing is stopped. */
        public void editingStopped(ChangeEvent changeevent)
        {
            completeEditing(false, false, true);
        }

        /** The editingCanceled method. Calls the completEditing method but indicates that the event
         * was not completed.
         * @param changeevent This is the event where editing is cancelled */
        public void editingCanceled(ChangeEvent changeevent)
        {
            completeEditing(false, false, false);
        }

        /** The CellEditorHandler constructor.
         * NOTE: The method is empty.
         */
        public CellEditorHandler()
        {
        }
    }

  /**
   * Listens for changes in the selection model and updates the display
   * accordingly.
   */
    public class GraphSelectionHandler
        implements GraphSelectionListener, Serializable
    {

        /** The valueChanged method. It updates the handle for the graph and repaints the graph.
         * @param graphselectionevent This is the event that occurs when the value changes */
        public void valueChanged(GraphSelectionEvent graphselectionevent)
        {
            updateHandle();
            graph.repaint();
        }

        /** The GraphSelectionHandler constructor
         * NOTE: This method is empty.
         */
        public GraphSelectionHandler()
        {
        }
    }

  /**
   * Listens for changes in the graph view and updates the size
   * accordingly.
   */
    public class GraphViewObserver
        implements Observer, Serializable
    {

        /** The update method runs when the graph view size needs to be modified
         * @param observable this is the Observable being monitored
         * @param obj this is the object being modified */
        public void update(Observable observable, Object obj)
        {
            updateSize();
        }

        /** The GraphViewObserver constructor
         * NOTE: This method is empty.
         */
        public GraphViewObserver()
        {
        }
    }

  /**
   * Listens for changes in the graph model and updates the view
   * accordingly.
   */
    public class GraphModelHandler
        implements GraphModelListener, Serializable
    {

        /** The graphChanged method. It updates the graphmodel object based on the changes made and repaints
         * the graphpad.
         * @param graphmodelevent this is the event where the model was changed */
        public void graphChanged(GraphModelEvent graphmodelevent)
        {
            if(graphLayoutCache != null)
            {
                graphLayoutCache.graphChanged(graphmodelevent.getChange());
            }
            Object aobj[] = graphmodelevent.getChange().getInserted();
            Object aobj1[] = graphmodelevent.getChange().getRemoved();
            Object aobj2[] = graphmodelevent.getChange().getChanged();
            if(aobj != null && aobj.length > 0)
            {
                focus = graphLayoutCache.getMapping(aobj[0], false);
            }
            if(aobj1 != null && aobj1.length > 0)
            {
                for(int i = 0; i < aobj1.length && focus != null; i++)
                {
                    if(aobj1[i] == focus.getCell())
                    {
                        focus = null;
                    }
                }

                graph.getSelectionModel().removeSelectionCells(aobj1);
            }
            updateSize();
            if(aobj != null && aobj.length > 0 && graph.isSelectNewCells())
            {
                ArrayList arraylist = new ArrayList();
                for(int j = 0; j < aobj.length; j++)
                {
                    if(graphModel.getParent(aobj[j]) == null && graphLayoutCache.isVisible(aobj[j]))
                    {
                        arraylist.add(aobj[j]);
                    }
                }

                graph.setSelectionCells(arraylist.toArray());
                graph.requestFocus();
            }
        }

        /** The GraphModelHandler constructor
         * NOTE:This method is empty.
         */
        public GraphModelHandler()
        {
        }
    }

  /**
   * Updates the preferred size when scrolling (if necessary).
   */
    public class ComponentHandler extends ComponentAdapter
        implements ActionListener
    {

        /** Timer used when inside a scrollplane and the scrollbar is adjusting
         */
        protected Timer timer;
        
        /** ScrollBar that is being adjusted
         */
        protected JScrollBar scrollBar;

        /** The componentMoved method. This event is fired when a component is moved.
         * @param componentevent This is the event in which a scrollbar is adjusted */
        public void componentMoved(ComponentEvent componentevent)
        {
            if(timer == null)
            {
                JScrollPane jscrollpane = getScrollPane();
                if(jscrollpane == null)
                {
                    updateSize();
                } else
                {
                    scrollBar = jscrollpane.getVerticalScrollBar();
                    if(scrollBar == null || !scrollBar.getValueIsAdjusting())
                    {
                        if((scrollBar = jscrollpane.getHorizontalScrollBar()) != null && scrollBar.getValueIsAdjusting())
                        {
                            startTimer();
                        } else
                        {
                            updateSize();
                        }
                    } else
                    {
                        startTimer();
                    }
                }
            }
        }

        /** This is the method that starts the timer.
         */
        protected void startTimer()
        {
            if(timer == null)
            {
                timer = new Timer(200, this);
                timer.setRepeats(true);
            }
            timer.start();
        }

        /** The getScrollPane method
         * @return Returns the scrollpane that was requested. */
        protected JScrollPane getScrollPane()
        {
            java.awt.Container container;
            for(container = graph.getParent(); container != null && !(container instanceof JScrollPane); container = container.getParent()) { }
            if(container instanceof JScrollPane)
            {
                return (JScrollPane)container;
            } else
            {
                return null;
            }
        }

        /** The actionPerformed method. It stops the 
         * @param actionevent The action event in which a scrollbar is adjusted */
        public void actionPerformed(ActionEvent actionevent)
        {
            if(scrollBar == null || !scrollBar.getValueIsAdjusting())
            {
                if(timer != null)
                {
                    timer.stop();
                }
                updateSize();
                timer = null;
                scrollBar = null;
            }
        }

        /** The ComponentHandler constructor
         */
        public ComponentHandler()
        {
        }
    }


    /** Controls live-preview in dragEnabled mode. This is used to disable
   * live-preview in dragEnabled mode on Java 1.4.1 to workaround a bug that
   * cause the VM to hang during concurrent DnD and repaints. */
    public static final boolean DNDPREVIEW = System.getProperty("java.version").compareTo("1.4.0") < 0;
    
  /** Border in pixels to scroll if marquee or dragging are active. */
    public static int SCROLLBORDER = 18;
  /** Multiplicator for width and height when autoscrolling (=stepsize). */
    public static float SCROLLSTEP = 0.05F;

  /** The maximum number of cells to paint when dragging. */
    public static int MAXCELLS = 20;
  /** The maximum number of handles to paint individually. */
    public static int MAXHANDLES = 20;
    /** minimum preffered size
     */
    protected Dimension preferredMinSize;
    /** component that is going to be drawed in
     */
    protected JGraph graph;
    /** reference to the graph's view (geometric pattern)
     */
    protected GraphLayoutCache graphLayoutCache;
    /** current editor for the graph
     */
    protected GraphCellEditor cellEditor;
  /** Set to false when editing and shouldSelectCell() returns true meaning
    * the node should be selected before editing, used in completeEditing. */
    protected boolean stopEditingInCompleteEditing;
    /** used to paint the CellRenderer
     */
    protected CellRendererPane rendererPane;
    /** size needed to completely display all the cells
     */
    protected Dimension preferredSize;
    /** is the preferred size valid?
     */
    protected boolean validCachedPreferredSize;
    /** used to determine what to display
     */
    protected GraphModel graphModel;
    /** model maintaining the selection
     */
    protected GraphSelectionModel graphSelectionModel;
    /** Default handle that we are going to use
     */
    protected CellHandle handle;
    /** MaruqueeHandler that we are going to use
     */
    protected BasicMarqueeHandler marquee;

  // Following 4 ivars are only valid when editing.

  /** When editing, this will be the Component that is doing the actual
    * editing. */
    protected Component editingComponent;
    /** The focused cell under the mousepointer
     */
    protected CellView focus;
    /** The cell that is being edited
     */
    protected Object editingCell;
    /** Set to true if the editor has a different size than the renderer
     */
    protected boolean editorHasDifferentSize;
    /** Needed to exchange information between Transfer- and MouseListener
     */
    protected Point insertionLocation;
    /** Needed to exchange information between DropTargetHandler and TransferHandler
     */
    protected int dropAction;

  /**
   * If true, the view under mousepointer will be snapped to the grid lines during
   * a drag operation. If snap-to-grid mode is disabled, views are moved by a snap
   * increment.
   */
    protected boolean snapSelectedView;
    /** Listens for JGraph property canges and updates display.
     */
    protected PropertyChangeListener propertyChangeListener;
    /** Listens for mouse events
     */
    protected MouseListener mouseListener;
    /** Listens for key events
     */
    protected KeyListener keyListener;
    /** Listens for component events
     */
    protected ComponentListener componentListener;
    /** Listens for cell editor events
     */
    protected CellEditorListener cellEditorListener;
    /** Listens for graph selection and updates the display when selection changes
     */
    protected GraphSelectionListener graphSelectionListener;
    /** Is responsible for updating the view based on model events
     */
    protected GraphModelListener graphModelListener;

    /** Updates the display when the view has changed
     */
    protected Observer graphViewObserver;

    /** The default transfer handler. event is fired whenever there is a transfer in the handle.
     */
    protected TransferHandler defaultTransferHandler;
    /** The default drop targer listener. This is called whenever there is a drop on the graph.
     */
    protected GraphDropTargetListener defaultDropTargetListener;

    /**
     * Creates the UI for the application by calling the BasicGraphUI constructor.
     *
     * @param jcomponent The component being created
     * @return Returns a new BasicGraphUI */
    public static ComponentUI createUI(JComponent jcomponent)
    {
        return new BasicGraphUI();
    }

    /** The BasicGraphUI
     */
    public BasicGraphUI()
    {
        dropAction = 0;
        snapSelectedView = false;
        cellEditorListener = createCellEditorListener();
    }

  //
  // Methods for configuring the behavior of the graph. None of them
  // push the value to the JGraph instance. You should really only
  // call these methods on the JGraph instance.
  //

  /**
   * Sets the GraphModel. This invokes <code>updateSize</code>.
     * @param graphmodel this is the model being set */
    protected void setModel(GraphModel graphmodel)
    {
        cancelEditing(graph);
        if(graphModel != null && graphModelListener != null)
        {
            graphModel.removeGraphModelListener(graphModelListener);
        }
        graphModel = graphmodel;
        if(graphModel != null && graphModelListener != null)
        {
            graphModel.addGraphModelListener(graphModelListener);
        }
        updateSize();
    }

    /**
     * Sets the setGraphLayoutCache for the graph.
     *
     * @param graphlayoutcache  Sets the GraphLayoutCache (geometric pattern).*/
    protected void setGraphLayoutCache(GraphLayoutCache graphlayoutcache)
    {
        cancelEditing(graph);
        if(graphLayoutCache != null && graphViewObserver != null)
        {
            graphLayoutCache.deleteObserver(graphViewObserver);
        }
        graphLayoutCache = graphlayoutcache;
        if(graphLayoutCache != null && graphViewObserver != null)
        {
            graphLayoutCache.addObserver(graphViewObserver);
        }
        updateSize();
    }

    /**
     * Sets the marquee for the graph.
     *
     * @param basicmarqueehandler sets the marqueeHandler */
    protected void setMarquee(BasicMarqueeHandler basicmarqueehandler)
    {
        marquee = basicmarqueehandler;
    }

    /**
     * Sets the selectionmodel for the graph.
     * @param graphselectionmodel resets the selection model */
    protected void setSelectionModel(GraphSelectionModel graphselectionmodel)
    {
        cancelEditing(graph);
        if(graphSelectionListener != null && graphSelectionModel != null)
        {
            graphSelectionModel.removeGraphSelectionListener(graphSelectionListener);
        }
        graphSelectionModel = graphselectionmodel;
        if(graphSelectionModel != null && graphSelectionListener != null)
        {
            graphSelectionModel.addGraphSelectionListener(graphSelectionListener);
        }
        if(graph != null)
        {
            graph.repaint();
        }
    }

  //
  // GraphUI methods
  //

  /**
   * Retunrs the handle for the graph.
   *
   *
   * @param jgraph the graph that is being worked on
   * @return Returns the CellHandle that is currently active or null */
  public CellHandle getHandle(JGraph jgraph)
  {
    return handle;
  }

  /**
   * Gets the cell which has focus now.
   *
   * @return Returns the cell that has the focus */
  protected Object getFocusedCell()
  {
    if(focus != null)
    {
      return focus.getCell();
    } else
    {
      return null;
    }
  }

  /**
   * Gets the preferred size for the cells.
   *
   * @param jgraph The current jgraph object.
   * @param cellview The current selected cell.
   * @return Returns the preferred size */
  public Dimension getPreferredSize(JGraph jgraph, CellView cellview)
    {
      if(cellview != null)
      {
            Object obj = cellview.getCell();
            String s = jgraph.convertValueToString(obj);
            boolean flag = s != null && s.length() > 0;
            boolean flag1 = GraphConstants.getIcon(cellview.getAllAttributes()) != null;
            if(flag || flag1)
            {
                boolean flag2 = getFocusedCell() == obj && jgraph.hasFocus();
                Component component = cellview.getRendererComponent(jgraph, flag2, false, false);
                if(component != null)
                {
                    jgraph.add(component);
                    component.validate();
                    return component.getPreferredSize();
                }
            }
            return cellview.getBounds().getSize();
        } else
        {
            return null;
        }
    }

    /**
     * Gets the insertion location.
     *
     * @return Returns the InsertionLocation point */
    public Point getInsertionLocation()
    {
        return insertionLocation;
    }

    /**
     * Sets the location at which points are to be inserted.
     *
     * @param point The point to set as the insertion location */
    public void setInsertionLocation(Point point)
    {
        insertionLocation = point;
    }

    /**
     * Determines the cells that are selected under this mouse event and selects it.
     *
     * @param jgraph The current graph.
     * @param aobj the cells being worked on
     * @param mouseevent select cells for this event */
    public void selectCellsForEvent(JGraph jgraph, Object aobj[], MouseEvent mouseevent)
    {
        selectCellsForEvent(aobj, mouseevent);
    }

    /**
     * Determines the cells that are selected under this mouse event and selects it.
     *
     * @param aobj the cells being worked on
     * @param mouseevent The MouseEvent
     */
    public void selectCellsForEvent(Object aobj[], MouseEvent mouseevent)
    {
        if(aobj == null)
        {
            return;
        }
        if(isToggleSelectionEvent(mouseevent))
        {
            for(int i = 0; i < aobj.length; i++)
            {
                toggleSelectionCellForEvent(aobj[i], mouseevent);
            }

        } else
        if(isAddToSelectionEvent(mouseevent))
        {
            graph.addSelectionCells(aobj);
        } else
        {
            graph.setSelectionCells(aobj);
        }
    }

    /**
     * Determines the cell that is selected under this mouse event and selects it.
     *
     * @param obj The cell being worked on
     * @param mouseevent The mouse event */
    public void selectCellForEvent(Object obj, MouseEvent mouseevent)
    {
        if(isToggleSelectionEvent(mouseevent))
        {
            toggleSelectionCellForEvent(obj, mouseevent);
        } else
        if(isAddToSelectionEvent(mouseevent))
        {
            graph.addSelectionCell(obj);
        } else
        {
            graph.setSelectionCell(obj);
        }
    }

    /**
     * Removes the specified cell from the list of selected cells, i.e. they are being toggled
     *
     * @param obj the cells that are being toggled
     * @param mouseevent The toggleSelection mouse event NOTE : THIS IS NEVER USED*/
    protected void toggleSelectionCellForEvent(Object obj, MouseEvent mouseevent)
    {
        if(graph.isCellSelected(obj))
        {
            graph.removeSelectionCell(obj);
        } else
        {
            graph.addSelectionCell(obj);
        }
    }

    /** Determines whether the shift key is pressed.
     *
     * @param mouseevent this is a selection mouse event
     * @return Return true if shift is being held down */
    public boolean isAddToSelectionEvent(MouseEvent mouseevent)
    {
        return mouseevent.isShiftDown();
    }

    /**
     * Determines whether control key is being held down.
     *
     * @param mouseevent this is a selection mouse event
     * @return Return true if control is being held down */
    public boolean isToggleSelectionEvent(MouseEvent mouseevent)
    {
        return mouseevent.isControlDown();
    }

    /**
     * Determines whether the specified is a marqueeevent.
     *
     * @param mouseevent this is a mouse event
     * @return Return true is the specified is a marquee event, false otherwise.
     */
    public boolean isForceMarqueeEvent(MouseEvent mouseevent)
    {
        if(marquee != null)
        {
            return marquee.isForceMarqueeEvent(mouseevent);
        } else
        {
            return false;
        }
    }

    /**
     * Determines whether the mouse is constrained.
     *
     * @param mouseevent this is a moving mouse event
     * @return Return true if shift is down */
    public boolean isConstrainedMoveEvent(MouseEvent mouseevent)
    {
        return mouseevent.isShiftDown();
    }

    /**
     * Determines whether a cell is being edited.
     *
     * @param jgraph this is the jgraph being edited
     * @return Return true if editingComponent is not null */
    public boolean isEditing(JGraph jgraph)
    {
        return editingComponent != null;
    }

    /**
     * Stops editing the cells if it is time to stop and returns true. Otherwise it returns false.
     *
     * @param jgraph this is the jgraph being edited
     * @return Return a true or false */
    public boolean stopEditing(JGraph jgraph)
    {
        if(editingComponent != null && cellEditor.stopCellEditing())
        {
            completeEditing(false, false, true);
            return true;
        } else
        {
            return false;
        }
    }

    /** This method cancels editing of cells.
     * @param jgraph this is the jgraph being edited */
    public void cancelEditing(JGraph jgraph)
    {
        if(editingComponent != null)
        {
            completeEditing(false, true, false);
        }
    }

    /** Specifies the cell at which editing should start.
     * @param jgraph this is the jgraph being edited
     * @param obj start editing at this cell */
    public void startEditingAtCell(JGraph jgraph, Object obj)
    {
        jgraph.scrollCellToVisible(obj);
        if(obj != null)
        {
            startEditing(obj, null);
        }
    }

    /** Returns the cell that is being edited.
     * @param jgraph this is the jgraph being edited
     * @return Return the cell that is being edited */
    public Object getEditingCell(JGraph jgraph)
    {
        return editingCell;
    }

    /** The installUI method
     * @param jcomponent this is the JComponent that is going to have stuff installed to it */
    public void installUI(JComponent jcomponent)
    {
        if(jcomponent == null)
        {
            throw new NullPointerException("null component passed to BasicGraphUI.installUI()");
        } else
        {
            graph = (JGraph)jcomponent;
            marquee = graph.getMarqueeHandler();
            prepareForUIInstall();
            installDefaults();
            installListeners();
            installKeyboardActions();
            installComponents();
            completeUIInstall();
            return;
        }
    }

    /** This prepares for the UI Installation. It stops editing and updates the graph's layout caches.
     */
    protected void prepareForUIInstall()
    {
        stopEditingInCompleteEditing = true;
        preferredSize = new Dimension();
        setGraphLayoutCache(graph.getGraphLayoutCache());
        setModel(graph.getModel());
    }

    /**
     * This completes the UI Installation by setting the selectionmodel for the application to be that of the current
     * graph's.
     */
    protected void completeUIInstall()
    {
        setSelectionModel(graph.getSelectionModel());
        updateSize();
    }

    /**
     * Sets up the graph object with all the default values, like the backgroundcolor, size, handle.
     */
    protected void installDefaults()
    {
        if(graph.getBackground() == null || (graph.getBackground() instanceof UIResource))
        {
            graph.setBackground(UIManager.getColor("Tree.background"));
        }
        if(graph.getFont() == null || (graph.getFont() instanceof UIResource))
        {
            graph.setFont(UIManager.getFont("Tree.font"));
        }
        graph.setMarqueeColor(UIManager.getColor("Table.gridColor"));
        graph.setHandleColor(UIManager.getColor("MenuItem.selectionBackground"));
        graph.setLockedHandleColor(UIManager.getColor("MenuItem.background"));
        graph.setGridColor(UIManager.getColor("Tree.selectionBackground"));
        graph.setOpaque(true);
    }

    /** This installs all the listeners for the graph. The listeners are listed below.
     * @see #transferHandler
     * @see #graphViewObserver
     * @see #defaultDropTargetListener
     * @see #componentListener
     * @see #propertyChangeListener
     * @see #mouseListener
     * @see #keyListener
     * @see #graphModelListener
     * @see #graphSelectionListener
     */
    protected void installListeners()
    {
        TransferHandler transferhandler = graph.getTransferHandler();
        if(transferhandler == null || (transferhandler instanceof UIResource))
        {
            defaultTransferHandler = createTransferHandler();
            graph.setTransferHandler(defaultTransferHandler);
        }
        if(graphLayoutCache != null)
        {
            graphViewObserver = createGraphViewObserver();
            graphLayoutCache.addObserver(graphViewObserver);
        }
        DropTarget droptarget = graph.getDropTarget();
        try
        {
            if(droptarget != null)
            {
                defaultDropTargetListener = new GraphDropTargetListener();
                droptarget.addDropTargetListener(defaultDropTargetListener);
            }
        }
        catch(TooManyListenersException toomanylistenersexception) { }
        if((propertyChangeListener = createPropertyChangeListener()) != null)
        {
            graph.addPropertyChangeListener(propertyChangeListener);
        }
        if((mouseListener = createMouseListener()) != null)
        {
            graph.addMouseListener(mouseListener);
            if(mouseListener instanceof MouseMotionListener)
            {
                graph.addMouseMotionListener((MouseMotionListener)mouseListener);
            }
        }
        if((keyListener = createKeyListener()) != null)
        {
            graph.addKeyListener(keyListener);
        }
        if((graphModelListener = createGraphModelListener()) != null && graphModel != null)
        {
            graphModel.addGraphModelListener(graphModelListener);
        }
        if((graphSelectionListener = createGraphSelectionListener()) != null && graphSelectionModel != null)
        {
            graphSelectionModel.addGraphSelectionListener(graphSelectionListener);
        }
    }

    /** This installs the keyboard actions by calling the SwingUtilities methods on the current graph.
     */
    protected void installKeyboardActions()
    {
        InputMap inputmap = getInputMap(1);
        SwingUtilities.replaceUIInputMap(graph, 1, inputmap);
        inputmap = getInputMap(0);
        SwingUtilities.replaceUIInputMap(graph, 0, inputmap);
        SwingUtilities.replaceUIActionMap(graph, createActionMap());
    }

    /** This gets the input map. i.e. the map of all actions associated with input.
     *
     * @param i 1 gets the ancestor map, 0 gets the focus map
     * @return Returns the input map
     */
    InputMap getInputMap(int i)
    {
        if(i == 1)
        {
            return (InputMap)UIManager.get("Tree.ancestorInputMap");
        }
        if(i == 0)
        {
            return (InputMap)UIManager.get("Tree.focusInputMap");
        } else
        {
            return null;
        }
    }

    /**
     * This creates a action map. An action map maps an event (a class) with a specific string name that represents that event.
     *
     * @return Returns the created map
     */
    ActionMap createActionMap()
    {
        ActionMapUIResource actionmapuiresource = new ActionMapUIResource();
        actionmapuiresource.put("selectPrevious", new GraphIncrementAction(1, "selectPrevious"));
        actionmapuiresource.put("selectPreviousChangeLead", new GraphIncrementAction(1, "selectPreviousLead"));
        actionmapuiresource.put("selectPreviousExtendSelection", new GraphIncrementAction(1, "selectPreviousExtendSelection"));
        actionmapuiresource.put("selectParent", new GraphIncrementAction(4, "selectParent"));
        actionmapuiresource.put("selectParentChangeLead", new GraphIncrementAction(4, "selectParentChangeLead"));
        actionmapuiresource.put("selectNext", new GraphIncrementAction(3, "selectNext"));
        actionmapuiresource.put("selectNextChangeLead", new GraphIncrementAction(3, "selectNextLead"));
        actionmapuiresource.put("selectNextExtendSelection", new GraphIncrementAction(3, "selectNextExtendSelection"));
        actionmapuiresource.put("selectChild", new GraphIncrementAction(2, "selectChild"));
        actionmapuiresource.put("selectChildChangeLead", new GraphIncrementAction(2, "selectChildChangeLead"));
        actionmapuiresource.put("cancel", new GraphCancelEditingAction("cancel"));
        actionmapuiresource.put("startEditing", new GraphEditAction("startEditing"));
        actionmapuiresource.put("selectAll", new GraphSelectAllAction("selectAll", true));
        actionmapuiresource.put("clearSelection", new GraphSelectAllAction("clearSelection", false));
        actionmapuiresource.put(TransferHandler.getCutAction().getValue("Name"), TransferHandler.getCutAction());
        actionmapuiresource.put(TransferHandler.getCopyAction().getValue("Name"), TransferHandler.getCopyAction());
        actionmapuiresource.put(TransferHandler.getPasteAction().getValue("Name"), TransferHandler.getPasteAction());
        return actionmapuiresource;
    }

    /** This installs rendererPane components to the graph.Calls the add method.
     */
    protected void installComponents()
    {
        if((rendererPane = createCellRendererPane()) != null)
        {
            graph.add(rendererPane);
        }
    }

    /** This creates a new GraphTransferHandler
     * @return Returns the new GraphTransferHandler */
    protected TransferHandler createTransferHandler()
    {
        return new GraphTransferHandler();
    }

    /** This creates a new propertyChangeListener
     * @return Returns the new PropertyChangeListener */
    protected PropertyChangeListener createPropertyChangeListener()
    {
        return new PropertyChangeHandler();
    }

    /** This creates a new MouseListener
     * @return Returns the new MouseListener */
    protected MouseListener createMouseListener()
    {
        return new MouseHandler();
    }

    /** This creates a new KeyListener
     * @return Returns the new KeyListener */
    protected KeyListener createKeyListener()
    {
        return new KeyHandler();
    }

    /** This creates the new GraphSelectionListener
     * @return Returns the new listener */
    protected GraphSelectionListener createGraphSelectionListener()
    {
        return new GraphSelectionHandler();
    }

    /** This creates a new CellEditorListener
     * @return Returns the new listener */
    protected CellEditorListener createCellEditorListener()
    {
        return new CellEditorHandler();
    }

    /** This creates a new ComponentListener
     * @return Returns the new listener */
    protected ComponentListener createComponentListener()
    {
        return new ComponentHandler();
    }

    /**
     * This creates a new CellRendererPane
     *
     * @return Returns the new pane
     * */
    protected CellRendererPane createCellRendererPane()
    {
        return new CellRendererPane();
    }

    /**
     * This creates a new GraphViewObserver
     *
     * @return Returns the new observer
     */
    protected Observer createGraphViewObserver()
    {
        return new GraphViewObserver();
    }

    /**
     * This creates a new GraphModelListener
     *
     * @return Returns the new listener created
     * */
    protected GraphModelListener createGraphModelListener()
    {
        return new GraphModelHandler();
    }

    /** This uninstalls the UI
     *
     * @param jcomponent this is the component that the UI is uninstalled from
     */
    public void uninstallUI(JComponent jcomponent)
    {
        cancelEditing(graph);
        uninstallListeners();
        uninstallKeyboardActions();
        uninstallComponents();
        completeUIUninstall();
    }

    /**
     * This completes the UI uninstallation. It sets the following objects to null,
     * graphLayoutCache,rendererPane, componentListener, keyListener, propertyChangeListener,graph,graphModel,graphSelectionModel
     * graphSelectionListener
     */
    protected void completeUIUninstall()
    {
        graphLayoutCache = null;
        rendererPane = null;
        componentListener = null;
        propertyChangeListener = null;
        keyListener = null;
        setSelectionModel(null);
        graph = null;
        graphModel = null;
        graphSelectionModel = null;
        graphSelectionListener = null;
    }

    /**
     * This uninstalls all the listeners for this graph. The following listeners are made null.
     * @see #transferHandler
     * @see #graphViewObserver
     * @see #defaultDropTargetListener
     * @see #componentListener
     * @see #propertyChangeListener
     * @see #mouseListener
     * @see #keyListener
     * @see #graphModelListener
     * @see #graphSelectionListener
     */
    protected void uninstallListeners()
    {
        TransferHandler transferhandler = graph.getTransferHandler();
        if(transferhandler == defaultTransferHandler)
        {
            graph.setTransferHandler(null);
        }
        BasicMarqueeHandler basicmarqueehandler = graph.getMarqueeHandler();
        if(graphViewObserver != null)
        {
            graphLayoutCache.deleteObserver(graphViewObserver);
        }
        if(graph.getDropTarget() != null && defaultDropTargetListener != null)
        {
            graph.getDropTarget().removeDropTargetListener(defaultDropTargetListener);
        }
        if(componentListener != null)
        {
            graph.removeComponentListener(componentListener);
        }
        if(propertyChangeListener != null)
        {
            graph.removePropertyChangeListener(propertyChangeListener);
        }
        if(mouseListener != null)
        {
            graph.removeMouseListener(mouseListener);
            if(mouseListener instanceof MouseMotionListener)
            {
                graph.removeMouseMotionListener((MouseMotionListener)mouseListener);
            }
        }
        if(keyListener != null)
        {
            graph.removeKeyListener(keyListener);
        }
        if(graphModel != null && graphModelListener != null)
        {
            graphModel.removeGraphModelListener(graphModelListener);
        }
        if(graphSelectionListener != null && graphSelectionModel != null)
        {
            graphSelectionModel.removeGraphSelectionListener(graphSelectionListener);
        }
    }

    /**
     * This uninstalls all the keyboard actions. Calls the replaceUIActionMap, replaceUIInputMap, replaceUIInputMap
     * method for the current graph.
     */
    protected void uninstallKeyboardActions()
    {
        SwingUtilities.replaceUIActionMap(graph, null);
        SwingUtilities.replaceUIInputMap(graph, 1, null);
        SwingUtilities.replaceUIInputMap(graph, 0, null);
    }

    /**
     * This uninstalls the components from the graph. It removes the entire rendering pane.
     * @see #renderPane
     */
    protected void uninstallComponents()
    {
        if(rendererPane != null)
        {
            graph.remove(rendererPane);
        }
    }

    /** This paints the Graphics specified into the JComponent, which is also specified in the parameter.
     *
     * @param g The Graphics to be used for the component.
     * @param jcomponent The component to be painted
     */
    public void paint(Graphics g, JComponent jcomponent)
    {
        if(graph != jcomponent)
        {
            throw new InternalError("BasicGraphUI cannot paint " + jcomponent.toString() + "; " + graph + " was expected.");
        }
        Graphics2D graphics2d = (Graphics2D)g;
        Rectangle rectangle = new Rectangle(g.getClipBounds());
        Rectangle rectangle1 = graph.fromScreen(new Rectangle(rectangle));
        paintBackground(g);
        java.awt.geom.AffineTransform affinetransform = graphics2d.getTransform();
        if(graph.isAntiAliased())
        {
            graphics2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        }
        double d = graph.getScale();
        graphics2d.scale(d, d);
        CellView acellview[] = graphLayoutCache.getRoots();
        for(int i = 0; i < acellview.length; i++)
        {
            Rectangle rectangle2 = acellview[i].getBounds();
            if(rectangle2 != null && rectangle1 != null && rectangle2.intersects(rectangle1))
            {
                paintCell(g, acellview[i], rectangle2, false);
            }
        }

        graphics2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
        graphics2d.setTransform(affinetransform);
        paintForeground(g);
        if(handle != null)
        {
            handle.paint(g);
        }
        g.setColor(graph.getMarqueeColor());
        g.setXORMode(graph.getBackground());
        marquee.paint(g);
        g.setPaintMode();
        if(rendererPane != null)
        {
            rendererPane.removeAll();
        }
    }

    /** This paints a cell with the Graphics provided.
     *
     * @param g The Graphics object that holds the style information.
     * @param cellview The cell to paint.
     * @param rectangle The Rectangle needed.
     * @param flag The fourth parameter to CellView.getRendererComponent().
     */
    public void paintCell(Graphics g, CellView cellview, Rectangle rectangle, boolean flag)
    {
        boolean flag1 = cellview == focus;
        boolean flag2 = graph.isCellSelected(cellview.getCell());
        Component component = cellview.getRendererComponent(graph, flag2, flag1, flag);
        rendererPane.paintComponent(g, component, graph, rectangle.x, rectangle.y, rectangle.width, rectangle.height, true);
        if(!cellview.isLeaf())
        {
            CellView acellview[] = cellview.getChildViews();
            for(int i = 0; i < acellview.length; i++)
            {
                paintCell(g, acellview[i], acellview[i].getBounds(), flag);
            }

        }
    }

    /** This paints the background of the grid.
     *
     * @param g The Graphics object with all the information about the graph, such as the size, bounds.
     */
    protected void paintBackground(Graphics g)
    {
        if(graph.isGridVisible())
        {
            paintGrid(graph.getGridSize(), g, g.getClipBounds());
        }
    }

    /** This paints a grid
     *
     * @param i This is the size of the grid squares
     * @param g The graphics object that describes the appearance of the graph.
     * @param rectangle The rectangle that represents the region of the graph that we want to change.
     */
    protected void paintGrid(int i, Graphics g, Rectangle rectangle)
    {
        if(i > 0)
        {
            int j = (int)Math.ceil(rectangle.x / i) * i;
            int k = (int)Math.ceil((rectangle.x + rectangle.width) / i) * i;
            int l = (int)Math.ceil(rectangle.y / i) * i;
            int i1 = (int)Math.ceil((rectangle.y + rectangle.height) / i) * i;
            g.setColor(graph.getGridColor());
            for(int j1 = j; j1 <= k; j1 += i)
            {
                for(int k1 = l; k1 <= i1; k1 += i)
                {
                    g.drawLine(j1, k1, j1, k1);
                }

            }

        }
    }

    /** This paints the foreground of the application.
     *
     * @param g The information that is to be painted at the ports.
     */
    protected void paintForeground(Graphics g)
    {
        if(graph.isPortsVisible())
        {
            paintPorts(g, graphLayoutCache.getPorts());
        }
    }

    /** This paints ports to the screen.
     *
     * @param g The information that is to be painted at the ports.
     * @param aportview The ports to be painted.
     */
    public void paintPorts(Graphics g, PortView aportview[])
    {
        if(aportview != null)
        {
            Rectangle rectangle = g.getClipBounds();
            for(int i = 0; i < aportview.length; i++)
            {
                if(aportview[i] != null)
                {
                    Rectangle rectangle1 = new Rectangle(aportview[i].getBounds());
                    Point point = graph.toScreen(new Point((int)rectangle1.getCenterX(), (int)rectangle1.getCenterY()));
                    rectangle1.setLocation(point.x - rectangle1.width / 2, point.y - rectangle1.height / 2);
                    CellView cellview = aportview[i].getParentView();
                    if(rectangle == null || rectangle1.intersects(rectangle))
                    {
                        paintCell(g, aportview[i], rectangle1, false);
                    }
                }
            }

        }
    }

    /**
     * This method updates the handle for the class or creates a new handle.
     *
     */
    protected void updateHandle()
    {
        if(graphLayoutCache != null)
        {
            Object aobj[] = graphLayoutCache.order(graph.getSelectionCells());
            if(aobj != null && aobj.length > 0)
            {
                handle = createHandle(createContext(graph, aobj));
            } else
            {
                handle = null;
            }
        }
    }




    /** This method creates a new graphcontext
     *
     * @param jgraph this is the jgraph that is being edited
     * @param aobj this is the cell that is being edited
     * @return Returns the new context
     */
    protected GraphContext createContext(JGraph jgraph, Object aobj[])
    {
        return new GraphContext(jgraph, aobj);
    }

    /**
     * Creates a new RootHandle for the cells based on the specified granphcontext
     *
     * @param graphcontext The current context of the graph.
     * @return The RootHandle for this graphcontext or null if the graphcontext is null/empty
     */
    public CellHandle createHandle(GraphContext graphcontext)
    {
      if(graphcontext != null && !graphcontext.isEmpty() && graph.isEnabled())
      {
        return new RootHandle(graphcontext);
      }
      else
      {
        return null;
      }
    }

    /**
     * Sets as invalid the chachedPreferredSize. It also calls the graphDidChange () method to indicate that the graph
     * changed and updates the handles.
     * @see #validCachedPreferredSize
     */
    public void updateSize()
    {
        validCachedPreferredSize = false;
        graph.graphDidChange();
        updateHandle();
    }

    /** This method updates the cached preferred size for the graph.
     */
    protected void updateCachedPreferredSize()
    {
        Rectangle rectangle = AbstractCellView.getBounds(graphLayoutCache.getRoots());
        if(rectangle == null)
        {
            rectangle = new Rectangle();
        }
        Point point = new Point(rectangle.x + rectangle.width, rectangle.y + rectangle.height);
        Dimension dimension = graph.getMinimumSize();
        Point point1 = dimension == null ? new Point(0, 0) : graph.toScreen(new Point(dimension.width, dimension.height));
        Point point2 = graph.toScreen(point);
        preferredSize = new Dimension(Math.max(point1.x, point2.x), Math.max(point1.y, point2.y));
        Insets insets = graph.getInsets();
        if(insets != null)
        {
            preferredSize.width += insets.left + insets.right;
            preferredSize.height += insets.top + insets.bottom;
        }
        validCachedPreferredSize = true;
    }

    /**
     * This method sets the preferred minimum size for this graph.
     *
     * @param dimension The preferredMinSize of the graph as a Dimension object.
     * @see #preferredMinSize
    */
    public void setPreferredMinSize(Dimension dimension)
    {
        preferredMinSize = dimension;
    }

    /**
     * This method returns the preferred minimum size for this graph.
     *
     * @return The preferredMinSize of the graph as a Dimension object. <code>null</code> if preferredMinSize is null
     * @see #preferredMinSize
    */
    public Dimension getPreferredMinSize()
    {
        if(preferredMinSize == null)
        {
            return null;
        } else
        {
            return new Dimension(preferredMinSize);
        }
    }

    /**
     * Returns the preferred size for the specified jcomponent. The preferredSize is computed based on the preferredMinSize
     * for the components in this class.
     *
     * @param jcomponent This is the jcomponent that we want to resize.
     * @return A Dimension object thatis preferred dimension for this component.
     * @see #preferredMinSize
     * @see #getPreferredMinSize ()
     */
    public Dimension getPreferredSize(JComponent jcomponent)
    {
        Dimension dimension = getPreferredMinSize();
        if(!validCachedPreferredSize)
        {
            updateCachedPreferredSize();
        }
        if(graph != null)
        {
            if(dimension != null)
            {
                return new Dimension(Math.max(dimension.width, preferredSize.width), Math.max(dimension.height, preferredSize.height));
            } else
            {
                return new Dimension(preferredSize.width, preferredSize.height);
            }
        }
        if(dimension != null)
        {
            return dimension;
        } else
        {
            return new Dimension(0, 0);
        }
    }

    /** Returns the minimum size possible for the specified component.
     *
     * @param jcomponent This is the component that we are interested in.
     * @return A Dimnesion object representing the minimum size for this component.
     */
    public Dimension getMinimumSize(JComponent jcomponent)
    {
        if(getPreferredMinSize() != null)
        {
            return getPreferredMinSize();
        } else
        {
            return new Dimension(0, 0);
        }
    }

    /** Returns the maximum size possible for the specified component.
     *
     * @param jcomponent This is the component that we are interested in.
     * @return A Dimnesion object representing the maximum size for this component.
     */
    public Dimension getMaximumSize(JComponent jcomponent)
    {
        if(graph != null)
        {
            return getPreferredSize(graph);
        }
        if(getPreferredMinSize() != null)
        {
            return getPreferredMinSize();
        } else
        {
            return new Dimension(0, 0);
        }
    }

    /**
     * This completes the editing of the cell. This method just calls completeEditing(boolean flag, boolean flag1, boolean flag2)
     * with suiteable default parameters.
     *
     * @see #completeEditing(boolean flag, boolean flag1, boolean flag2)
     */
    protected void completeEditing()
    {
        if(graph.getInvokesStopCellEditing() && stopEditingInCompleteEditing && editingComponent != null)
        {
            cellEditor.stopCellEditing();
        }
        completeEditing(false, true, false);
    }

    /** This completes the editing of the cell. It also has additional information about whether the cell edit
     * should be stopped or cancelled.
     *
     * @param flag If true, stopCellEditing.
     * @param flag1 If true, cancelCellEditing.
     * @param flag2 If true, requestFocus for this cell.
     */
    protected void completeEditing(boolean flag, boolean flag1, boolean flag2)
    {
        if(stopEditingInCompleteEditing && editingComponent != null)
        {
            Component component = editingComponent;
            Object obj = editingCell;
            GraphCellEditor graphcelleditor = cellEditor;
            Object obj1 = graphcelleditor.getCellEditorValue();
            Rectangle rectangle = graph.getCellBounds(editingCell);
            boolean flag3 = graph != null && (graph.hasFocus() || editingComponent.hasFocus());
            editingCell = null;
            editingComponent = null;
            if(flag)
            {
                graphcelleditor.stopCellEditing();
            } else
            if(flag1)
            {
                graphcelleditor.cancelCellEditing();
            }
            graph.remove(component);
            if(flag3)
            {
                graph.requestFocus();
            }
            if(flag2)
            {
                Map map = GraphConstants.createAttributes(obj, "value", obj1);
                graphLayoutCache.edit(map, null, null, null);
            }
            updateSize();
            if(graphcelleditor != null && cellEditorListener != null)
            {
                graphcelleditor.removeCellEditorListener(cellEditorListener);
            }
            cellEditor = null;
        }
    }

    /** This method provides the editing capabilities for the graph.
     *
     * @param obj This is the cell to edit.
     * @param mouseevent This is a mouse event which records the mouse clicks.
     * @return Return true if the cell is editable.
     */
    protected boolean startEditing(Object obj, MouseEvent mouseevent)
    {
        completeEditing();
        if(graph.isCellEditable(obj))
        {
            CellView cellview = graphLayoutCache.getMapping(obj, false);
            cellEditor = cellview.getEditor();
            editingComponent = cellEditor.getGraphCellEditorComponent(graph, obj, graph.isCellSelected(obj));
            if(cellEditor.isCellEditable(mouseevent))
            {
                Rectangle rectangle = graph.getCellBounds(obj);
                editingCell = obj;
                Dimension dimension = editingComponent.getPreferredSize();
                if(dimension.width != rectangle.width || dimension.height != rectangle.height)
                {
                    editorHasDifferentSize = true;
                    updateSize();
                } else
                {
                    editorHasDifferentSize = false;
                }
                graph.add(editingComponent);
                Point point = graph.toScreen(new Point(rectangle.x, rectangle.y));
                if(obj instanceof DefaultEdge)
                {
                    EdgeRenderer edgerenderer = EdgeView.renderer;
                    EdgeView edgeview = (EdgeView)graphLayoutCache.getMapping(obj, false);
                    point = edgerenderer.getLabelPosition(edgeview);
                    point.x = point.x - dimension.width / 2;
                    point.y = point.y - dimension.height / 2;
                    graph.toScreen(point);
                }
                editingComponent.setBounds(point.x, point.y, dimension.width, dimension.height);
                editingCell = obj;
                editingComponent.validate();
                if(cellEditorListener == null)
                {
                    cellEditorListener = createCellEditorListener();
                }
                if(cellEditor != null && cellEditorListener != null)
                {
                    cellEditor.addCellEditorListener(cellEditorListener);
                }
                Rectangle rectangle1 = graph.getVisibleRect();
                graph.paintImmediately(point.x, point.y, (rectangle1.width + rectangle1.x) - rectangle.x, dimension.height);
                if(cellEditor.shouldSelectCell(mouseevent))
                {
                    stopEditingInCompleteEditing = false;
                    try
                    {
                        graph.setSelectionCell(obj);
                    }
                    catch(Exception exception)
                    {
                        System.err.println("Editing exception: " + exception);
                    }
                    stopEditingInCompleteEditing = true;
                }
                if(mouseevent instanceof MouseEvent)
                {
                    Point point1 = SwingUtilities.convertPoint(graph, new Point(mouseevent.getX(), mouseevent.getY()), editingComponent);
                    Component component = SwingUtilities.getDeepestComponentAt(editingComponent, point1.x, point1.y);
                    if(component != null)
                    {
                        new MouseInputHandler(graph, component, mouseevent);
                    }
                }
                return true;
            }
            editingComponent = null;
        }
        return false;
    }

    /** This sets the autoscroll for the graph. This keeps changing as different components are chosen on the slide.
     *
     * @param jgraph The graph that is currently being viewed.
     * @param point This is the current point of the scrollbar.
     */
    public static void autoscroll(JGraph jgraph, Point point)
    {
        Rectangle rectangle = jgraph.getBounds();
        if(jgraph.getParent() instanceof JViewport)
        {
            rectangle = ((JViewport)jgraph.getParent()).getViewRect();
        }
        if(rectangle.contains(point))
        {
            Point point1 = new Point(point);
            int i = (int)((float)jgraph.getWidth() * SCROLLSTEP);
            int j = (int)((float)jgraph.getHeight() * SCROLLSTEP);
            if(point1.x - rectangle.x < SCROLLBORDER)
            {
                point1.x -= i;
            }
            if(point1.y - rectangle.y < SCROLLBORDER)
            {
                point1.y -= j;
            }
            if((rectangle.x + rectangle.width) - point1.x < SCROLLBORDER)
            {
                point1.x += i;
            }
            if((rectangle.y + rectangle.height) - point1.y < SCROLLBORDER)
            {
                point1.y += j;
            }
            jgraph.scrollPointToVisible(point1);
        }
    }

    /**
     * This method inserts cells in the graph.
     *
     * @param aobj The array of cells that need to be inserted.
     * @param map This is the map that describes where to clone the cells.
     * @param connectionset Tthe connectionset to clone.
     * @param flag If this is false, need to clone and then insert cells.
     * @param i This is the first graphconstant.
     * @param j This is the second graphconstant.
     */
    public void insertCells(Object aobj[], Map map, ConnectionSet connectionset, boolean flag, int i, int j)
    {
        Object aobj1[];
        if(flag)
        {
            Map map1 = graph.cloneCells(aobj);
            map = GraphConstants.replaceKeys(map1, map);
            GraphConstants.translate(map.values(), i, j);
            connectionset = connectionset.clone(map1);
            aobj1 = new Object[aobj.length];
            for(int k = 0; k < aobj.length; k++)
            {
                aobj1[k] = map1.get(aobj[k]);
            }

        } else
        {
            aobj1 = aobj;
        }
        graphLayoutCache.insert(aobj1, map, connectionset, null, null);
    }

    /** Determines whether snapview is selected
     *
     * @return Return true if snapview is selected, false otherwise.
     */
    public boolean isSnapSelectedView()
    {
        return snapSelectedView;
    }

    /**
     * Sets or clears the snapview.
     *
     * @param flag True if snapview should be set, flase otherwise.
     */
    public void setSnapSelectedView(boolean flag)
    {
        snapSelectedView = flag;
    }

}
