// COPY OF JAVAX.SWING.PLAF.BASIC WITH A PUBLIC MODIFIER AND CONSTRUCTOR
/*
 * @(#)BasicTransferable.java	1.4 01/06/28
 *
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
 */
import java.awt.datatransfer.*;
import java.io.*;
import javax.swing.plaf.UIResource;

/**
 * <p>
 * COPY OF JAVAX.SWING.PLAF.BASIC WITH A PUBLIC MODIFIER AND CONSTRUCTOR
 * <p>
 * A transferable implementation for the default data transfer of some Swing
 * components.
 *
 * @author  Timothy Prinzing
 * @version 2.0 TerpPresent.
 */
public class BasicTransferable
    implements Transferable, UIResource, Serializable
{

    /** This holds the html flavors
     */
    private static DataFlavor htmlFlavors[];
    /** This holds the string flavors
     */
    private static DataFlavor stringFlavors[];
    /** This holds the plain flavors
     */
    private static DataFlavor plainFlavors[];

    /** This is the BasicTransferable constructor
     */
    public BasicTransferable()
    {
    }

  /**
   * Returns an array of DataFlavor objects indicating the flavors the data
   * can be provided in.  The array should be ordered according to preference
   * for providing the data (from most richly descriptive to least descriptive).
   * @return an array of data flavors in which this data can be transferred
   */
    public DataFlavor[] getTransferDataFlavors()
    {
        DataFlavor adataflavor[] = getRicherFlavors();
        int i = adataflavor == null ? 0 : adataflavor.length;
        int j = isHTMLSupported() ? htmlFlavors.length : 0;
        int k = isPlainSupported() ? plainFlavors.length : 0;
        int l = isPlainSupported() ? stringFlavors.length : 0;
        int i1 = i + j + k + l;
        DataFlavor adataflavor1[] = new DataFlavor[i1];
        int j1 = 0;
        if(i > 0)
        {
            System.arraycopy(adataflavor, 0, adataflavor1, j1, i);
            j1 += i;
        }
        if(j > 0)
        {
            System.arraycopy(htmlFlavors, 0, adataflavor1, j1, j);
            j1 += j;
        }
        if(k > 0)
        {
            System.arraycopy(plainFlavors, 0, adataflavor1, j1, k);
            j1 += k;
        }
        if(l > 0)
        {
            System.arraycopy(stringFlavors, 0, adataflavor1, j1, l);
            j1 += l;
        }
        return adataflavor1;
    }

  /**
   * Returns whether or not the specified data flavor is supported for
   * this object.
   * @param dataflavor the requested flavor for the data
   * @return boolean indicating whether or not the data flavor is supported
   */
    public boolean isDataFlavorSupported(DataFlavor dataflavor)
    {
        DataFlavor adataflavor[] = getTransferDataFlavors();
        for(int i = 0; i < adataflavor.length; i++)
        {
            if(adataflavor[i].equals(dataflavor))
            {
                return true;
            }
        }

        return false;
    }

    /**
   * Returns an object which represents the data to be transferred.  The class
   * of the object returned is defined by the representation class of the flavor.
     * @param dataflavor this is the flavor of what you are trying to transfer
     * @throws UnsupportedFlavorException this occurs if the flavor is not supported
     * @throws IOException this is a general exception
     * @return Returns an object representing the data to be transferred */
    public Object getTransferData(DataFlavor dataflavor)
        throws UnsupportedFlavorException, IOException
    {
        DataFlavor adataflavor[] = getRicherFlavors();
        if(isRicherFlavor(dataflavor))
        {
            return getRicherData(dataflavor);
        }
        if(isHTMLFlavor(dataflavor))
        {
            String s = getHTMLData();
            s = s != null ? s : "";
            if((java.lang.String.class).equals(dataflavor.getRepresentationClass()))
            {
                return s;
            }
            if((java.io.Reader.class).equals(dataflavor.getRepresentationClass()))
            {
                return new StringReader(s);
            }
            if((java.io.InputStream.class).equals(dataflavor.getRepresentationClass()))
            {
                return new StringBufferInputStream(s);
            }
        } else
        if(isPlainFlavor(dataflavor))
        {
            String s1 = getPlainData();
            s1 = s1 != null ? s1 : "";
            if((java.lang.String.class).equals(dataflavor.getRepresentationClass()))
            {
                return s1;
            }
            if((java.io.Reader.class).equals(dataflavor.getRepresentationClass()))
            {
                return new StringReader(s1);
            }
            if((java.io.InputStream.class).equals(dataflavor.getRepresentationClass()))
            {
                return new StringBufferInputStream(s1);
            }
        } else
        if(isStringFlavor(dataflavor))
        {
            String s2 = getPlainData();
            s2 = s2 != null ? s2 : "";
            return s2;
        }
        throw new UnsupportedFlavorException(dataflavor);
    }

    /** This checks to see if there is a richer flavor available.
     * @param dataflavor this is the flavor that is being assessed for richness
     * @return Returns true if this equals an existing richer flavor */
    protected boolean isRicherFlavor(DataFlavor dataflavor)
    {
        DataFlavor adataflavor[] = getRicherFlavors();
        int i = adataflavor == null ? 0 : adataflavor.length;
        for(int j = 0; j < i; j++)
        {
            if(adataflavor[j].equals(dataflavor))
            {
                return true;
            }
        }

        return false;
    }

    /**
   * Some subclasses will have flavors that are more descriptive than HTML
   * or plain text.  If this method returns a non-null value, it will be
   * placed at the start of the array of supported flavors.
     * @return Returns null */
    protected DataFlavor[] getRicherFlavors()
    {
        return null;
    }

    /** This method checks for richer data.
     * @param dataflavor This is the flavor in question
     * @throws UnsupportedFlavorException This is thrown if the dataflavor is not avaialable
     * @return Returns null */
    protected Object getRicherData(DataFlavor dataflavor)
        throws UnsupportedFlavorException
    {
        return null;
    }

  /**
   * Returns whether or not the specified data flavor is an HTML flavor that
   * is supported.
   * @param dataflavor the requested flavor for the data
   * @return boolean indicating whether or not the data flavor is supported
   */
    protected boolean isHTMLFlavor(DataFlavor dataflavor)
    {
        DataFlavor adataflavor[] = htmlFlavors;
        for(int i = 0; i < adataflavor.length; i++)
        {
            if(adataflavor[i].equals(dataflavor))
            {
                return true;
            }
        }

        return false;
    }

    /**
   * Should the HTML flavors be offered?  If so, the method
   * getHTMLData should be implemented to provide something reasonable.
     * @return Returns false  */
    protected boolean isHTMLSupported()
    {
        return false;
    }

    /**
   * Fetch the data in a text/html format
     * @return Returns null  */
    protected String getHTMLData()
    {
        return null;
    }

  /**
   * Returns whether or not the specified data flavor is an plain flavor that
   * is supported.
   * @param dataflavor the requested flavor for the data
   * @return boolean indicating whether or not the data flavor is supported
   */
    protected boolean isPlainFlavor(DataFlavor dataflavor)
    {
        DataFlavor adataflavor[] = plainFlavors;
        for(int i = 0; i < adataflavor.length; i++)
        {
            if(adataflavor[i].equals(dataflavor))
            {
                return true;
            }
        }

        return false;
    }

    /**
   * Should the plain text flavors be offered?  If so, the method
   * getPlainData should be implemented to provide something reasonable.
     * @return Returns false */
    protected boolean isPlainSupported()
    {
        return false;
    }

    /**
   * Fetch the data in a text/plain format.
     * @return Returns null */
    protected String getPlainData()
    {
        return null;
    }

  /**
   * Returns whether or not the specified data flavor is a String flavor that
   * is supported.
   * @param dataflavor the requested flavor for the data
   * @return boolean indicating whether or not the data flavor is supported
   */
    protected boolean isStringFlavor(DataFlavor dataflavor)
    {
        DataFlavor adataflavor[] = stringFlavors;
        for(int i = 0; i < adataflavor.length; i++)
        {
            if(adataflavor[i].equals(dataflavor))
            {
                return true;
            }
        }

        return false;
    }

    static
    {
        try
        {
            htmlFlavors = new DataFlavor[3];
            htmlFlavors[0] = new DataFlavor("text/html;class=java.lang.String");
            htmlFlavors[1] = new DataFlavor("text/html;class=java.io.Reader");
            htmlFlavors[2] = new DataFlavor("text/html;charset=unicode;class=java.io.InputStream");
            plainFlavors = new DataFlavor[3];
            plainFlavors[0] = new DataFlavor("text/plain;class=java.lang.String");
            plainFlavors[1] = new DataFlavor("text/plain;class=java.io.Reader");
            plainFlavors[2] = new DataFlavor("text/plain;charset=unicode;class=java.io.InputStream");
            stringFlavors = new DataFlavor[2];
            stringFlavors[0] = new DataFlavor("application/x-java-jvm-local-objectref;class=java.lang.String");
            stringFlavors[1] = DataFlavor.stringFlavor;
        }
        catch(ClassNotFoundException classnotfoundexception)
        {
            System.err.println("error initializing javax.swing.plaf.basic.BasicTranserable");
        }
    }
}
