
import java.awt.*;
import java.util.Map;

/**  
 * This class deals with a cell viewing manipulation.
 *
 * Referenced classes of package com.jraph.graph are
 * GraphContext, CellHandle and GraphCellEditor.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */

public interface CellView

{
  
  /** 
   * Retrieves a cell.
   * @return An Object data type.
   * @see Object
   */    
  public abstract Object getCell();
  
  /** 
   * Refreshes a cell.
   * @param flag A boolean data type.
   */    
  public abstract void refresh(boolean flag);
  
  /**  
   * Updates a cell.
   */    
  public abstract void update();
  
  /**
   * Updates a child cell.
   */ 
  public abstract void childUpdated();
  
  /**
   * Retrieves a parent's view.
   * @return A CellView data type.
   * @see CellView
   */    
  public abstract CellView getParentView();
  
  /**
   * Retrieves all the child views as an array.
   * @return A CellView[] data type.
   * @see CellView
   */    
  public abstract CellView[] getChildViews();
  
  /**
   * Removes a child from a parent cell.
   */    
  public abstract void removeFromParent();
  
  /**
   * Checks if a cell is leaf.
   * @return A boolean data type.
   */    
  public abstract boolean isLeaf();
  
  /**
   * Returns a rectangle with its bounds.
   * @return A Rectangle data type.
   * @see Rectangle
   */    
  public abstract Rectangle getBounds();
  
  /**
   * Checks if a Graphics Object and a Rectangle intersect each other.
   * @param g A variable of Graphics data type.
   * @param rectangle A variable of Rectangle data type.
   * @return A boolean data type.
   * @see Graphics
   * @see Rectangle
   */    
  public abstract boolean intersects(Graphics g, Rectangle rectangle);
  
  /**
   * Sets certain attributes to a Map Object.
   * @param map A variable of Map data type.
   * @return A Map data type.
   * @see Map
   */    
  public abstract Map setAttributes(Map map);
  
  /**
   * Retrieves a Map with its attributes
   * @return A Map data type
   * @see Map
   */    
  public abstract Map getAttributes();
  
  /**
   * Retrieves a Map with all its attributes.
   * @return A Map data type.
   * @see Map
   */    
  public abstract Map getAllAttributes();
  
  /**
   * Retrieves a renderer component.
   * @param jgraph A variable of JGraph data type.
   * @param flag A variable of boolean data type.
   * @param flag1 A variable of boolean data type.
   * @param flag2 A variable of boolean data type.
   * @return A Component data type.
   * @see JGraph
   * @see Component
   */    
  public abstract Component getRendererComponent(JGraph jgraph, boolean flag, boolean flag1, boolean flag2);
  
  /**
   * Retrieves a handle from an object.
   * @param graphcontext A variable of GraphContext data type.
   * @return A CellHandle data type.
   * @see GraphContext
   * @see CellHandle
   */    
  public abstract CellHandle getHandle(GraphContext graphcontext);
  
  /**
   * Retrieves a cell editor.
   * @return A GraphCellEditor data type.
   * @see GraphCellEditor
   */    
  public abstract GraphCellEditor getEditor();
}
