import java.io.Serializable;
import java.util.*;

/** 
 * A public class that implements Serializable.
 *
 * Referenced classes of package com.jgraph.graph are
 * Edge, EdgeView, GraphModel, PortView, Port and CellView.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 *
 * @see Serializable
 */

public class ConnectionSet
    implements Serializable
{
    public static class Connection
        implements Serializable
    {

        /** 
         * An instance of Object.
         */        
        protected Object edge;
        
        /** 
         * An instance of Object.
         */        
        protected Object port;
        
        /** 
         * An instance of boolean.
         */        
        protected boolean isSource;

        /**
         * Retrieves an edge.
         * @return An object data type.
         */        
        public Object getEdge()
        {
            return edge;
        }

        /**  
         * Retrieves a port.
         * @return An Object data type.
         */        
        public Object getPort()
        {
            return port;
        }

        /** 
         * Checks if a current object is a source or not.
         * @return A boolean data type.
         */        
        public boolean isSource()
        {
            return isSource;
        }

        /** 
         * Checks if an Object is a Connection.
         * @param obj A variable of Object data type.
         * @return A boolean data type.
         * @see Object
         */        
        public boolean equals(Object obj)
        {
            if(obj instanceof Connection)
            {
                Connection connection = (Connection)obj;
                return connection.getEdge().equals(edge) && connection.isSource() == isSource;
            } else
            {
                return false;
            }
        }

        /**
         * Retrieves a hashcode.
         * @return An integer data type.
         */        
        public int hashCode()
        {
            return edge.hashCode();
        }

        /** 
         * Constructor.
         * @param obj A variable of Object data type.
         * @param obj1 A variable of Object data type.
         * @param flag A variable of boolean data type.
         * @see Object
         */        
        public Connection(Object obj, Object obj1, boolean flag)
        {
            edge = obj;
            port = obj1;
            isSource = flag;
        }
    }


    /**
     * An instance of Set.
     */    
    protected Set connections;
    
    /** 
     * An instance of Set.
     */    
    protected Set edges;

    /** 
     * Creates a connection set.
     * @param graphmodel A variable of GraphModel data type.
     * @param aobj A variable of Object data type.
     * @param flag A variable of boolean data type.
     * @return A ConnectionSet data type.
     * @see GraphModel
     * @see Object
     * @see ConnectionSet
     */    
    public static ConnectionSet create(GraphModel graphmodel, Object aobj[], boolean flag)
    {
        ConnectionSet connectionset = new ConnectionSet();
        for(int i = 0; i < aobj.length; i++)
        {
            Object obj = aobj[i];
            if(obj instanceof Edge)
            {
                if(flag)
                {
                    connectionset.disconnect(obj);
                } else
                {
                    connectionset.connect(obj, graphmodel.getSource(obj), graphmodel.getTarget(obj));
                }
            }
            for(Iterator iterator = graphmodel.edges(obj); iterator.hasNext();)
            {
                Object obj1 = iterator.next();
                if(graphmodel.getSource(obj1) == obj)
                {
                    connect(connectionset, obj1, obj, true, flag);
                } else
                if(graphmodel.getTarget(obj1) == obj)
                {
                    connect(connectionset, obj1, obj, false, flag);
                }
            }

        }

        return connectionset;
    }

    /** 
     * Constructor that initializes Set variables without any parameters.
     */    
    public ConnectionSet()
    {
        connections = new HashSet();
        edges = new HashSet();
    }

    /** 
     * Constructor.
     * @param edge A variable of Edge data type.
     * @param port A variable of Port data type.
     * @param flag A variable of boolean data type.
     * @see Edge
     * @see Port
     */    
    public ConnectionSet(Edge edge, Port port, boolean flag)
    {
        connections = new HashSet();
        edges = new HashSet();
        connect(edge, port, flag);
    }

    /** 
     * Constructor.
     * @param edge A variable of Edge data type.
     * @param port A variable of Port data type.
     * @param flag A variable of Port data type.
     * @see Edge
     * @see Port
     */    
    public ConnectionSet(Edge edge, Port port, Port port1)
    {
        connections = new HashSet();
        edges = new HashSet();
        connect(edge, port, port1);
    }

    /**
     * Calls a ConnectionSet connect or disconnect method depending of the flag value.
     * @param connectionset A variable of ConnectionSet data type.
     * @param obj A variable of Object data type.
     * @param obj1 A variable of Object data type.
     * @param flag A variable of boolean data type.
     * @param flag1 A variable of boolean data type.
     * @see ConnectionSet
     * @see Object
     */    
    protected static void connect(ConnectionSet connectionset, Object obj, Object obj1, boolean flag, boolean flag1)
    {
        if(flag1)
        {
            connectionset.disconnect(obj, flag);
        } else
        {
            connectionset.connect(obj, obj1, flag);
        }
    }

    /**
     * Adds connections using an array of objects in parameter list.
     * @param acellview A variable of an array of CellViews data type.
     * @see CellView
     */    
    public void addConnections(CellView acellview[])
    {
        for(int i = 0; i < acellview.length; i++)
        {
            if(acellview[i] instanceof EdgeView)
            {
                EdgeView edgeview = (EdgeView)acellview[i];
                Object obj = edgeview.getCell();
                PortView portview = edgeview.getSource();
                PortView portview1 = edgeview.getTarget();
                Object obj1 = null;
                if(portview != null)
                {
                    obj1 = portview.getCell();
                }
                Object obj2 = null;
                if(portview1 != null)
                {
                    obj2 = portview1.getCell();
                }
                connect(obj, obj1, obj2);
            }
        }

    }

    /**
     * Calls a connect function using parameter list.
     * @param obj A variable of Object data type.
     * @param obj1 A variable of Object data type.
     * @param obj2 A variable of Object data type.
     * @see Object
     */    
    public void connect(Object obj, Object obj1, Object obj2)
    {
        connect(obj, obj1, true);
        connect(obj, obj2, false);
    }

    /** 
     * Adds a new connection to a set of edges using parameter list.
     * @param obj A variable of Object data type.
     * @param obj1 A variable of Object data type.
     * @param flag A variable of boolean data type.
     * @see Object
     */    
    public void connect(Object obj, Object obj1, boolean flag)
    {
        Connection connection = new Connection(obj, obj1, flag);
        connections.remove(connection);
        connections.add(connection);
        edges.add(obj);
    }

    /**
     * Removes a specified connection object.
     * @param obj A variable of Object data type.
     * @see Object
     */    
    public void disconnect(Object obj)
    {
        disconnect(obj, true);
        disconnect(obj, false);
    }

    /**
     * Removes a connection.
     * @param obj A variable of Object data type.
     * @param flag A variable of boolean data type.
     * @see Object
     */    
    public void disconnect(Object obj, boolean flag)
    {
        connections.add(new Connection(obj, null, flag));
        edges.add(obj);
    }

    /**
     * Checks if a connection set is empty.
     * @return A boolean data type.
     */    
    public boolean isEmpty()
    {
        return connections.isEmpty();
    }

    /**
     * Shows size of the connection set.
     * @return An integer data type.
     */    
    public int size()
    {
        return connections.size();
    }

    /**
     * Gets a Connection Iterator.
     * @return An Iterator data type.
     */    
    public Iterator connections()
    {
        return connections.iterator();
    }

    /**
     * Returns a set of edges.
     * @return A Set data type.
     */    
    public Set getChangedEdges()
    {
        return edges;
    }

    /**
     * Returns a connectionset from a provided map.
     * @param map A variable of Map data type.
     * @return A ConnectionSet data type.
     */    
    public ConnectionSet clone(Map map)
    {
        ConnectionSet connectionset = new ConnectionSet();
        for(Iterator iterator = connections(); iterator.hasNext();)
        {
            Connection connection = (Connection)iterator.next();
            Object obj = map.get(connection.getEdge());
            Object obj1 = connection.getPort();
            if(obj1 != null)
            {
                obj1 = map.get(obj1);
            }
            if(obj != null && obj1 != null)
            {
                connectionset.connect(obj, obj1, connection.isSource());
            } else
            if(obj != null)
            {
                connectionset.connect(obj, connection.getPort(), connection.isSource());
            }
        }

        return connectionset;
    }
}
