
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Map;

/** 
 * This class creates a default edge. 
 *
 * Referenced classes of package com.jgraph.graph are
 * DefaultGraphCell, Edge, GraphConstants, PortView,
 * EdgeView and CellView.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 *
 * @see Edge
 * @see DefaultGraphCell 
 */

public class DefaultEdge extends DefaultGraphCell
    implements Edge
{
    /** 
     * A nested class DefaultRouting implements Edge.Routing.
     */    
    public static class DefaultRouting
        implements Edge.Routing
    {

        /**
         * Creates a route for an edge.
         * @param edgeview A variable EdgeView data type.
         * @param list A variable java.util.List data type.
         * @see EdgeView
         * @see java.util.List
         */        
        public void route(EdgeView edgeview, java.util.List list)
        {
            int i = list.size();
            Point point = edgeview.getPoint(0);
            if(edgeview.getSource() instanceof PortView)
            {
                point = edgeview.getSource().getLocation(null);
            }
            Point point1 = edgeview.getPoint(i - 1);
            if(edgeview.getTarget() instanceof PortView)
            {
                point1 = edgeview.getTarget().getLocation(null);
            }
            if(point != null && point1 != null)
            {
                Point apoint[];
                if(edgeview.getSource() == edgeview.getTarget() && edgeview.getSource() != null)
                {
                    Rectangle rectangle = edgeview.getSource().getParentView().getBounds();
                    int l = edgeview.getGraph().getGridSize();
                    int j1 = (int)(rectangle.getWidth() / 3D);
                    apoint = new Point[4];
                    apoint[0] = new Point(rectangle.x + j1, rectangle.y + rectangle.height);
                    apoint[1] = new Point(rectangle.x + j1, rectangle.y + rectangle.height + l);
                    apoint[2] = new Point(rectangle.x + 2 * j1, rectangle.y + rectangle.height + l);
                    apoint[3] = new Point(rectangle.x + 2 * j1, rectangle.y + rectangle.height);
                } else
                {
                    int j = Math.abs(point.x - point1.x);
                    int i1 = Math.abs(point.y - point1.y);
                    int k1 = point.x + (point1.x - point.x) / 2;
                    int l1 = point.y + (point1.y - point.y) / 2;
                    apoint = new Point[2];
                    if(j > i1)
                    {
                        apoint[0] = new Point(k1, point.y);
                        apoint[1] = new Point(k1, point1.y);
                    } else
                    {
                        apoint[0] = new Point(point.x, l1);
                        apoint[1] = new Point(point1.x, l1);
                    }
                }
                for(int k = 0; k < apoint.length; k++)
                {
                    if(list.size() > k + 2)
                    {
                        list.set(k + 1, apoint[k]);
                    } else
                    {
                        list.add(k + 1, apoint[k]);
                    }
                }

                for(; list.size() > apoint.length + 2; list.remove(list.size() - 2)) { }
            }
        }

        /** 
         * Default constructor for DefaultRouting, does nothing.
         */        
        public DefaultRouting()
        {
        }
    }


    /** 
     * Creates an instance of integer equal to 500.
     */    
    protected static final int center = 500;
    
    /**
     * Creates an Instance of Point at 500,500.
     */    
    protected static final Point defaultLabel = new Point(500, 500);
    
    /**
     * An instance of ArrayList.
     */    
    protected static final ArrayList defaultPoints;
    
    /**
     * An instance of Object.
     */    
    protected Object source;
    
    /**
     * An instance of Object.
     */    
    protected Object target;

    /**
     * Makes a current object in scope null.
     */    
    public DefaultEdge()
    {
        this(null);
    }

    /** 
     * Constructor takes an object parameter to initialize the current object.
     * @param obj A variable of Object data type.
     * @see Object
     */    
    public DefaultEdge(Object obj)
    {
        this(obj, false);
    }

    /** 
     * Constructor takes two parameters to initialize the object.
     * @param obj A variable of Object data type.
     * @param flag A variable of boolean data type.
     * @see Object
     */    
    public DefaultEdge(Object obj, boolean flag)
    {
        super(obj, flag);
        GraphConstants.setPoints(attributes, defaultPoints);
        GraphConstants.setLabelPosition(attributes, defaultLabel);
    }

    /**
     * Changes attributes of an input map.
     * @param map A variable of Map data type.
     * @return A Map data type.
     * @see Map
     */    
    public Map changeAttributes(Map map)
    {
        Map map1 = super.changeAttributes(map);
        java.util.List list = GraphConstants.getPoints(attributes);
        if(list == null)
        {
            GraphConstants.setPoints(attributes, defaultPoints);
        }
        return map1;
    }

    /**
     * Returns a source.
     * @return An Object data type.
     * @see Object
     */    
    public Object getSource()
    {
        return source;
    }

    /**
     * Returns a target.
     * @return An Object data type.
     * @see Object
     */
    public Object getTarget()
    {
        return target;
    }

    /**
     * Sets a source.
     * @param obj A variable Object data type.
     * @see Object
     */
    public void setSource(Object obj)
    {
        source = obj;
    }
    
    /**
     * Sets a target.
     * @param obj A variable of Object data type.
     * @see Object
     */
    public void setTarget(Object obj)
    {
        target = obj;
    }

    /**
     * Clones an edge.
     * @return An Object data type.
     * @see DefaultEdge
     * @see Object
     */
    public Object clone()
    {
        DefaultEdge defaultedge = (DefaultEdge)super.clone();
        defaultedge.source = null;
        defaultedge.target = null;
        return defaultedge;
    }

    static 
    {
        defaultPoints = new ArrayList();
        defaultPoints.add(new Point(10, 10));
        defaultPoints.add(new Point(20, 20));
    }
}
