import java.awt.Rectangle;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Map;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;

/** 
 * This class extends DefaultMutableTreeNode to implement a graph cell.
 *
 * Referenced classes of package com.jgraph.graph are
 * GraphCell and GraphConstants.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 *
 * @see DefaultMutableTreeNode
 * @see GraphCell
 * @see Cloneable
 */

public class DefaultGraphCell extends DefaultMutableTreeNode
  implements GraphCell, Cloneable
  
{
  
  public static interface ValueChangeHandler
    
  {
    
    /**
     * Abstract casting of valueChanged.
     * @param obj A variable of Object data type.
     * @return An Object data type.
     * @see Object
     */        
    public abstract Object valueChanged(Object obj);
    
    /**
     * Abstract casting of clone.
     * @return An Object data type.
     * @see Object
     */        
    public abstract Object clone();
  }
  
  
  /**
   * Creates a rectangle with default bounds, 10, 10, 20, 20.
   */    
  public static final Rectangle defaultBounds = new Rectangle(10, 10, 20, 20);
  
  /**
   * An instance a Map.
   */    
  protected Map attributes;
  
  /**
   * Default constructor. 
   * Initializes current object to null.
   */    
  public DefaultGraphCell()
  {
    this(null);
  }
  
  /** 
   * Constructor with one parameter.
   * Initializes current object with parameter object.
   * @param obj A variable of Object data type.
   * @see Object
   */    
  public DefaultGraphCell(Object obj)
  {
    this(obj, ((MutableTreeNode []) (null)));
  }
  
  /**
   * Constructor with two parameters.
   * Initializes object with two parameter.
   * @param obj A variable of Object data type.
   * @param amutabletreenode A variable of MutableTreeNode.
   * @see Object
   * @see MutableTreeNode
   */    
  public DefaultGraphCell(Object obj, MutableTreeNode amutabletreenode[])
  {
    super(obj, true);
    attributes = GraphConstants.createMap();
    setUserObject(obj);
    if(amutabletreenode != null)
    {
      for(int i = 0; i < amutabletreenode.length; i++)
      {
        add(amutabletreenode[i]);
      }
      
    }
    GraphConstants.setBounds(attributes, defaultBounds);
  }
  
  /** 
   * Another constructor with two parameters.
   * Initializes object with two parameters.
   * @param obj A variable of Object data type.
   * @param flag A variable of boolean data type.
   * @see Object
   */      
  public DefaultGraphCell(Object obj, boolean flag)
  {
    super(obj, flag);
    attributes = GraphConstants.createMap();
  }
  
  /** 
   * Sets an user Object to the parameter Object.
   * @param obj A variable of Object data type.
   * @see Object
   */    
  public void setUserObject(Object obj)
  {
    if(userObject instanceof ValueChangeHandler)
    {
      ((ValueChangeHandler)userObject).valueChanged(obj);
    } else
    {
      super.setUserObject(obj);
    }
    obj = getUserObject();
    if(obj == null)
    {
      GraphConstants.setValue(attributes, "");
    } else
    {
      GraphConstants.setValue(attributes, obj);
    }
  }
  
  /**
   * Retrieves children of current Object if any.
   * @return A List data type.
   * @see List
   */    
  public java.util.List getChildren()
  {
    if(children == null)
    {
      return Collections.EMPTY_LIST;
    } else
    {
      return children;
    }
  }
  
  /**
   * Retrieves current attributes of an object.
   * @return A Map data type.
   * @see Map
   */
  public Map getAttributes()
  {
    return attributes;
  }
  
  /**
   * Changes current attributes of an object.
   * @param map A variable of Map data type.
   * @return A Map data type.
   * @see Map
   * @see Object
   * @see Rectangle
   */
  public Map changeAttributes(Map map)
  {
    Map map1 = GraphConstants.applyMap(map, attributes);
    Object obj = GraphConstants.getValue(attributes);
    if(userObject != null && obj == null)
    {
      GraphConstants.setValue(attributes, userObject);
    } else
      if(userObject instanceof ValueChangeHandler)
    {
      Object obj1 = ((ValueChangeHandler)userObject).valueChanged(obj);
      if(obj1 == null || !obj.toString().equals(obj1.toString()))
      {
        GraphConstants.setValue(map1, obj1);
      }
      GraphConstants.setValue(attributes, userObject);
    } else
    {
      userObject = obj;
    }
    Rectangle rectangle = GraphConstants.getBounds(attributes);
    if(rectangle == null)
    {
      GraphConstants.setBounds(attributes, defaultBounds);
    }
    return map1;
  }
  
  /**
   * Sets attributes of current Object.
   * @param map A variable of Map data type.
   * @see Map
   */
  public void setAttributes(Map map)
  {
    attributes = map;
  }
  
  /**
   * Clones a DefaultGraphCell Object.
   * @return An Object data type.
   * @see DefaultGraphCell
   * @see Object
   */
  public Object clone()
  {
    DefaultGraphCell defaultgraphcell = (DefaultGraphCell)super.clone();
    defaultgraphcell.attributes = new Hashtable(attributes);
    defaultgraphcell.userObject = cloneUserObject();
    return defaultgraphcell;
  }
  
  /**
   * Clones a user Object.
   * @return An Object data type.
   * @see ValueChangeHandler
   * @see Object
   */
  protected Object cloneUserObject()
  {
    if(userObject instanceof ValueChangeHandler)
    {
      return ((ValueChangeHandler)userObject).clone();
    } else
    {
      return userObject;
    }
  }
  
}
