/*
 * @(#)DefaultGraphModelProvider.java	1.0 17.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;
//import javax.swing.tree.TreeNode;


/**Graph model provider for the default graph model.
 * 
 * @author luzar
 * @version 2.0 TerpPresent
 */
public class DefaultGraphModelProvider implements GraphModelProvider {

	/** Reference to the default xml file format.
	 */
	DefaultGraphModelFileFormatXML defaultGraphModelFileFormatXML; 
	
	/** Reference to the default serial file format.
	 */
	DefaultGraphModelFileFormatSerial defaultGraphModelFileFormatSerial; 

	/**
	 * Constructor for DefaultGraphModelProvider.
	 */
	public DefaultGraphModelProvider() {
		defaultGraphModelFileFormatXML = new DefaultGraphModelFileFormatXML();
		defaultGraphModelFileFormatSerial = new DefaultGraphModelFileFormatSerial();
	}

	/** Returns <tt>Default Graph Model</tt>.
         *
         * @see com.jgraph.pad.GraphModelProvider#getPresentationName()
         * @return String type, default Graph Model.
         */
	public String getPresentationName() {
		return "Default Graph Model";
	}

	/** Creates the Cells for the Default graph model.
         *
         * @see DefaultEdge
         * @see EllipseCell
         * @see ImageCell
         * @see DefaultPort
         * @see TextCell
         * @param model GraphModel type, the model.
         * @param type int type.
         * @param userObject Object type, the user object.
         * @param map Map type, the current map.
         * @return Object, returns type of userObject.
         */
	public Object createCell(GraphModel model, int type, Object userObject, Map map) {
		
		switch (type){
			case CELL_EDGE_DEFAULT :
				return new DefaultEdge(userObject);
			case CELL_VERTEX_ELLIPSE:
				return new EllipseCell(userObject);
			case CELL_VERTEX_IMAGE :
				return new ImageCell(userObject);
			case CELL_PORT_DEFAULT:
				return new DefaultPort(userObject);
			case CELL_VERTEX_TEXT:
				return new TextCell(userObject, true);
		}
		return new DefaultGraphCell(userObject);
	}



	/**
	 * Creates a clean default graph model.
	 * 
	 * @see DefaultGraphModel
	 * @see com.jgraph.pad.GraphModelProvider#createCleanGraphModel()
         * @return GraphModel type, the new default graph model.
	 */
	public GraphModel createCleanGraphModel() {
		return new DefaultGraphModel();
	}

	/** Currently, returns false every time.
	 * 
	 * @see com.jgraph.pad.GraphModelProvider#isMutateAbleTo(Class)
         * @param otherGraphModelClass Class type, the other model class.
         * @return boolean type, false.
	 */
	public boolean isMutateAbleTo(Class otherGraphModelClass) {
		return false;
	}

	/**
	 * Currently, returns null every time.
	 * 
	 * @see com.jgraph.pad.GraphModelProvider#mutateTo(GraphModel, Class)
         * @param sourceGraphModel GraphModel type, the source graph model.
         * @param otherGraphModelClass Class type, the other graph model class.
         * @return GraphModel type, returns null.
	 */
	public GraphModel mutateTo(
		GraphModel sourceGraphModel,
		Class otherGraphModelClass) {
		return null;
	}

	/**
	 * Returns the graph model file format objects for this provider.
	 * 
	 * @see DefaultGraphModelFileFormatXML
	 * @see DefaultGraphModelFileFormatSerial
	 * @see com.jgraph.pad.GraphModelProvider#getGraphModelFileFormats()
         * @return GraphModelFileFormat[] type, the graph model file format object for the current provider.
	 */
	public GraphModelFileFormat[] getGraphModelFileFormats() {
		return new GraphModelFileFormat[]{defaultGraphModelFileFormatXML, defaultGraphModelFileFormatSerial};
	}

	/**Adds a Port to the model specific vertex object
	 * 
	 * @see com.jgraph.pad.GraphModelProvider#addPort(Object, Object)
         * @param vertex Object type, the vertex.
         * @param port Object type, the port.
	 */
	public void addPort(Object vertex, Object port) {
		if (port instanceof DefaultPort){
			if (vertex instanceof DefaultMutableTreeNode){
				((DefaultMutableTreeNode)vertex).add((DefaultPort)port);
			}
		}
	}

	/**
	 * Creates a new clean graph for this model provider.
	 * 
	 * @see com.jgraph.pad.GraphModelProvider#createCleanGraph()
         * @param model GraphModel type, the model.
         * @return GPGraph type, a new GPgraph based on the model input.
	 */
	public GPGraph createCleanGraph(GraphModel model) {
		return new GPGraph(model);
	}

}
