import java.util.*;


/**
 * A simple implementation for a port.
 *
 * Referenced classes of package com.jgraph.graph are
 * DefaulGraphCell and Port.
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 */

public class DefaultPort extends DefaultGraphCell
  implements Port
{
  /** 
   * An instance of HashSet.
   * Edges that are connected to the port. 
   */
  protected HashSet edges;
  
  /** 
   * An instance of Port.
   * Reference to the anchor of this port. 
   */
  protected Port anchor;
  
  /**
   * Constructs an empty port.
   */
  public DefaultPort()
  {
    this(null, null);
  }
  
  /**
   * Constructs a vertex that holds a reference to the specified user object.
   * @param obj A varibale of Object data type.
   * @see Object
   */
  public DefaultPort(Object obj)
  {
    this(obj, null);
  }
  
  /**
   * Constructs a vertex that holds a reference to the specified user object
   * and a reference to the specified anchor.
   * @param obj A variable of Object data type.
   * @param port A variable of Port data type.
   * @see Object
   * @see Port
   */
  public DefaultPort(Object obj, Port port)
  {
    super(obj, false);
    edges = new HashSet();
    anchor = port;
  }
  
  /**
   * Returns an iterator of the edges connected
   * to the port.
   * @return An Iterator data type.
   * @see Iterator
   */
  public Iterator edges()
  {
    return edges.iterator();
  }
  
  /**
   * Adds edge to the list of ports.
   * @param obj A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public boolean addEdge(Object obj)
  {
    return edges.add(obj);
  }
  /**
   * Removes edge from the list of ports.
   * @param obj A variable of Object data type.
   * @return A boolean data type.
   * @see Object
   */
  public boolean removeEdge(Object obj)
  {
    return edges.remove(obj);
  }
  
  /**
   * Returns the anchor of this port.
   * @return A Set data type.
   * @see Set
   */
  public Set getEdges()
  {
    return new HashSet(edges);
  }
  
  /**
   * Sets the anchor of this port.
   * @param set A variable of Set data type.
   * @see Set
   */
  public void setEdges(Set set)
  {
    edges = new HashSet(set);
  }
  
  /**
   * Returns the anchor of this port.
   * @return A Port data type.
   * @see Port
   */
  public Port getAnchor()
  {
    return anchor;
  }
  
  /**
   * Sets the anchor of this port.
   * @param port A variable of Port data type.
   * @see Port
   */
  public void setAnchor(Port port)
  {
    anchor = port;
  }
  
  /**
   * Create a clone of the cell. The cloning of the
   * user object is deferred to the cloneUserObject()
   * method.
   * @return An Object data type.
   * @see Object
   */
  public Object clone()
  {
    DefaultPort defaultport = (DefaultPort)super.clone();
    defaultport.edges = new HashSet();
    return defaultport;
  }
}
