import java.io.Serializable;
import java.util.List;

/**
 * This interface provides the means of handling and tracking edge objects as used throughout the program.
 * The abstract functions provide common tools for interacting with Edges.
 *
 * Referenced classes of package com.jgraph.graph are
 * GraphCell and EdgeView.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 *
 * @see GraphCell
 */

public interface Edge
  extends GraphCell
  
{
  /** 
   * This interface handles the routing of Edge object data.
   * @author TerpPresent
   * @version 2.0 TerpPresent
   * @see Serializable
   */    
  public static interface Routing
    extends Serializable
  {
    
    /**
     * Adds the given edgeview to the list of routed edgeviews.
     * @param edgeview A variable of EdgeView data type.
     * @param list A variable of List data type.
     * @see EdgeView
     * @see List
     */        
    public abstract void route(EdgeView edgeview, List list);
  }
  
  /**
   * Retrieves to the calling member an object with the requisite Edge data.
   * @return An Object data type.
   * @see Object
   */    
  public abstract Object getSource();
  
  /**
   * Retrieves to the calling member an object representing the target Edge.
   * @return an Object data type.
   * @see Object
   */     
  public abstract Object getTarget();
  
  /**
   * Initializes the source data with an object representing an Edge.
   * @param obj A variable of Object data type.
   * @see Object
   */    
  public abstract void setSource(Object obj);
  
  /**
   * Initializes the target data with an object representing an Edge.
   * @param obj A variable of Object data type.
   * @see Object
   */    
  public abstract void setTarget(Object obj);
}
