
import java.awt.*;
//import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import java.io.Serializable;
import javax.swing.JComponent;
import javax.swing.UIManager;

/** 
 * A Public class EdgeRenderer extends JComponent to implement edge renderer.
 * This class takes the abstracted Edge data and provides additional tools 
 * for rendering the edges to screen.
 *
 * Referenced classes of package com.jgraph.graph are
 * EdgeView, CellViewRenderer, GraphConstants and CellView.
 *
 * @author TerpPresent
 * @version 2.0
 *
 * @see JComponent
 * @see CellViewRenderer
 * @see Serializable
 */

public class EdgeRenderer extends JComponent
    implements CellViewRenderer, Serializable
{

    /**
     * An instance of Graphics.
     * The current font graphic data.
     */    
    protected transient Graphics fontGraphics;
    
    /**
     * An instance of FontMetrics.
     * The current font metric data.
     */    
    protected transient FontMetrics metrics;
    
    /** 
     * An instance of JGraph.
     * The current jgraph data.
     */    
    protected transient JGraph graph;
    
    /**
     * An instance of EdgeView.
     * The current edgeview data.
     */    
    protected transient EdgeView view;
    
    /**
     * An instance of integer.
     * The value of the begining or initial point.
     */    
    protected transient int beginDeco;
    
    /**
     * An instance of integer.
     * The value of the ending or destination point.
     */    
    protected transient int endDeco;
    
    /**
     * An instance of integer.
     * The initial size of the object.
     */    
    protected transient int beginSize;
    
    /**
     * An instance of integer.
     * The final size of the Object.
     */    
    protected transient int endSize;
    
    /**
     * An instance of integer.
     * The value determining what style the line object is.
     */    
    protected transient int lineStyle;
    
    /**
     * An instance of float.
     * The width of the line object.
     */    
    protected transient float lineWidth;
    
    /**
     * An instance of boolean.
     * Flag for whether the border has labeled.
     */    
    protected transient boolean labelBorder;
    
    /**
     * An instance of boolean.
     * Flag for whether the object has been begun filling.
     */    
    protected transient boolean beginFill;
    
    /**
     * An instance of boolean.
     * Flag for whether the object has finished filling.
     */    
    protected transient boolean endFill;
    
    /**
     * An instance of boolean.
     * Flag for whether the object is currently focussed (selected).
     */    
    protected transient boolean focus;
    
    /**
     * An instance of boolean.
     * Flag for whether the object is currently selected.
     */    
    protected transient boolean selected;
    
    /**
     boolean
     * Flag for whether the object is being previewed.
     */    
    protected transient boolean preview;
    
    /**
     * An instance of boolean.
     * Flag for whether the object is opaque.
     */    
    protected transient boolean opaque;
    
    /**
     * An instance of Color.
     * The current color of the object's border.
     */    
    protected transient Color borderColor;
    
    /**
     * An instance of Color.
     * The default foreground color.
     */    
    protected transient Color defaultForeground;
    
    /**
     * An instance of Color.
     * The default background color.
     */    
    protected transient Color defaultBackground;
    
    /**
     * An instance of Color.
     * The current color of the font.
     */    
    protected transient Color fontColor;
    
    /**
     * An instance of Shape.
     * The starting state of the shape.
     */    
    protected transient Shape beginShape;
    
    /**
     * An instance of Shape.
     * The final state of the shape.
     */    
    protected transient Shape endShape;
    
    /**
     * An instance of Shape.
     * The shape of the line.
     */    
    protected transient Shape lineShape;
    
    /**
     * An instance of GeneralPath.
     * Indicator for sharing of like paths.
     */    
    protected transient GeneralPath sharedPath;
    
    /**
     * An instance of float.
     * The number of dashes for the line object.
     */    
    protected transient float lineDash[];

    /**
     * Main constructor for EdgeRenderer class.
     */    
    public EdgeRenderer()
    {
        fontGraphics = (new BufferedImage(1, 1, 1)).getGraphics();
        sharedPath = new GeneralPath(1);
        defaultForeground = UIManager.getColor("Tree.textForeground");
        defaultBackground = UIManager.getColor("Tree.textBackground");
    }

    /**
     * Obtains a compontent of the current rendering schema.
     * @param jgraph A variable of JGraph data type.
     * @param cellview A variable of CellView data type.
     * @param flag A variable of boolean data type.
     * @param flag1 A variable of boolean data type.
     * @param flag2 A variable of boolean data type.
     * @return A Component data type.
     * @see JGraph
     * @see CellView
     * @see Component
     */    
    public Component getRendererComponent(JGraph jgraph, CellView cellview, boolean flag, boolean flag1, boolean flag2)
    {
        if((cellview instanceof EdgeView) && jgraph != null && cellview != null)
        {
            view = (EdgeView)cellview;
            graph = jgraph;
            focus = flag1;
            selected = flag;
            preview = flag2;
            installAttributes(cellview);
            return this;
        } else
        {
            return null;
        }
    }

    /**
     * Finds out whether the region of the rectangle is in a state of intersection.
     * @param g A variable of Graphics data type.
     * @param cellview A variable of CellView data type.
     * @param rectangle A variable of Rectangle data type.
     * @return A boolean data type.
     * @see Graphics
     * @see CellView
     * @see Rectangle
     */    
    public boolean intersects(Graphics g, CellView cellview, Rectangle rectangle)
    {
        if((cellview instanceof EdgeView) && g != null && cellview != null)
        {
            view = (EdgeView)cellview;
            installAttributes(view);
            Graphics2D graphics2d = (Graphics2D)g;
            if(graphics2d.hit(rectangle, createShape(), true))
            {
                return true;
            }
            Rectangle rectangle1 = getLabelBounds(view);
            if(rectangle1 != null)
            {
                return rectangle1.intersects(rectangle);
            }
        }
        return false;
    }

    /**
     * Returns the region bounded by the current cellview.
     * @param cellview A variable of CellView data type.
     * @return A Rectangle data type.
     * @see CellView
     * @see Rectangle
     */    
    public Rectangle getBounds(CellView cellview)
    {
        if((cellview instanceof EdgeView) && cellview != null)
        {
            view = (EdgeView)cellview;
            installAttributes(view);
            Rectangle rectangle = getPaintBounds(view);
            Rectangle rectangle1 = getLabelBounds(view);
            if(rectangle1 != null)
            {
                rectangle = rectangle.union(rectangle1);
            }
            int i = (int)Math.ceil(lineWidth);
            rectangle.x = rectangle.x - i;
            rectangle.y = rectangle.y - i;
            rectangle.width = rectangle.width + 2 * i;
            rectangle.height = rectangle.height + 2 * i;
            return rectangle;
        } else
        {
            return null;
        }
    }
    
    /**
     * Returns the label's region bounded by the current edgeview.
     * @param edgeview A variable of EdgeView data type.
     * @return A Rectangle data type.
     * @see EdgeView
     * @see Rectangle
     */    
    public Rectangle getLabelBounds(EdgeView edgeview)
    {
        Point point = getLabelPosition(view);
        Dimension dimension = getLabelSize(view);
        if(point != null && dimension != null)
        {
            point.translate(-dimension.width / 2, -dimension.height / 2);
            return new Rectangle(point.x, point.y, dimension.width + 1, dimension.height + 1);
        } else
        {
            return null;
        }
    }

    /**
     * Returns the label's position within the current edgeview.
     * @param edgeview A variable of EdgeView data type.
     * @return A Point data type.
     * @see EdgeView
     * @see Point
     */    
    public Point getLabelPosition(EdgeView edgeview)
    {
        Rectangle rectangle = getPaintBounds(edgeview);
        Point point = edgeview.getLabelPosition();
        char c = '\u03E8';
        if(point != null && rectangle != null)
        {
            int i = rectangle.x + (rectangle.width * point.x) / c;
            int j = rectangle.y + (rectangle.height * point.y) / c;
            return new Point(i, j);
        } else
        {
            return null;
        }
    }

    /**
     * Returns the label's size within the current edgeview.
     * @param edgeview A variable of EdgeView data type.
     * @return A Dimension data type.
     * @see EdgeView
     * @see Dimension
     */    
    public Dimension getLabelSize(EdgeView edgeview)
    {
        String s = edgeview.getGraph().convertValueToString(edgeview);
        if(s != null && s.toString().length() > 0)
        {
            fontGraphics.setFont(GraphConstants.getFont(edgeview.getAllAttributes()));
            metrics = ((Graphics2D)fontGraphics).getFontMetrics();
            int i = metrics.stringWidth(s.toString());
            int j = metrics.getHeight();
            return new Dimension(i, j);
        } else
        {
            return null;
        }
    }

    /**
     * Sets the appropriate attributes into the current cellview.
     * @param cellview A variable of CellView data type.
     * @see CellView
     */    
    protected void installAttributes(CellView cellview)
    {
        java.util.Map map = cellview.getAllAttributes();
        beginDeco = GraphConstants.getLineBegin(map);
        beginSize = GraphConstants.getBeginSize(map);
        beginFill = GraphConstants.isBeginFill(map) && isFillable(beginDeco);
        endDeco = GraphConstants.getLineEnd(map);
        endSize = GraphConstants.getEndSize(map);
        endFill = GraphConstants.isEndFill(map) && isFillable(endDeco);
        lineWidth = GraphConstants.getLineWidth(map);
        lineStyle = GraphConstants.getLineStyle(map);
        lineDash = GraphConstants.getDashPattern(map);
        borderColor = GraphConstants.getBorderColor(map);
        Color color = GraphConstants.getLineColor(map);
        setForeground(color == null ? defaultForeground : color);
        Color color1 = GraphConstants.getBackground(map);
        setBackground(color1 == null ? defaultBackground : color1);
        setOpaque(GraphConstants.isOpaque(map));
        setFont(GraphConstants.getFont(map));
        Color color2 = GraphConstants.getForeground(map);
        fontColor = color2 == null ? getForeground() : color2;
        fontGraphics.setFont(getFont());
        metrics = ((Graphics2D)fontGraphics).getFontMetrics();
    }

    /**
     * Determines whether the current object is fillable.
     * @param i A variable of integer data type.
     * @return A boolean data type.
     */    
    protected boolean isFillable(int i)
    {
        return i != 4 && i != 7 && i != 8;
    }

    /**
     * Finds the bounds of the painted area within edgeview.
     * @param edgeview A variable of EdgeView data type.
     * @return A Rectangle data type.
     * @see EdgeView
     * @see Rectangle
     */    
    public Rectangle getPaintBounds(EdgeView edgeview)
    {
        view = edgeview;
        installAttributes(edgeview);
        return createShape().getBounds();
    }

    /**
     * Repaints the graphics plane.
     * @param g A variable of Graphics data type.
     * @see Graphics
     */    
    public void paint(Graphics g)
    {
        Shape shape = createShape();
        if(shape != null)
        {
            Graphics2D graphics2d = (Graphics2D)g;
            int i = 0;
            int j = 0;
            graphics2d.setStroke(new BasicStroke(lineWidth, i, j));
            g.translate(-getX(), -getY());
            g.setColor(getForeground());
            if(beginShape != null)
            {
                if(beginFill)
                {
                    graphics2d.fill(beginShape);
                }
                graphics2d.draw(beginShape);
            }
            if(endShape != null)
            {
                if(endFill)
                {
                    graphics2d.fill(endShape);
                }
                graphics2d.draw(endShape);
            }
            if(lineDash != null)
            {
                graphics2d.setStroke(new BasicStroke(lineWidth, i, j, 10F, lineDash, 0.0F));
            }
            graphics2d.draw(lineShape);
            if(selected)
            {
                graphics2d.setStroke(GraphConstants.SELECTION_STROKE);
                graphics2d.setColor(graph.getHighlightColor());
                if(beginShape != null)
                {
                    graphics2d.draw(beginShape);
                }
                graphics2d.draw(lineShape);
                if(endShape != null)
                {
                    graphics2d.draw(endShape);
                }
            }
            if(graph.getEditingCell() != view.getCell())
            {
                String s = graph.convertValueToString(view);
                if(s != null)
                {
                    graphics2d.setStroke(new BasicStroke(1.0F));
                    g.setFont(getFont());
                    paintLabel(g, s.toString());
                }
            }
        }
    }

    /**
     * Paints the label of the graphic.
     * @param g A variable of Graphics data type.
     * @param s A variable of String data type.
     * @see Graphics
     */    
    protected void paintLabel(Graphics g, String s)
    {
        Point point = getLabelPosition(view);
        if(point != null && s != null && s.length() > 0)
        {
            Graphics2D graphics2d = (Graphics2D)g;
            int i = metrics.stringWidth(s);
            int j = metrics.getHeight();
            if(isOpaque())
            {
                g.setColor(getBackground());
                g.fillRect(point.x - i / 2 - 1, point.y - j / 2 - 1, i + 2, j + 2);
            }
            if(borderColor != null)
            {
                g.setColor(borderColor);
                g.drawRect(point.x - i / 2 - 1, point.y - j / 2 - 1, i + 2, j + 2);
            }
            g.setColor(fontColor);
            ((Graphics2D)g).drawString(s, point.x - i / 2, point.y + j / 4);
        }
    }

    /**
     * Returns the newly created shape object.
     * @return A Shape data type.
     * @see Shape
     */    
    protected Shape createShape()
    {
        beginShape = lineShape = endShape = null;
        int i = view.getPointCount();
        if(i > 1)
        {
            EdgeView edgeview = view;
            Point apoint[] = new Point[i];
            for(int j = 0; j < i; j++)
            {
                apoint[j] = new Point(edgeview.getPoint(j));
            }

            if(view != edgeview)
            {
                view = edgeview;
                installAttributes(view);
            }
            sharedPath.reset();
            beginShape = lineShape = endShape = null;
            Point point = apoint[0];
            Point point1 = apoint[i - 1];
            Point point2 = apoint[1];
            Point point3 = apoint[i - 2];
            beginShape = view.beginShape;
            if(beginDeco != 0)
            {
                beginShape = createLineEnd(beginSize, beginDeco, point2, point);
                view.beginShape = beginShape;
            }
            endShape = view.endShape;
            if(endDeco != 0)
            {
                endShape = createLineEnd(endSize, endDeco, point3, point1);
                view.endShape = endShape;
            }
            if(view.sharedPath == null)
            {
                sharedPath.moveTo(point.x, point.y);
                if(lineStyle == 12 && i > 2)
                {
                    sharedPath.quadTo(point2.x, point2.y, point1.x, point1.y);
                } else
                if(lineStyle == 13 && i > 3)
                {
                    sharedPath.curveTo(point2.x, point2.y, point3.x, point3.y, point1.x, point1.y);
                } else
                {
                    for(int k = 1; k < i - 1; k++)
                    {
                        sharedPath.lineTo(apoint[k].x, apoint[k].y);
                    }

                    sharedPath.lineTo(point1.x, point1.y);
                }
                lineShape = (GeneralPath)sharedPath.clone();
                if(endShape != null)
                {
                    sharedPath.append(endShape, true);
                }
                if(beginShape != null)
                {
                    sharedPath.append(beginShape, true);
                }
                view.sharedPath = (GeneralPath)sharedPath.clone();
                view.lineShape = lineShape;
            } else
            {
                sharedPath = (GeneralPath)view.sharedPath.clone();
                lineShape = view.lineShape;
            }
            return sharedPath;
        } else
        {
            return null;
        }
    }

    /**
     * Creates a full line object.
     * @param i A variable of integer data type.
     * @param j A variable of integer data type.
     * @param point A variable of Point data type.
     * @param point1 A variable of Point data type.
     * @return A Shape data type.
     * @see Point
     * @see Shape
     */    
    protected Shape createLineEnd(int i, int j, Point point, Point point1)
    {
        int k = (int)Math.max(1.0D, point1.distance(point));
        int l = -((i * (point1.x - point.x)) / k);
        int i1 = -((i * (point1.y - point.y)) / k);
        if(j == 9)
        {
            Polygon polygon = new Polygon();
            polygon.addPoint(point1.x, point1.y);
            polygon.addPoint(point1.x + l / 2 + i1 / 3, (point1.y + i1 / 2) - l / 3);
            Point point2 = new Point(point1);
            point1.setLocation(point1.x + l, point1.y + i1);
            polygon.addPoint(point1.x, point1.y);
            polygon.addPoint((point2.x + l / 2) - i1 / 3, point2.y + i1 / 2 + l / 3);
            return polygon;
        }
        if(j == 2 || j == 1)
        {
            Polygon polygon1 = new Polygon();
            polygon1.addPoint(point1.x, point1.y);
            polygon1.addPoint(point1.x + l + i1 / 2, (point1.y + i1) - l / 2);
            Point point3 = new Point(point1);
            if(j == 1)
            {
                point1.setLocation(point1.x + (l * 2) / 3, point1.y + (i1 * 2) / 3);
                polygon1.addPoint(point1.x, point1.y);
            } else
            if(j == 9)
            {
                point1.setLocation(point1.x + 2 * l, point1.y + 2 * i1);
                polygon1.addPoint(point1.x, point1.y);
            } else
            {
                point1.setLocation(point1.x + l, point1.y + i1);
            }
            polygon1.addPoint((point3.x + l) - i1 / 2, point3.y + i1 + l / 2);
            return polygon1;
        }
        if(j == 4)
        {
            GeneralPath generalpath = new GeneralPath(1, 4);
            generalpath.moveTo(point1.x + l + i1 / 2, (point1.y + i1) - l / 2);
            generalpath.lineTo(point1.x, point1.y);
            generalpath.lineTo((point1.x + l) - i1 / 2, point1.y + i1 + l / 2);
            return generalpath;
        }
        if(j == 5)
        {
            java.awt.geom.Ellipse2D.Float float1 = new java.awt.geom.Ellipse2D.Float((point1.x + l / 2) - i / 2, (point1.y + i1 / 2) - i / 2, i, i);
            point1.setLocation(point1.x + l, point1.y + i1);
            return float1;
        }
        if(j == 7 || j == 8)
        {
            GeneralPath generalpath1 = new GeneralPath(1, 4);
            generalpath1.moveTo(point1.x + l / 2 + i1 / 2, (point1.y + i1 / 2) - l / 2);
            generalpath1.lineTo((point1.x + l / 2) - i1 / 2, point1.y + i1 / 2 + l / 2);
            if(j == 8)
            {
                generalpath1.moveTo(point1.x + l / 3 + i1 / 2, (point1.y + i1 / 3) - l / 2);
                generalpath1.lineTo((point1.x + l / 3) - i1 / 2, point1.y + i1 / 3 + l / 2);
            }
            return generalpath1;
        } else
        {
            return null;
        }
    }

    /**
     * Validates an Object.
     */    
    public void validate()
    {
    }

    /**
     * Revalidates an Object.
     */    
    public void revalidate()
    {
    }

    /**
     * Repaints an Object.
     * @param l A variable of long data type.
     * @param i A variable of integer data type.
     * @param j A variable of integer data type.
     * @param k A variable of integer data type.
     * @param i1 A variable of integer data type.
     */    
    public void repaint(long l, int i, int j, int k, int i1)
    {
    }

    /**
     * Repaints an Objet
     * @param rectangle A variable of Rectangle data type.
     * @see Rectangle
     */    
    public void repaint(Rectangle rectangle)
    {
    }

    /**
     * Initiates a property change.
     * @param s A variable of String data type.
     * @param obj A variable of Object data type.
     * @param obj1 A variable of Object data type.
     * @see Object
     */    
    protected void firePropertyChange(String s, Object obj, Object obj1)
    {
        if(s == "text")
        {
            super.firePropertyChange(s, obj, obj1);
        }
    }

    /**
     * Changes the fire property of an Object.
     * @param s A variable of String data type.
     * @param byte0 A variable of byte data type.
     * @param byte1 A variable of byte data type.
     */   
    public void firePropertyChange(String s, byte byte0, byte byte1)
    {
    }

    /**
     * Changes the fire property of an Object.
     * @param s A variable of String data type.
     * @param c A variable of char data type.
     * @param c1 A variable of char data type.
     */    
    public void firePropertyChange(String s, char c, char c1)
    {
    }

    /**
     * Changes the fire property of an Object.
     * @param s A variable of String data type.
     * @param word0 A variable of short data type.
     * @param word1 A variable of short data type.
     */     
    public void firePropertyChange(String s, short word0, short word1)
    {
    }

    /**
     * Changes the fire property of an Object.
     * @param s A variable of String data type.
     * @param i A variable of integer data type.
     * @param j A variable of integer data type.
     */    
    public void firePropertyChange(String s, int i, int j)
    {
    }

    /**
     * Changes the fire property of an Object.
     * @param s A variable of String data type.
     * @param l A variable of long data type.
     * @param l1 A variable of long data type.
     */     
    public void firePropertyChange(String s, long l, long l1)
    {
    }

    /**
     * Changes the fire property of an Object.
     * @param s A variable of String data type.
     * @param f A variable of float data type.
     * @param f1 A variable of float data type.
     */     
    public void firePropertyChange(String s, float f, float f1)
    {
    }

    /**
     * Changes the fire property of an Object. 
     * @param s A variable of String data type.
     * @param d A variable of double data type.
     * @param d1 A variable of double data type.
     */     
    public void firePropertyChange(String s, double d, double d1)
    {
    }

    /**
     * Changes the fire property of an Object.
     * @param s A variable of String data type.
     * @param flag A variable of boolean data type.
     * @param flag1 A variable of boolean data type.
     */     
    public void firePropertyChange(String s, boolean flag, boolean flag1)
    {
    }
}
