/*
 * @(#)EditCut.java 1.2 02.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.event.ActionEvent;

import java.awt.*;
import java.awt.image.*;
import java.awt.datatransfer.*;
import javax.swing.Icon; 

/** Action that cuts a cell (placing it into the buffer to be copied).
 *
 * @author sven.luzar
 * @version 3.0 TerpPresent
 */
public class EditCut extends AbstractActionDefault implements ClipboardOwner
{
 public static GPGraphpad mygp;

 public EditCut (){}
 /**
  * Constructor for EditCut.
  * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
  */
 public EditCut(GPGraphpad graphpad) {
  super(graphpad);
  mygp = graphpad;
 }

 /**
  * Constructor for EditCut.
  * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
  * @param name This is the name of the action, and is 
  * sent to super().
  */
 public EditCut(GPGraphpad graphpad, String name) {
  super(graphpad, name);
  mygp = graphpad;
 }

 /**
  * Constructor for EditCut.
  * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
  * @param name This is the name of the action, and is
  * sent to super().
  * @param icon The icon for the action, and is sent to super().
  */
 public EditCut(GPGraphpad graphpad, String name, Icon icon) {
  super(graphpad, name, icon);
  mygp = graphpad;
 }

 /** Performs the cut action.
  * 
  * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
  * @param e The action to be performed.
  * @author Hari Sivaramakrishnan, Modified March 25th 2004.
  */
 public void actionPerformed(ActionEvent e) 
 { 
   CopyToClipBoard(true);
   
   // This is the default cut action implemented by the JGraphpad guys
   // TransferHandler.getCutAction().actionPerformed(
   //		new ActionEvent(getCurrentGraph(), e.getID() , e.getActionCommand() ));
   
 }
  
 /**
  * Cutting the contents of the cell places them in the system clipboard.
  * @param EditCut This is true if the method was called because the action was
  * Edit -> Cut. If itis false, then this method was called from the Ctrl+X action.
  * @see DefaultGraphModel#remove
  * @author Hari Sivaramakrishnan, Modified March 25th 2004.
  */ 
 public static void CopyToClipBoard(boolean EditCut)
 {
 	boolean stringFlavor = false ;
 	
 	Object[] cells = mygp.getCurrentGraph().getSelectionCells();
 	
 	for (int i = 0; i <cells.length; i++)
 		if ( cells [i] instanceof TextCell)
 		{
 			mygp.clip_obj = new UMClipObj (((TextCell)cells[i]).getUserObject().toString(), "") ;
 			stringFlavor = true ;
 			break ;
 		}
 	
 		if (!stringFlavor)
 		{	
 			BufferedImage copy_image = (BufferedImage)(GPConverter.toImage(mygp.getCurrentGraph(), cells)) ;
 			mygp.clip_obj = new UMClipObj(copy_image, "TerpPresent");
 		}
 	
 	if (cells != null) 
 	{
 		cells =DefaultGraphModel.getDescendants(mygp.getCurrentGraph().getModel(), cells).toArray();
 		if (EditCut)
 			mygp.getCurrentGraph().getModel().remove(cells);
 	}
 	try 
	{
 		Toolkit.getDefaultToolkit().getSystemClipboard().setContents( mygp.clip_obj, new EditCut ());
 	}
 	catch (Exception e)
	{
 		System.out.println ("Copy to clipboard has failed") ;
 	}
 }
 
 /* If the cut could not be made.
  * @param clipboard The clipboard Object.
  * @param transferable A Transferable Object.
  */ 
 public void lostOwnership(Clipboard clipboard, Transferable transferable)
 {
   System.out.println ("Lost Clipboard ownership");
 }
}

