/*
 * @(#)EditPaste.java 1.2 02.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

import java.awt.image.*;
import java.awt.datatransfer.*;
import java.awt.event.ActionEvent;
import java.awt.*;
import javax.swing.* ;
import java.util.Map;

import javax.swing.Icon;

/** An action for pasting selections into a cell.
 * 
 * @author sven.luzar
 * @version 2.0
 *
 */
public class EditPaste extends AbstractActionDefault
{  
  /** A reference to a graphpad for pasting.
   * */
  public static GPGraphpad mygp;
  
  public EditPaste (){}
  
  /**
   * Constructor for EditPaste.
   * @param graphpad This is a reference to the graphpad for this action,
   * and is sent to super().
   */
  public EditPaste(GPGraphpad graphpad) {
    super(graphpad);
    mygp = graphpad;
    //System.out.println("EditPaste constructor 2 " ) ;
  }
  
  /**
   * Constructor for EditPaste.
   * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
   * @param name The name of the action, and is sent to super().
   */
  public EditPaste(GPGraphpad graphpad, String name) {
    super(graphpad, name);
    mygp = graphpad;
  }
  
  /**
   * Constructor for EditPaste.
   * @param graphpad This is a reference to the graphpad for this action,
  * and is sent to super().
   * @param name The name of the action, and is 
   * sent to super().
   * @param icon The icon for the action, and is sent to super().
   */
  public EditPaste(GPGraphpad graphpad, String name, Icon icon) {
    super(graphpad, name, icon);
    mygp = graphpad;
  }
  
  /** Performs the action.
   * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
   * @param e The action to be performed.
   * @author Hari Sivaramakrishnan, Modified 26th March 2004
   */
  public void actionPerformed(ActionEvent e) 
	{
    if (!Paste ())
    {
    	// The default paste handler for the application.
    	// Update, this will almost never be called.
    	TransferHandler.getPasteAction().actionPerformed(
    			new ActionEvent(getCurrentGraph(), e.getID() , e.getActionCommand() ));
    }
  }
  
  /** 
   * If the clipboard has a BuffedImage object, paste it.
   * 
   * @return True if a paste was possible, false otherwise.
   * @author Hari Sivaramkrishnan, 
   * @version 3.0 TerpPresent
   */
  public static boolean Paste ()
  { 
    BufferedImage fromClip = null; 
    Transferable transferable = null; 
    Object clipboardContents = null; 
    String application = "" ;
    
    Clipboard clipboard;
    clipboard=Toolkit.getDefaultToolkit().getSystemClipboard(); 
    if (clipboard == null)
    { 
      System.out.println("Clipboard is null"); 
      return false ; 
    }
    
    try 
		{ 
      clipboardContents = clipboard.getContents(new EditPaste ()); 
      
      if (clipboardContents != null)
      {   
        transferable = clipboard.getContents(new EditPaste ()); 
        
        /* Determine the application that is put this in the clipboard */
//        try
//				{
//        	application = (String) transferable.getTransferData(DataFlavor.plainTextFlavor) ;
//        }
//        catch (Exception e) // The stuff on clipboard was not copied by a TerpOffice application.
//				{   }
        
        /* If it is an image (JPG), perform the paste with this code, otherwise
         * return false and the alternate paste (in the actionPerformed method) will be used.
         * @author Hari Sivaramakrishnan, Modified 26th March, 2004
         */
        if (transferable.isDataFlavorSupported(DataFlavor.imageFlavor))
        { 
        	/*JPG from the clipboard which has to be pasted */
          fromClip = (BufferedImage) (transferable.getTransferData( (DataFlavor) DataFlavor.imageFlavor));   
          
          /* Create a rectangle inside which the image will be pasted. Once
           * this rectangle is created, it is selected by default.
           * The userObject parameter is used to store which object this came from.
           */
          if (fromClip != null ) // actually have an image in the clipboard
          {
          	mygp.getMarqueeHandler().addVertex(DefaultGraphModelProvider.CELL_VERTEX_DEFAULT, application , new Rectangle (0,0,fromClip.getWidth()+2,fromClip.getHeight()+2),false, Color.WHITE) ;
          
          	/* Create the necessary features for the image to be inserted */
          	Toolkit toolkit = Toolkit.getDefaultToolkit();
          	ImageIcon icon = new ImageIcon(fromClip);
          	Map map = GraphConstants.createMap();
          	GraphConstants.setIcon(map, icon);
          	
          	/* Paste the image onto the application, inside the rectangle that was just created */
          	mygp.getCurrentDocument().setSelectionAttributes (map) ;
          	mygp.getCurrentGraph().repaint();
          } 
        }
         
        if ( transferable.isDataFlavorSupported(DataFlavor.stringFlavor) && fromClip == null )
        {
        	/*text from the clipboard which has to be pasted */
        	String text = (String) (transferable.getTransferData( (DataFlavor) DataFlavor.stringFlavor));   
        	
        	/* Create a rectangle inside which the text will be pasted. Once
        	 * this rectangle is created, it is selected by default.
        	 */
        	//mygp.getMarqueeHandler().addVertex(TPGraphModelProvider.CELL_VERTEX_TEXT, "",new Rectangle (0,0,text.length(),10),false, Color.WHITE) ;
        	TPGraphModelProvider tmp = (TPGraphModelProvider) mygp.getCurrentDocument().getGraphModelProvider();
        	
        	/* Create the necessary features for the text to be inserted */
        	//Toolkit toolkit = Toolkit.getDefaultToolkit();
        	//ImageIcon icon = new ImageIcon(fromClip);
        	Map map = GraphConstants.createMap();
        	Object cell = tmp.createCell(mygp.getCurrentDocument().getModel(), TPGraphModelProvider.CELL_VERTEX_TEXT, text, map ) ;
        	
        	// GraphConstants.setIcon(map, icon);
        	Object[] insert = new Object[] { cell };
        	mygp.getCurrentGraph().getModel().insert(insert, map ,null, null, null);
        	
        	/* Paste the image onto the application, inside the rectangle that was just created */
        	mygp.getCurrentDocument().setSelectionAttributes (map) ;
        	mygp.getCurrentGraph().repaint();
        }
        else /* Clipboard does not have an image or text object */
        {	
        	System.out.println("Clipboard data is unknown type"); 
        	return false ;
        }
      }
      else
      	return false ; /* There is nothing in the clipboard */
    } 
    catch(Exception e)
		{ 
      System.out.println("Error: " + e.getMessage());
      e.printStackTrace() ;
    } 
    
    return true ; /* No issues, paste done */
  }
}
