/*
 * @(#)EditUndo.java 1.2 30.01.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JMenuItem;
import javax.swing.undo.CannotUndoException;


/** An action for undoing the most recently performed action.
 * 
 * @author sven.luzar
 * @version 2.0
 *
 */
public class EditUndo extends AbstractActionDefault {
  
  /** A Vector of menuItems that will be used for repeating the action. */
  protected Vector menuItems = new Vector();
  
  /**
   * Constructor for EditUndo.
   * @param graphpad This is a reference to the graphpad for this action,
   * and is sent to super().
   */
  public EditUndo(GPGraphpad graphpad) {
    super(graphpad);
    setEnabled(false);
  }
  
  /** Performs an action.
   * @param e The action to be performed.
   * */
  public void actionPerformed(ActionEvent e) {
    try {
      getCurrentDocument().getGraphUndoManager().undo(
                                                      graphpad.getCurrentGraph().getGraphLayoutCache());
    } catch (CannotUndoException ex) {
      System.out.println("Unable to graphUndoManager: " + ex);
      ex.printStackTrace();
    }
    graphpad.update();
  }
  
  /** Updates the items of the cell once the action has been performed.
   */ 
  public void update() {
    Enumeration enum = menuItems.elements();
    
    while (enum.hasMoreElements()) {
      JMenuItem item = (JMenuItem) enum.nextElement();
      if (getCurrentDocument() != null &&
          getCurrentDocument().getGraphUndoManager().canUndo(getCurrentGraphLayoutCache())) {
        setEnabled(true);
        item.setText(
                     getCurrentDocument().getGraphUndoManager().getUndoPresentationName());
      } else {
        setEnabled(false);
        item.setText(Translator.getString("EditUndoLabel"));
      }
    }
  }
  
  /** Gets the JMenuItem for the cell, indicated by <tt>actionCommand</tt>.
   * 
   * @see com.jgraph.pad.actions.AbstractActionDefault#getMenuComponent(String)
   * @param actionCommand Indicates which item to perform the action on.
   */
  protected Component getMenuComponent(String actionCommand) {
    JMenuItem item = new JMenuItem(this);
    
    GPBarFactory.fillAbstractButton(item, getName(), actionCommand);
    
    menuItems.add(item);
    return item;
  }
  
}
