import java.awt.event.ActionEvent;
import java.io.ObjectOutputStream;
import java.io.Serializable;


/**
 * Action that saves the library to a file at specified location.
 * 
 * @author sven.luzar
 * @version 2.0
 */
public class FileLibrarySaveAs extends AbstractActionFile {

	/** Constructs a GPGraphpad from the graphpad of current document.
         * @param graphpad the graphpad to be edited
         */
	public FileLibrarySaveAs(GPGraphpad graphpad) {
		super(graphpad);
	}

	/** Saves the library at the location specified by the user. Location must be valid.
         * @see com.jgraph.pad.actions.AbstractActionFile#saveDialog(String,String,String) saveDialog
         * @param e the event trigger
         */
	public void actionPerformed(ActionEvent e) {
		String libraryExtension = Translator.getString("LibraryExtension");

		String name =
			saveDialog(
				Translator.getString("FileSaveAsLabel"),
				libraryExtension,
				"JGraphpad Library (*." + libraryExtension + ")");
		if (name != null) {
			GPLibraryPanel.ScrollablePanel panel =
				getCurrentDocument().getLibraryPanel().getPanel();
			if (panel != null) {
				Serializable s = panel.getArchiveableState();
				try {
					ObjectOutputStream out = createOutputStream(name);
					out.writeObject(s);
					out.flush();
					out.close();
				} catch (Exception ex) {
					graphpad.error(ex.toString());
				} finally {
					graphpad.invalidate();
				}
			}
		}
	}

}
