import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;


/** 
* Action saves the current graph to the current opened file
*
* @author sven.luzar
* @version 3.0
*
*/
public class FileSave extends AbstractActionFile 
{
	
	/**
	 * Constructs a GPGraphpad for the current graphpad.
	 * @param graphpad the graphpad to be edited
	 * 
	 */
	public FileSave(GPGraphpad graphpad) {
		super(graphpad);
	}
	
	/** Saves the graph to the current opened file.
	 *  The graph must be in the corrected format and graph model.
	 *	@see GraphModelProviderRegistry#getGraphModelFileFormat(File) getGraphFileFormat
	 *  @param e the event trigger
	 */
	public void actionPerformed(ActionEvent e) 
	{
		//(new Exception()).printStackTrace() ;
		GPDocument doc = getCurrentDocument();
		String filename = doc.getFilename();
		GPGraph gpGraph = doc.getGraph();
		GraphModel model = gpGraph.getModel();
		JComponent accessory = null;
		
		File file;
		if (filename == null) 
		{
			GPFileChooser chooser ;
			FileFilter fileFilter ;
			
			// show the file chooser
			chooser = new GPFileChooser(doc);
			chooser.setDialogTitle(Translator.getString("FileSaveAsLabel"));
			
			int result = chooser.showSaveDialog(graphpad);
			
			if (result == JFileChooser.CANCEL_OPTION)
			{
				// Just set this to a String object, so that the calling method GpDocument.Close will know
				// that the user has chosen the cancel option.
				e .setSource(new String ("Source")) ;
				return;
			}
			
			// get the file format
			fileFilter = chooser.getFileFilter();
			file = chooser.getSelectedFile();

			
			/* Allows for just saving files with no extension.
			 * If the filename contains one of the extensions that are accepted by 
			 * TerpPresent, that extension is used, otherwise the graph is stored in XML format.
			 */
			if ( file.getName().startsWith("\"") && file.getName().endsWith("\""))
			{ 
				/* Creates a new file with the name of the file having the full path 
				 * file.getParent() returns the parent directory
				 * file.separator is the separator character, / in unix, \ in windows
				 */
				file = new File ( file.getParent () + File.separator + file.getName().substring(1, file.getName().length() - 1)) ;
				
				// User has specified the .pad and .pad_xml formats by himself, just remove the enclosing ""
				if ( ! (file.getName().endsWith(".tpb") ))
				{
					/* Use the xml file format for storing all the information */
					try 
					{
						GraphModelFileFormat graphModelFileFormat =
							GraphModelProviderRegistry.getGraphModelFileFormat("foo.tpb");
						
						/*TPDocument tpDoc = (TPDocument)doc;
						SortedMap gpDocMap = tpDoc.getGPDocumentMap();
						
						List gpGraphList = new LinkedList();
						for( Iterator iter = gpDocMap.values().iterator(); iter.hasNext() ; ) {
							GPDocument d = (GPDocument)iter.next();
							gpGraphList.add( d.getGraph() );
						}*/

                
                                                TPDocument tpDoc = (TPDocument)doc;
                                                DefaultListModel listModel = tpDoc.getListModel();
                                                List gpGraphList = new LinkedList();
                                                for( int j = 0 ; j < listModel.getSize(); j++ ) {
                                                    GPDocument d = (GPDocument)listModel.elementAt( j );
                                                    gpGraphList.add( d.getGraph() );
                                                }

                                                
						((TPGraphModelFileFormatXML)graphModelFileFormat).write(file.getAbsolutePath(),gpGraph.getWriteProperties(),gpGraphList,gpGraph.getModel()) ;
						doc.setFilename(file.getAbsolutePath());
						doc.setModified(false);
					} 
					catch (Exception ex) 
					{
						JOptionPane.showMessageDialog(graphpad,ex.getLocalizedMessage(),Translator.getString("Error"),
								JOptionPane.ERROR_MESSAGE);
					} 
						return ;
				}
			}
			
			GraphModelFileFormat graphModelFileFormat =
				GraphModelProviderRegistry.getGraphModelFileFormat(file);
			
			// if no file format was found try to specify the
			// file format by using the file filter
			if (graphModelFileFormat == null)
			{
				graphModelFileFormat =
					GraphModelProviderRegistry.getGraphModelFileFormat(fileFilter);
				file = new File(file.getAbsolutePath() + "." + graphModelFileFormat.getFileExtension());
			}
			
			// sets the writeProperties
			// for the next write process
			gpGraph.setWriteProperties(graphModelFileFormat.getWriteProperties(chooser.getAccessory()));
			
			// memorize the filename
			filename = file.getAbsolutePath() ;
		} 
		else 
			file = new File(filename); // extract the file object
		
		
		// exract the provider and the file format
		GraphModelProvider graphModelProvider = GraphModelProviderRegistry.getGraphModelProvider(file);
		GraphModelFileFormat graphModelFileFormat = GraphModelProviderRegistry.getGraphModelFileFormat(file);
		
		// extract the writer properties
		Hashtable props = gpGraph.getWriteProperties();
		
		// try to write the file
		try 
		{
			TPDocument tpDoc = (TPDocument)doc;
			//SortedMap gpDocMap = tpDoc.getGPDocumentMap();

                        DefaultListModel listModel = tpDoc.getListModel();
			List gpGraphList = new LinkedList();
			//for( Iterator iter = gpDocMap.values().iterator(); iter.hasNext() ; ) {
			for( int j = 0 ; j < listModel.getSize(); j++ ) {
                            //GPDocument d = (GPDocument)iter.next();
                            GPDocument d = (GPDocument)listModel.elementAt( j );
                            gpGraphList.add( d.getGraph() );
			}
			
			if( graphModelFileFormat instanceof TPGraphModelFileFormatSerial ) {
				((TPGraphModelFileFormatSerial)graphModelFileFormat).write(
						file.getAbsolutePath(),props, gpGraphList,gpGraph.getModel());
			} else if( graphModelFileFormat instanceof TPGraphModelFileFormatXML ) {
				((TPGraphModelFileFormatXML)graphModelFileFormat).write(
						file.getAbsolutePath(),props, gpGraphList,gpGraph.getModel());
			} else {
				System.err.println( "This format is currently not supported." );
				throw new Exception( "Invalid GraphModelFileFormat used." );
			}
			
			//		  graphModelFileFormat.write(file.getAbsolutePath(),props,gpGraph,gpGraph.getModel());
			doc.setFilename(filename);
			doc.setModified(false);
		} 
		catch (Exception ex) 
		{
			ex.printStackTrace() ;
			JOptionPane.showMessageDialog(graphpad,ex.getLocalizedMessage(),Translator.getString("Error"),
					JOptionPane.ERROR_MESSAGE);
		} 
		finally 
		{
			//*****begin update recent files list added 04/05/2003*****
			File recentFiles;
			
			try
			{
				String[] oldRecentList = new String[4];
				String[] newRecentList = new String[4];
				
				String temp = null;
				
				recentFiles = new File("RecentFiles.txt");
				
				BufferedReader recentFilesReader = new BufferedReader(new FileReader(recentFiles));
				
				//read from the old recent files list
				int oldCount = 0;
				
				temp = recentFilesReader.readLine();
				
				while (temp != null && oldCount < 4)
				{
					oldRecentList[oldCount] = temp.trim();
					temp = recentFilesReader.readLine();
					oldCount++;
				}
				
				for (int x = 0; x < oldCount; x++)
					System.out.println(oldRecentList[x]);
				
				recentFilesReader.close();
				
				//update new recent files list
				newRecentList[0] = filename;
				int counter = 1;
				
				for (int x = 0; x < oldCount && counter < 4; x++)
				{
					if (!oldRecentList[x].equals(filename))
					{
						newRecentList[counter] = oldRecentList[x];
						counter++;
					}
				}
				
				//clear the file and rewrite
				recentFiles = new File("RecentFiles.txt");
				
				BufferedWriter recentFilesWriter = new BufferedWriter(new FileWriter(recentFiles));
				//recentFilesWriter.flush();
				
				for (int x = 0; x < counter; x++)
				{
					recentFilesWriter.write(newRecentList[x], 0, newRecentList[x].length());
					recentFilesWriter.newLine();
				}
				
				recentFilesWriter.close();
			}
			catch(Exception myE)
			{
				System.out.println("Error");
			}
			//*****end update recent files list added 04/05/2003*****
			
			
			graphpad.update();
			graphpad.invalidate();
		}
		
	}
	
}
