import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;

import javax.swing.ImageIcon;


/**
 * Action opens a file dialog to select an image.
 * Afterthat, the Action applies the selected image 
 * to the graph background.
 * 
 * @author sven.luzar
 *
 */
public class FormatBackgroundImage extends AbstractActionFile {

	/** Constructor for GraphBackgroundImage.
         * This constructor is called from GPGraphPad.java.
         * @param graphpad the current state of of the graph
         */
	public FormatBackgroundImage(GPGraphpad graphpad) {
		super(graphpad);
	}

        /** Action opens a file dialog to select an image when an action event is initiated.
         *  The Action applies the selected image to the graph background.
         *
         * @param e event trigger when menu-item is clicked or action is performed
         */        
	public void actionPerformed(ActionEvent e) {
		String name =
			openDialog(Translator.getString("ImageDialog"), null, null);
		if (name != null) {
			try {
				// JDK 1.3
				//File f = new File(name);
				//Image value = new ImageIcon(ImageIO.read(f)).getImage();
				Toolkit toolkit = Toolkit.getDefaultToolkit();
				ImageIcon icon = new ImageIcon(toolkit.getImage(name));
				Image value = toolkit.getImage(name);
				graphpad.getCurrentGraph().setBackgroundImage(value);
				graphpad.getCurrentGraph().repaint();
			} catch (Exception ex) {
				graphpad.error(ex.toString());
			}
		}
	}

}
