/*
 * @(#)FormatBorderColorList.java	1.2 04.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.Color;
import java.awt.Rectangle;
import java.util.Map;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JColorChooser;
import javax.swing.border.Border;

/**
 * Presents a list of colors for selection and fills the border with selected color.
 * @author sven.luzar
 * @version 1.0
 *
 */
public class FormatBorderColorList extends AbstractActionListCellColor {
	/** Borders of the current graph.
	 */
	protected Border[] borders =
		new Border[] {
			BorderFactory.createRaisedBevelBorder(),
			BorderFactory.createLoweredBevelBorder(),
			BorderFactory.createEtchedBorder()};

	/**
	 * Constructor for FormatBorderColorList.
	 * This constructor is called from GPGraphPad.java.
	 * @param graphpad the current state of the graph
	 */
	public FormatBorderColorList(GPGraphpad graphpad) {
		super(graphpad);
	}

	/** Fills table with the colors for selection.
	 * @see com.jgraph.pad.actions.AbstractActionListCell#fillCustomItems(Vector)
	 * @param items vector of colors
	 */
	protected void fillCustomItems(Vector items) {
		VertexView v;
		Map map;

		for (int i = 0; i < borders.length; i++) {
			v =
				new VertexView(
					null,
					dummyGraph,
					dummyGraph.getGraphLayoutCache());
			map = GraphConstants.createMap();
			GraphConstants.setBounds(map, new Rectangle(point, size));
			GraphConstants.setBorder(map, borders[i]);
			GraphConstants.setRemoveAttributes(
				v.getAttributes(),
				new Object[] { GraphConstants.BORDER });
			v.setAttributes(map);
			items.add(v);
		}

		for (int i = 0; i < colors.length; i++) {
			v =
				new VertexView(
					null,
					dummyGraph,
					dummyGraph.getGraphLayoutCache());
			map = GraphConstants.createMap();
			GraphConstants.setBounds(map, new Rectangle(point, size));
			GraphConstants.setBorderColor(map, colors[i]);
			v.setAttributes(map);
			GraphConstants.setRemoveAttributes(
				v.getAttributes(),
				new Object[] { GraphConstants.BORDER });
			items.add(v);
		}
	}

	/** Fills the border with the default color which is black.
	 * @see com.jgraph.pad.actions.AbstractActionListCell#fillResetMap(Map)
	 * @param target the map which its border is being filled
	 */
	protected void fillResetMap(Map target) {
		Object[] keys =
			new Object[] { GraphConstants.BORDER, GraphConstants.BORDERCOLOR };
		GraphConstants.setRemoveAttributes(target, keys);
		GraphConstants.setBorderColor(target, Color.BLACK );
	}

	/** Fills the border of the selected map with the selected color.
	 * @see com.jgraph.pad.actions.AbstractActionListCell#fillApplyMap(CellView, Map)
	 * @param source the cell that being edited
	 * @param target the map contains the cell that being edited
	 */
	protected void fillApplyMap(CellView source, Map target) {
		Object[] keys = new Object[] { GraphConstants.BORDER, GraphConstants.BORDERCOLOR };
		GraphConstants.setRemoveAttributes(target, keys);
		Color c = GraphConstants.getBorderColor(source.getAttributes());
		if (c != null)
			GraphConstants.setBorderColor(target, c );
		Border b = GraphConstants.getBorder(source.getAttributes());
		if (b != null)
			GraphConstants.setBorder(target, b );
	}

	/** Shows a dialog color for selection and fills the border with the selected color.
	 * @param target the map that being edited
	 * @see com.jgraph.pad.actions.AbstractActionListCell#selectAndFillMap(Map)
	 */
	protected void selectAndFillMap(Map target) {
		if (getCurrentGraph().getSelectionCount() > 0) {
			Color value =
				JColorChooser.showDialog(
					graphpad.getFrame(),
					Translator.getString("ColorDialog"),
					null);
			if (value != null) {
				Object[] keys = new Object[] { GraphConstants.BORDER };
				GraphConstants.setRemoveAttributes(target, keys);
				GraphConstants.setBorderColor(target, value);
			}
		}
	}

}
