/*
 * @(#)FormatFillColor.java	1.2 31.01.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.util.Map;

import javax.swing.JColorChooser;

/**
 * Action that sets the selections fill color 
 * using a color dialog. After that, it fills the selected cell with the selected color.
 * 
 * @author sven.luzar
 * @version 1.0
 *
 */
public class FormatFillColor extends AbstractActionDefault {

	/**
	 * Constructor for FormatFillColor.
	 * This constructor is called from GPGraphPad.java.
     * @param graphpad the current state of the graph
	 */
	public FormatFillColor(GPGraphpad graphpad) {
		super(graphpad);
	}

	/** Action that sets the selections fill color using a color dialog.
      * Fills the selected cell with the selected color when an action
      * event is initiated.
      * @param e event trigger when menu-item is clicked or action is performed
	  * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	  */
	public void actionPerformed(ActionEvent e) {
		if (getCurrentGraph().getSelectionCount() > 0) {
			Color value =
				JColorChooser.showDialog(
					graphpad.getFrame(),
					Translator.getString("ColorDialog"),
					null);
			if (value != null) {
				Map map = GraphConstants.createMap();
				GraphConstants.setBackground(map, value);
				GraphConstants.setOpaque(map, true);
				setSelectionAttributes(map);
			}
		}
	}

}
