/*
 * @(#)FormatFillColorList.java	1.2 04.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.Color;
//import java.awt.Dimension;
//import java.awt.Point;
import java.awt.Rectangle;
//import java.awt.event.ActionEvent;
import java.util.Map;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.JColorChooser;
//import javax.swing.JComboBox;


/**
 * Presents a list of colors for selection and fills the selected cell with selected color.
 * @author sven.luzar
 * @version 1.0
 *
 */
public class FormatFillColorList extends AbstractActionListCellColor {

	/**
	 * Constructor for FormatFillColorList.
	 * This constructor is called from GPGraphPad.java.
	 * @param graphpad the current state of the graph
	 */
	public FormatFillColorList(GPGraphpad graphpad) {
		super(graphpad);
	}

	/**
	 * Constructor for FormatFillColorList.
	 * This constructor is called from GPGraphPad.java.
	 * @param graphpad the current state of the graph
	 * @param name the name of this action
	 */
	public FormatFillColorList(GPGraphpad graphpad, String name) {
		super(graphpad, name);
	}

	/**
	 * Constructor for FormatFillColorList.
	 * This constructor is called from GPGraphPad.java.
	 * @param graphpad the current state of the graph
	 * @param name the name of this action
	 * @param icon icon of this action
	 */
	public FormatFillColorList(GPGraphpad graphpad, String name, Icon icon) {
		super(graphpad, name, icon);
	}

	/** Fills table with the colors for selection.
	 * @see com.jgraph.pad.actions.AbstractActionListCell#fillCustomItems(Vector)
	 * @param items vector of colors
	 */
	protected void fillCustomItems(Vector items){
		VertexView v;
		Map map;

		for (int i = 0; i < colors.length; i++) {
			v =
				new VertexView(
					null,
					dummyGraph,
					dummyGraph.getGraphLayoutCache());
			map = GraphConstants.createMap();
			GraphConstants.setBounds(map, new Rectangle(point, size));
			GraphConstants.setBackground(map, colors[i]);
			GraphConstants.setOpaque(map, true);
			v.setAttributes(map);
			items.add(v);
		}
	}

	/** Fills the cell with the default color which is black.
	 * @see com.jgraph.pad.actions.AbstractActionListCell#fillResetMap(Map)
	 * @param target the map which its selected cell is being filled
	 */
	protected void fillResetMap(Map target) {
		GraphConstants.setOpaque(target, false);
	}

	/** Fills the cell of the map with the selected color.
	 * @see com.jgraph.pad.actions.AbstractActionListCell#fillApplyMap(CellView, Map)
	 * @param source the cell that being edited
	 * @param target the map contains the cell that being edited
	 */
	protected void fillApplyMap(CellView source, Map target) {
		Color value = GraphConstants.getBackground(source.getAttributes());
		if (value == null)
			return;
		GraphConstants.setOpaque(target, true);
		GraphConstants.setBackground(target, value);
	}

	/** Shows a dialog color for selection and fills the cell with the selected color.
	 * @param target the map that being edited
	 * @see com.jgraph.pad.actions.AbstractActionListCell#selectAndFillMap(Map)
	 */
	protected void selectAndFillMap(Map target) {
		Color value =
			JColorChooser.showDialog(
				graphpad.getFrame(),
				Translator.getString("ColorDialog"),
				null);
		if (value != null) {
			GraphConstants.setOpaque(target, true);
			GraphConstants.setBackground(target, value);
		}
	}
}
