/*
 * @(#)FormatTextFont.java 1.2 01.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//import java.awt.Container;
import java.awt.Dimension;
import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.util.Map;

import javax.swing.JComboBox;


/** Allows the user to change the font type.
 * @author sven.luzar
 * @version 2.0 TerpPresent
 */
public class FormatTextFont extends AbstractActionList {

 /** Constructor for FormatTextFont.
         * This constructor is called from GPGraphPad.java
         * @param graphpad Passes the current state of jGraphPad.
         */
 public FormatTextFont(GPGraphpad graphpad) {
  super(graphpad);
 }

 /** Returns an array of Objects that contains the various fonts that the user may choose
         * for the formatted text.
         * @see com.jgraph.pad.actions.AbstractActionList#getPossibleActionCommands()
         * @return - An array objects containing the various fonts.
         */
 public Object[] getItems() {
  GraphicsEnvironment gEnv =
   GraphicsEnvironment.getLocalGraphicsEnvironment();
  return gEnv.getAvailableFontFamilyNames();
 }

 /** Allows the user to change the font type when an action is
         * initiated
         * @param e Called when a menu-item is clicked or an Action is performed.
         * @return void
         */
 public void actionPerformed(ActionEvent e) {
  String item = (String)getSelectedItem(e);
  Map map = GraphConstants.createMap();
  GraphConstants.setFontName(map, item);
  setSelectionAttributes(map);
 }

        /** Creates a dropDown comboBox that will contain the list of
         * the available fonts.
         * @return - Type JcomboBox - comboBox that will contain
         * the list of the available fonts.
         */        
 protected JComboBox getToolBarComponent() {
  JComboBox combo = new JComboBox();
  
  combo.setPreferredSize(new Dimension(2 * 66, 26));
  combo.setMaximumSize(combo.getPreferredSize());
  return combo;
 }

}
