/*
 * @(#)FormatTextFontColorList.java 1.2 02.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.Map;
import java.util.Vector;

import javax.swing.JColorChooser;


/** Creates a list of color-combinations for the dropdown
 * menu. A color combination is then selected to fill the
 * color of the selected text.
 * @author sven.luzar
 * @version 2.0 TerpPresent
 */
public class FormatTextFontColorList extends AbstractActionListCellColor {

 /** Constructor for FormatTextFontColorList.
         * This constructor is called from GPGraphPad.java
         *
         * @param graphpad  - Passes the current state of jGraphPad.
         */
 public FormatTextFontColorList(GPGraphpad graphpad) {
  super(graphpad);
 }

 /** Fill the Applied Map with custom colors
         * @see com.jgraph.pad.actions.AbstractActionListCell#fillApplyMap(CellView, Map)
         * @param source - type CellView - Used to get the attributes of cell
         * @param target - type Map - Used to fill the Map with custom
         * colors
         * @return void
         */
 protected void fillApplyMap(CellView source, Map target) {
  GraphConstants.setForeground(
   target,
   GraphConstants.getForeground(source.getAttributes()));
 }

 /** Adds custom colors (most frequently used colors) in the
         * drop-down list.
         * @see com.jgraph.pad.actions.AbstractActionListCell#fillCustomItems(Vector)
         * @param items - Vector items - that contains the custom colors
         */
 protected void fillCustomItems(Vector items) {
  for (int i = 0; i < colors.length; i++) {
   VertexView v = new VertexView(" A", dummyGraph, null);
   Map map = GraphConstants.createMap();
   GraphConstants.setBounds(
    map,
    new Rectangle(new Point(0, 0), new Dimension(14, 15)));
   GraphConstants.setForeground(map, colors[i]);
   v.setAttributes(map);
   items.add(v);
  }
 }

 /** Resets the Map with the custom colors
         * @see com.jgraph.pad.actions.AbstractActionListCell#fillResetMap(Map)
         * @param target - target of type Map - resetting the map
         * @return void
         */
 protected void fillResetMap(Map target) {
   GraphConstants.setForeground(target, Color.BLACK);
 }

 /** Fills the selected Map (object) with the color chosen
         * @see com.jgraph.pad.actions.AbstractActionListCell#selectAndFillMap(Map)
         * @param target - type Map - selected Map (object) is filled with
         * the color selected.
         * @return void
         */
 protected void selectAndFillMap(Map target) {
  Color value =
   JColorChooser.showDialog(
    graphpad.getFrame(),
    Translator.getString("ColorDialog"),
    null);
  if (value != null) {
   GraphConstants.setForeground(target, value);
  }
 }

}
