/*
 * @(#)FormatTextFontSizeList.java 1.2 02.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.Map;

import javax.swing.JComboBox;


/** Creates a list of all possible fontSizes.
 * @author sven.luzar
 * @version 2.0 TerpPresent
 */
public class FormatTextFontSizeList extends AbstractActionList {

 /** Sizes of the possible fonts
  */
 public static final String[] fontSizes =
  new String[] { "12", "14", "16", "18", "20", "24", "30", "36" };

 /** Constructor for FormatTextFontSizeList.
         * This constructor is called from GPGraphPad.java
         * @param graphpad Passes the current state of jGraphPad.
         */
 public FormatTextFontSizeList(GPGraphpad graphpad) {
  super(graphpad);
 }

 /** Creates a list of all possible fontSizes when an
         * action is initiated.
         * @param e Called when a menu-item is clicked or an Action is performed
         * @throws ex Throws exception when encountering an invalid float
         * @return void
         */
 public void actionPerformed(ActionEvent e) {
  String item = (String)getSelectedItem(e);
  try {
   float f = Float.parseFloat(item) ;
   Map map = GraphConstants.createMap();
   GraphConstants.setFontSize(map, f);
   setSelectionAttributes(map);
  } catch (Exception ex) {
   // invalid float
  }
 }

 /** Creates a comboBox that contains the list of possible
         *  fontSizes.
         * @see com.jgraph.pad.actions.AbstractActionList#getToolBarComponent()
         * @return JComboBox Returns a comboBox of type JComboxBox that contains the list of
         *  possible font sizes.
         */
 protected JComboBox getToolBarComponent() {
  JComboBox combo = new JComboBox();
  combo.setPreferredSize(new Dimension(66 - 15, 26));
  combo.setMaximumSize(combo.getPreferredSize());
  combo.setEditable(true);
  return combo;
 }

 /** Returns an array of Objects - string in this case
         * Each string is the possible size of the font.
         * Example: "12", "14", ...
         * @see com.jgraph.pad.actions.AbstractActionList#getItems()
         * @return items - Returns an array of Objects that contains
         * the available fontSizes.
         */
 protected Object[] getItems() {
  return fontSizes;
 }

}
