/*
 * @(#)GPFileChooser.java	1.0 17.02.2003
 *
 * Copyright (C) 2003 luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.Component;
import java.awt.Window;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileSystemView;

/**File chooser for open or close graph pad files.
 * 
 * The file chooser uses the file formats from the
 * graph model provider at the GPDocument. If the
 * GPDocument is null the file chooser uses each
 * available file format.
 * 
 * 
 * @author luzar
 * @version 2.0 TerpPresent
 */
public class GPFileChooser extends JFileChooser {

	/** Constructor for GPFileChooser.
         * @param doc a GPDocument
         */
	public GPFileChooser(GPDocument doc) {
		super();
		init(doc);
	}

	/** Constructor for GPFileChooser.
         * @param doc a GPDocument
         * @param currentDirectoryPath the path for the current directory in string
         */
	public GPFileChooser(GPDocument doc, String currentDirectoryPath) {
		super(currentDirectoryPath);
		init(doc);
	}

	/** Constructor for GPFileChooser.
         * @param doc a GPDocument
         * @param currentDirectory the path for the current directory in string
         */
	public GPFileChooser(GPDocument doc, File currentDirectory) {
		super(currentDirectory);
		init(doc);
	}

	/** Constructor for GPFileChooser.
         * @param doc a GPDocument
         * @param fsv file system view
         */
	public GPFileChooser(GPDocument doc, FileSystemView fsv) {
		super(fsv);
		init(doc);
	}

	/** Constructor for GPFileChooser.
         * @param doc a GPDocument
         * @param currentDirectory file object for the current directory
         * @param fsv the file system view
         */
	public GPFileChooser(GPDocument doc, File currentDirectory, FileSystemView fsv) {
		super(currentDirectory, fsv);
		init(doc);
	}

	/** Constructor for GPFileChooser.
         * @param doc a GPDocument
         * @param currentDirectoryPath file object of the current directory
         * @param fsv the file system veiw
         */
	public GPFileChooser(GPDocument doc, String currentDirectoryPath, FileSystemView fsv) {
		super(currentDirectoryPath, fsv);
		init(doc);
	}

	/** Initializes the gui.
         *
         *  Adds a property change listener to catch
         *  changes at the file filter. If the user
         *  changes the file filter the listener
         *  changes the accessory for the correct file format.
         * @param doc a GPDocument
         */
	protected void init(GPDocument doc) {
		addPropertyChangeListener(new PropertyChangeListener() {
			/**
			 * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
			 */
			public void propertyChange(PropertyChangeEvent evt) {
				if (evt != null
					&& evt.getPropertyName() != null
					&& (evt.getPropertyName().equals(
						JFileChooser.FILE_FILTER_CHANGED_PROPERTY)
						|| evt.getPropertyName().equals(
						JFileChooser.DIALOG_TYPE_CHANGED_PROPERTY))) {
					FileFilter ff = (FileFilter)getFileFilter();
					GraphModelFileFormat gmff =
						GraphModelProviderRegistry.getGraphModelFileFormat(ff);
					if (gmff == null)
						setAccessory(null);
					else if (getDialogType() == SAVE_DIALOG)
						setAccessory(gmff.getWriteAccessory());
					else if (getDialogType() == OPEN_DIALOG)
						setAccessory(gmff.getReadAccessory());
					Component comp = GPFileChooser.this;

					while (!(comp instanceof Window)) {
						if (comp == null)
							return;
						comp = comp.getParent();
					}
					if (comp != null)
						 ((Window) comp).pack();
				}
			}
		});
		GraphModelFileFormat[] formats;
		if (doc == null){
			formats =
				GraphModelProviderRegistry.getGraphModelFileFormats();
		} else {
			formats = doc.getGraphModelProvider().getGraphModelFileFormats() ;
		}

		for (int i = 0; i < formats.length; i++) {
			GraphModelFileFormat format = formats[i];
			addChoosableFileFilter(format.getFileFilter());
		}

	}
	
}
