/*
 * @(#)GPGraph.java	1.2 11/11/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
//import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.print.PageFormat;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.border.Border;

/** a graph that can be displayed on a page
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */
public class GPGraph extends JGraph implements Serializable {

	public static String FILE_FORMAT_VERSION = "PAD-1.0";
	
    /** Boolean to make page visible or not.
     */    
	protected boolean pagevisible = false;

        /** The page format.
         */        
	protected transient PageFormat pageFormat = new PageFormat();

        /** The background of the page.
         */        
	protected Image background = null;
	
        /** Write property hashtable.
         */        
	protected Hashtable writeProperties = new Hashtable();

        /** The default border color.
         */        
	protected transient Color defaultBorderColor = Color.black;

        /** Default constructor.
         */        
	public GPGraph() {
		this(null);
	}

        /** Constructor with specified model.
         * @param model the model of the GPGraph
         */        
	public GPGraph(GraphModel model) {
		this(model, null);
	}

        /** Constructor with specified model.
         * @param model the model of the GPGraph
         * @param view the view of the GPGraph
         */    
	public GPGraph(GraphModel model, GraphLayoutCache view) {
		super(model, view, null);

	}

        /** Get the archiveable state.
         * @return a serializable object with cells, veiw and write info
         */        
	public Serializable getArchiveableState() {
		Object[] cells = getGraphLayoutCache().order(getAll());
		Map viewAttributes =
			GraphConstants.createAttributes(cells, getGraphLayoutCache());
		Vector v = new Vector();
		v.add(cells);
		v.add(viewAttributes);
		v.add(writeProperties);
		return v;
	}

	/**
	 * 
	 * @return
	 */
	public Serializable getArchiveableState_New() {
		Object[] cells = getGraphLayoutCache().order(getAll());
		Map viewAttributes =
			GraphConstants.createAttributes(cells, getGraphLayoutCache());
		Vector v = new Vector();
		v.add(FILE_FORMAT_VERSION);
		v.add(cells);
		v.add(viewAttributes);
		v.add(writeProperties);
		v.add(new Boolean(isEditable()));
		v.add(new Boolean(isBendable()));
		v.add(new Boolean(isCloneable()));
		v.add(new Boolean(isConnectable()));
		v.add(new Boolean(isDisconnectable()));
		v.add(new Boolean(isDisconnectOnMove()));
		v.add(new Boolean(isDoubleBuffered()));
		v.add(new Boolean(isDragEnabled()));
		v.add(new Boolean(isDropEnabled()));
		v.add(new Boolean(isMoveable()));
		v.add(new Boolean(isPageVisible()));
		v.add(new Boolean(isSelectNewCells()));
		v.add(new Boolean(isSizeable()));
		v.add(new Boolean(isGridVisible()));
		v.add(new Boolean(isGridEnabled()));
		v.add(new Integer(getGridSize()));
		//v.add(new Integer(getGridMode()));
		v.add(new Double(getScale()));
		v.add(this.getModel()) ;
		//v.add(new Boolean(isAntiAliased())); next file version...
		return v;
	}
	
	public void setArchivedState_New(Object s) {
		if (s instanceof Vector) {
			Vector v = (Vector) s;
			
			// Supports Multiple Versions
			Object ident = v.get(0);
			
			if (ident instanceof String) {
				
				if (ident.equals(FILE_FORMAT_VERSION)) { // Current File Format
					int index = 1;
					Object[] cells = (Object[]) v.get(index++);
					Map attrib = (Map) v.get(index++);
					getGraphLayoutCache().insert(cells, attrib, null, null, null);
					setWriteProperties((Hashtable)v.get(index++));
					setEditable(((Boolean) (v.get(index++))).booleanValue());
					setBendable(((Boolean) (v.get(index++))).booleanValue());
					setCloneable(((Boolean) (v.get(index++))).booleanValue());
					setConnectable(((Boolean) (v.get(index++))).booleanValue());
					setDisconnectable(((Boolean) (v.get(index++))).booleanValue());
					setDisconnectOnMove(((Boolean) (v.get(index++))).booleanValue());
					setDoubleBuffered(((Boolean) (v.get(index++))).booleanValue());
					setDragEnabled(((Boolean) (v.get(index++))).booleanValue());
					setDropEnabled(((Boolean) (v.get(index++))).booleanValue());
					setMoveable(((Boolean) (v.get(index++))).booleanValue());
					setPageVisible(((Boolean) (v.get(index++))).booleanValue());
					setSelectNewCells(((Boolean) (v.get(index++))).booleanValue());
					setSizeable(((Boolean) (v.get(index++))).booleanValue());
					setGridVisible(((Boolean) (v.get(index++))).booleanValue());
					setGridEnabled(((Boolean) (v.get(index++))).booleanValue());
					setGridSize(((Integer) (v.get(index++))).intValue());
					//setGridMode(((Integer) (v.get(index++))).intValue());
					setScale(((Double) (v.get(index++))).doubleValue());
					this.setModel((GraphModel)v.get(index++)) ;
					//setAntiAliased(((Boolean) (v.get(index++))).booleanValue()); next file version...
				} else
					System.out.println("Unknown file format: "+ident);
				
			} else {
				// Unversioned (old) file format
				Object[] cells = (Object[]) v.get(0);
				Map attrib = (Map) v.get(1);
				getGraphLayoutCache().insert(cells, attrib, null, null, null);
				setWriteProperties((Hashtable)v.get(2));
			}
		}
	}
	
        /** Set archived state.
         * @param s the state
         */        
	public void setArchivedState(Object s) {
		if (s instanceof Vector) {
			Vector v = (Vector) s;
			Object[] cells = (Object[]) v.get(0);
			Map attrib = (Map) v.get(1);
			getGraphLayoutCache().insert(cells, attrib, null, null, null);
			setWriteProperties((Hashtable)v.get(2));
		}
	}

        /** Set the page format.
         * @param format the page format
         */        
	public void setPageFormat(PageFormat format) {
		pageFormat = format;
	}

        /** Get the page format.
         * @return the page format object
         */        
	public PageFormat getPageFormat() {
		return pageFormat;
	}

        /** Set page to be visible or not.
         * @param flag the flag to set page visible
         */        
	public void setPageVisible(boolean flag) {
		pagevisible = flag;
	}

        /** Return if page is visible.
         * @return page visibility status
         */        
	public boolean isPageVisible() {
		return pagevisible;
	}

        /** Set the background to an image.
         * @param img the background image
         */        
	public void setBackgroundImage(Image img) {
		background = img;
	}

        /** Get the background image.
         * @return the background image
         */        
	public Image getBackgroundImage() {
		return background;
	}

	/** Creates and returns a default <code>GraphView</code>.
         *
         * @return the default <code>GraphView</code>
         * @param v an shape on the page
         * @param cm the cell mapper
         */
	protected VertexView createVertexView(Object v, CellMapper cm) {
		if (v instanceof EllipseCell)
			return new EllipseView(v, this, cm);
		else if (v instanceof ImageCell)
			return new ScaledVertexView(v, this, cm);
		else if ((v instanceof TextCell) &&
		 		  ((TextCell) v).isMultiLined())
		 	return new MultiLinedView(v, this, cm);
		return super.createVertexView(v, cm);
	}

        /** A render for vertex.
         */        
	public static VertexRenderer renderer = new ScaledVertexRenderer();

        /** The scaled vertex veiw of a graph.
         */        
	public class ScaledVertexView extends VertexView {

            /** Constructor for ScaledVertexView.
             * @param v a vertex
             * @param graph a JGraph
             * @param cm a CellMapper
             */            
		public ScaledVertexView(Object v, JGraph graph, CellMapper cm) {
			super(v, graph, cm);
		}

                /** Get the cell veiw renderer.
                 * @return a CellVeiwRenderer
                 */                
		public CellViewRenderer getRenderer() {
			return GPGraph.renderer;
		}

	}

        /** Scaled vertex renderer.
         */        
	public static class ScaledVertexRenderer extends VertexRenderer {

            /** Paints the graphics.
             * @param g the graphics for painting
             */            
		public void paint(Graphics g) {
			Icon icon = getIcon();
			setIcon(null);
			Dimension d = getSize();
			Image img = null;
			if (icon instanceof ImageIcon)
				img = ((ImageIcon) icon).getImage();
			if (img != null)
				g.drawImage(img, 0, 0, d.width - 1, d.height - 1, graph);
			super.paint(g);
		}

	}

	//
	// Defines Semantics of the Graph
	//

	/** Returns true if <code>object</code> is a vertex, that is, if it
         * is not an instance of Port or Edge, and all of its children are
         * ports, or it has no children.
         * @param cell a cell of the graph
         * @return boolean for grouping status
         */
	public boolean isGroup(Object cell) {
		// Map the Cell to its View
		CellView view = getGraphLayoutCache().getMapping(cell, false);
		if (view != null)
			return !view.isLeaf();
		return false;
	}

	/** Returns true if <code>object</code> is a vertex, that is, if it
         * is not an instance of Port or Edge, and all of its children are
         * ports, or it has no children.
         * @param object the object being checked
         * @return boolean for if object is a vertex
         */
	public boolean isVertex(Object object) {
		if (!(object instanceof Port) && !(object instanceof Edge))
			return !isGroup(object) && object != null;
		return false;
	}

        /** Check if object is a port.
         * @param object object being checked
         * @return boolean for if object is a port
         */        
	public boolean isPort(Object object) {
		return (object instanceof Port);
	}

        /** Check if object is an edge.
         * @param object object being checked
         * @return boolean for if object is an edge
         */        
	public boolean isEdge(Object object) {
		return (object instanceof Edge);
	}

        /** Get all the vertices.
         * @return an array of all the vertices
         */        
	public Object[] getSelectionVertices() {
		Object[] tmp = getSelectionCells();
		Object[] all =
			DefaultGraphModel.getDescendants(getModel(), tmp).toArray();
		return getVertices(all);
	}

        /** Get the vertices from cells.
         * @param cells the cells of the graph
         * @return an array of all the vertices
         */        
	public Object[] getVertices(Object[] cells) {
		if (cells != null) {
			ArrayList result = new ArrayList();
			for (int i = 0; i < cells.length; i++)
				if (isVertex(cells[i]))
					result.add(cells[i]);
			return result.toArray();
		}
		return null;
	}

        /** Get the selection edges.
         * @return an array of all the selection edges
         */        
	public Object[] getSelectionEdges() {
		return getEdges(getSelectionCells());
	}

        /** Get everything (descendants of the root).
         * @return an array storing the decendants from the root
         */        
	public Object[] getAll() {
		return getDescendants(getRoots());
	}

        /** Get all the edges.
         * @param cells the cells of the graph
         * @return an array of all the edges
         */        
	public Object[] getEdges(Object[] cells) {
		if (cells != null) {
			ArrayList result = new ArrayList();
			for (int i = 0; i < cells.length; i++)
				if (isEdge(cells[i]))
					result.add(cells[i]);
			return result.toArray();
		}
		return null;
	}

        /** Get the neighbour.
         * @param edge an edge
         * @param vertex an vertex
         * @return the neighbour object
         */        
	public Object getNeighbour(Object edge, Object vertex) {
		Object source = getSourceVertex(edge);
		if (vertex == source)
			return getTargetVertex(edge);
		else
			return source;
	}

        /** Get the source vertex.
         * @param edge an edge
         * @return the object with source vertex
         */        
	public Object getSourceVertex(Object edge) {
		Object sourcePort = graphModel.getSource(edge);
		return graphModel.getParent(sourcePort);
	}

        /** Get the target vertex.
         * @param edge an edge
         * @return an object with target vertex
         */        
	public Object getTargetVertex(Object edge) {
		Object targetPort = graphModel.getTarget(edge);
		return graphModel.getParent(targetPort);
	}

        /** Get the source view.
         * @param edge an edge
         * @return the cell source of the graph
         */        
	public CellView getSourceView(Object edge) {
		Object source = getSourceVertex(edge);
		return getGraphLayoutCache().getMapping(source, false);
	}

        /** Get the target view.
         * @param edge an edge
         * @return the cell target of the graph
         */        
	public CellView getTargetView(Object edge) {
		Object target = getTargetVertex(edge);
		return getGraphLayoutCache().getMapping(target, false);
	}

        /** Get the edge between two vertices.
         * @param vertex1 the first vertex
         * @param vertex2 the second vertex
         * @return all the edges between the two vertices
         */        
	public Object[] getEdgesBetween(Object vertex1, Object vertex2) {
		ArrayList result = new ArrayList();
		Set edges =
			DefaultGraphModel.getEdges(graphModel, new Object[] { vertex1 });
		Iterator it = edges.iterator();
		while (it.hasNext()) {
			Object edge = it.next();
			Object source = getSourceVertex(edge);
			Object target = getTargetVertex(edge);
			if ((source == vertex1 && target == vertex2)
				|| (source == vertex2 && target == vertex1))
				result.add(edge);
		}
		return result.toArray();
	}

	/**
	 * Overrides <code>JComponent</code>'buttonSelect <code>getToolTipText</code>
	 * method in order to allow the graph controller to create a tooltip
	 * for the topmost cell under the mousepointer. This differs from JTree
	 * where the renderers tooltip is used.
	 * <p>
	 * NOTE: For <code>JGraph</code> to properly display tooltips of its
	 * renderers, <code>JGraph</code> must be a registered component with the
	 * <code>ToolTipManager</code>.  This can be done by invoking
	 * <code>ToolTipManager.sharedInstance().registerComponent(graph)</code>.
	 * This is not done automatically!
	 * @param event the <code>MouseEvent</code> that initiated the
	 * <code>ToolTip</code> display
	 * @return a string containing the  tooltip or <code>null</code>
	 * if <code>event</code> is null
	 */
	public String getToolTipText(MouseEvent event) {
		if (event != null) {
			Object cell = getFirstCellForLocation(event.getX(), event.getY());
			if (cell != null) {
				String tmp = convertValueToString(cell);
				String s = "<html>";
				if (tmp != null && tmp.length() > 0)
					s = s + "<strong>" + tmp + "</strong><br>";
				return s + getToolTipForCell(cell) + "</html>";
			}
		}
		return null;
	}

        /** Get the tool tip for the cell.
         * @param cell the cell of the graph
         * @return a string with the tool tip information
         */        
	protected String getToolTipForCell(Object cell) {
		CellView view = getGraphLayoutCache().getMapping(cell, false);
		String s = "";
		Rectangle bounds = view.getBounds();
		if (bounds != null) {
			s = s + "Location: " + bounds.x + ", " + bounds.y + "<br>";
			s = s + "Size: " + bounds.width + ", " + bounds.height + "<br>";
		}
		java.util.List points = GraphConstants.getPoints(view.getAttributes());
		if (points != null)
			s = s + "Points: " + points.size() + "<br>";
		if (!(cell instanceof Edge) && !(cell instanceof Port)) {
			s = s + "Children: " + graphModel.getChildCount(cell) + "<br>";
			int n =
				DefaultGraphModel
					.getEdges(getModel(), new Object[] { cell })
					.size();
			s = s + "Edges: " + n;
		} else if (cell instanceof Edge) {
			Edge edge = (Edge) cell;
			Object source = graphModel.getSource(edge);
			if (source != null) {
				String host =
					convertValueToString(graphModel.getParent(source));
				String port = convertValueToString(source);
				s = s + "Source: " + host + ":" + port + "<br>";
			}
			Object target = graphModel.getTarget(edge);
			if (target != null) {
				String host =
					convertValueToString(graphModel.getParent(target));
				String port = convertValueToString(target);
				s = s + "Target: " + host + "/" + port + "<br>";
			}
		}
		return s;
	}

	/**
	 * Notification from the <code>UIManager</code> that the L&F has changed.
	 * Replaces the current UI object with the latest version from the
	 * <code>UIManager</code>. Subclassers can override this to support
	 * different GraphUIs.
	 * @see JComponent#updateUI
	 *
	 */
	public void updateUI() {
		setUI(new GPGraphUI());
		invalidate();
	}


	/** Returns true if the given vertices are conntected by a single edge
         * in this document.
         * @param v1 the first vertex
         * @param v2 the second vertex
         * @return boolean for if there is an edge
         */
	public boolean isNeighbour(Object v1, Object v2) {
		Object[] edges = getEdgesBetween(v1, v2);
		return (edges != null && edges.length > 0);
	}

	
        /** Reading an object with Serialization support.
         * @param s the object input stream
         * @throws IOException throw IOException for the stream
         * @throws ClassNotFoundException throw exception if class not found
         */        
	private void readObject(ObjectInputStream s)
		throws IOException, ClassNotFoundException {
		s.defaultReadObject();
		pageFormat = new PageFormat();
	}

	//
	// Multiline View
	//
	
        /** The multilined view.
         */        
	class MultiLinedView extends VertexView {
		// static
            /** The final version of the multilined renderer.
             */            
		final MultiLinedRenderer renderer = new MultiLinedRenderer();
		// static
                /** A multilined editor.
                 */                
		final MultiLinedEditor editor = new MultiLinedEditor();

                /** Constructor for multilined view.
                 * @param cell the cell for the graph
                 * @param graph a JGraph
                 * @param cm a cell mapper
                 */                
		public MultiLinedView(Object cell, JGraph graph, CellMapper cm) {
			super(cell, graph, cm);
		}

                /** Get the renderer.
                 * @return a cell view renderer
                 */                
		public CellViewRenderer getRenderer() {
			return renderer;
		}

                /** Get the editor.
                 * @return a GraphCellEditor object
                 */                
		public GraphCellEditor getEditor() {
			return editor;
		}

                /** The multilined editor.
                 */                
		class MultiLinedEditor extends DefaultGraphCellEditor {
                    /** A real cell editor.
                     */                    
			class RealCellEditor
				extends AbstractCellEditor
				implements GraphCellEditor {
                                    /** The editor component.
                                     */                                    
				JTextArea editorComponent = new JTextArea();
                                /** Real cell editor constructor.
                                 */                                
				public RealCellEditor() {
					editorComponent.setBorder(
						UIManager.getBorder("Tree.editorBorder"));
					editorComponent.setLineWrap(true);
					editorComponent.setWrapStyleWord(true);

					//substitute a JTextArea's VK_ENTER action with our own that will stop an edit.
					editorComponent.getInputMap(JComponent.WHEN_FOCUSED).put(
						KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
						"enter");
					editorComponent
						.getActionMap()
						.put("enter", new AbstractAction() {
						public void actionPerformed(ActionEvent e) {
							stopCellEditing();
						}
					});
				}

                                /** Get the graph cell editor.
                                 * @param graph a JGraph
                                 * @param value the value for the cell
                                 * @param isSelected boolean for if graph is selected
                                 * @return an editor component
                                 */                                
				public Component getGraphCellEditorComponent(
					JGraph graph,
					Object value,
					boolean isSelected) {
					editorComponent.setText(value.toString());
					editorComponent.selectAll();
					return editorComponent;
				}

                                /** Get the cell editor value.
                                 * @return the editor component text
                                 */                                
				public Object getCellEditorValue() {
					return editorComponent.getText();
				}

                                /** Stop the cell editing.
                                 * @return a boolean value of the stop status
                                 */                                
				public boolean stopCellEditing() {
					//set the size of a vertex to that of an editor.
					CellView view =
						graph.getGraphLayoutCache().getMapping(
							graph.getEditingCell(),
							false);
					Map map = view.getAllAttributes();
					Rectangle cellBounds = GraphConstants.getBounds(map);
					Rectangle editingBounds = editorComponent.getBounds();
					cellBounds.setBounds(cellBounds.x,
							cellBounds.y,
							editingBounds.width,
							editingBounds.height);
					// Call view.setBounds() to make sure the view stores
					// the new bounds
					return super.stopCellEditing();
				}

                                /** Select the cell.
                                 * @param event the event
                                 * @return boolean status of the selection
                                 */                                
				public boolean shouldSelectCell(EventObject event) {
					editorComponent.requestFocus();
					return super.shouldSelectCell(event);
				}
			}

                        /** A mutlilined editor constructor.
                         */                        
			public MultiLinedEditor() {
				super();
			}
			/** Overriding this in order to set the size of an editor to that of an edited view.
                         * @param graph the graph
                         * @param cell the cell for the graph
                         * @param isSelected boolean for if graph is selected
                         * @return return the component of the editor
                         */
			public Component getGraphCellEditorComponent(
				JGraph graph,
				Object cell,
				boolean isSelected) {

				Component component =
					super.getGraphCellEditorComponent(graph, cell, isSelected);

				//set the size of an editor to that of a view
				CellView view =
					graph.getGraphLayoutCache().getMapping(cell, false);
				editingComponent.setBounds(view.getBounds());

				//I have to set a font here instead of in the RealCellEditor.getGraphCellEditorComponent() because
				//I don't know what cell is being edited when in the RealCellEditor.getGraphCellEditorComponent().
				Font font = GraphConstants.getFont(view.getAllAttributes());
				editingComponent.setFont(
					(font != null) ? font : graph.getFont());

				return component;
			}

                        /** Create a new graph cell editor.
                         * @return a new multilined editor
                         */                        
			protected GraphCellEditor createGraphCellEditor() {
				return new MultiLinedEditor.RealCellEditor();
			}

			/** Overriting this so that I could modify an eiditor container.
                         * see http://sourceforge.net/forum/forum.php?thread_id=781479&forum_id=140880
                         * @return a new multilined editor
                         */
			protected Container createContainer() {
				return new MultiLinedEditor.ModifiedEditorContainer();
			}

                        /** A modified editor container.
                         */                        
			class ModifiedEditorContainer extends EditorContainer {
                            /** Set the layout.
                             */                            
				public void doLayout() {
					super.doLayout();
					//substract 2 pixels that were added to the preferred size of the container for the border.
					Dimension cSize = getSize();
					Dimension dim = editingComponent.getSize();
					editingComponent.setSize(dim.width - 2, dim.height);

					//reset container's size based on a potentially new preferred size of a real editor.
					setSize(cSize.width, getPreferredSize().height);
				}
			}
		}

                /** Constructor for multilined rednerer.
                 */                
		class MultiLinedRenderer
			extends JTextArea
			implements CellViewRenderer {
			{
				setLineWrap(true);
				setWrapStyleWord(true);
			}

                        /** Get the renderer's component.
                         * @param graph the JGraph
                         * @param view the cell view
                         * @param sel boolean selected
                         * @param focus boolean focused
                         * @param preview boolean for preview
                         * @return the rednerer's component
                         */                        
			public Component getRendererComponent(
				JGraph graph,
				CellView view,
				boolean sel,
				boolean focus,
				boolean preview) {
				setText(view.getCell().toString());

				Map attributes = view.getAllAttributes();
				installAttributes(graph, attributes);
				return this;
			}

                        /** Installation attributes.
                         * @param graph the JGrpha
                         * @param attributes a map of all the attributes
                         */                        
			protected void installAttributes(JGraph graph, Map attributes) {
				setOpaque(GraphConstants.isOpaque(attributes));
				Color foreground = GraphConstants.getForeground(attributes);
				setForeground(
					(foreground != null) ? foreground : graph.getForeground());
				Color background = GraphConstants.getBackground(attributes);
				setBackground(
					(background != null) ? background : graph.getBackground());
				Font font = GraphConstants.getFont(attributes);
				setFont((font != null) ? font : graph.getFont());
				Border border = GraphConstants.getBorder(attributes);
				Color bordercolor = GraphConstants.getBorderColor(attributes);
				if (border != null)
					setBorder(border);
				else if (bordercolor != null) {
					int borderWidth =
						Math.max(
							1,
							Math.round(
								GraphConstants.getLineWidth(attributes)));
					setBorder(
						BorderFactory.createLineBorder(
							bordercolor,
							borderWidth));
				}
			}
		}
	}


	/**
	 * Returns the writeProperties.
	 * @return Hashtable
	 */
	public Hashtable getWriteProperties() {
		return writeProperties;
	}

	/**
	 * Sets the writeProperties.
	 * @param writeProperties The writeProperties to set
	 */
	public void setWriteProperties(Hashtable writeProperties) {
		this.writeProperties = writeProperties;
	}

}