/*
 * @(#)GPGraphUI.java	1.2 11/11/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.geom.AffineTransform;
import java.io.Reader;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.JComponent;

//import com.jgraph.plaf.basic.TransferHandler;

/** the user interface of the graph
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */
public class GPGraphUI extends BasicGraphUI {

    /** Get the GPGraph object.
     * @return a GPGraph object
     */    
	public GPGraph getGPGraph() {
		return (GPGraph) graph;
	}

	//
	// Override Parent Methods
	//
	// @jdk14
	/*
	protected TransferHandler createTransferHandler() {
		return new GPTransferHandler();
	}*/

	/** Paint the background of this graph. Calls paintGrid.
         * @param g the graphics of the background
         */
	protected void paintBackground(Graphics g) {
		Rectangle pageBounds = graph.getBounds();
		if (getGPGraph().getBackgroundImage() != null) {
			// Use clip and pageBounds
			double s = graph.getScale();
			Graphics2D g2 = (Graphics2D) g;
			AffineTransform tmp = g2.getTransform();
			g2.scale(s, s);
			g.drawImage(getGPGraph().getBackgroundImage(), 0, 0, graph);
			g2.setTransform(tmp);
		} else if (getGPGraph().isPageVisible()) { // FIX: Use clip
			int w = (int) (getGPGraph().getPageFormat().getWidth());
			int h = (int) (getGPGraph().getPageFormat().getHeight());
			Point p = graph.toScreen(new Point(w, h));
			w = p.x;
			h = p.y;
			g.setColor(graph.getHandleColor());
			g.fillRect(0, 0, graph.getWidth(), graph.getHeight());
			g.setColor(Color.darkGray);
			g.fillRect(3, 3, w, h);
			g.setColor(graph.getBackground());
			g.fillRect(1, 1, w - 1, h - 1);
			pageBounds = new Rectangle(0, 0, w, h);
		}
		if (graph.isGridVisible())
			paintGrid(graph.getGridSize(), g, pageBounds);
	}

	/**
	 * TransferHandler that can import text.
	 */
	public class GPTransferHandler extends GraphTransferHandler {

            /** Create a graph transferable.
             * @param graph the JGraph object
             * @param cells an array of cells
             * @param cs the connection set
             * @param viewAttributes a map of the view attributes
             * @param bounds a rectangle bounds
             * @return a GraphTransferable object
             */            
		protected GraphTransferable create(
			JGraph graph,
			Object[] cells,
			ConnectionSet cs,
			Map viewAttributes,
			Rectangle bounds) {
			String gxl = GPConverter.toGXL(getGPGraph(), cells);
			return new GPTransferable(gxl, cells, cs, viewAttributes, bounds);
		}

                /** Import the data.
                 * @param comp a JComponent
                 * @param t a transferable
                 * @return boolean status of the import
                 */                
		public boolean importData(JComponent comp, Transferable t) {
			if (super.importData(comp, t))
				return true;
			else if (graph.isDropEnabled() && comp instanceof JGraph) {
				try {
					JGraph graph = (JGraph) comp;
					Object cell = null;
					Map map = GraphConstants.createMap();
					// Insert as Text
					DataFlavor bestFlavor =
						DataFlavor.selectBestTextFlavor(
							t.getTransferDataFlavors());
					if (bestFlavor != null && cell == null) {
						Reader reader = bestFlavor.getReaderForText(t);
						StringBuffer s = new StringBuffer();
						char[] c = new char[1];
						while (reader.read(c) != -1)
							s.append(c);
						Point p =
							graph.fromScreen(
								graph.snap(new Point(getInsertionLocation())));
						int gs = graph.getGridSize();
						if (p == null)
							p = new Point(gs, gs);
						cell = new DefaultGraphCell(new String(s));
						((DefaultGraphCell) cell).add(new DefaultPort());
						CellView view = graphLayoutCache.getMapping(cell, true);
						Dimension d = graph.snap(getPreferredSize(graph, view));
						if (d == null)
							d = new Dimension(2 * gs, 2 * gs);
						GraphConstants.setBounds(map, new Rectangle(p, d));
					}
					if (cell != null) {
						Map viewMap = new Hashtable();
						viewMap.put(cell, map);
						graph.getModel().insert(
							new Object[] { cell },
							viewMap,
							null,
							null,
							null);
					}
				} catch (Exception exception) {
					//System.out.println(
					//	"Cannot import data: " + exception.getMessage());
				}
			}
			return false;
		}

	}

}