/*
 * @(#)GPGraphpad.java 1.2 11/11/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Frame;
import java.awt.event.ContainerListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyVetoException;
import java.util.*;

import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.UIManager;

//added by Xun
import java.awt.image.BufferedImage;
import java.lang.reflect.Method;
import java.awt.event.MouseEvent;
import java.awt.Component;


/** Sample application using the simple text editor component that
* supports only one font.
*
* @author Gaudenz Alder
* @author Sven Luzar
* @version 2.0 TerpPresent
*/
public class GPGraphpad extends JPanel {
    
    /** Application Icon. From resource file.
    */
    public static ImageIcon applicationIcon;
    
    /** Application Icon. From resource file.
    */
    protected static ImageIcon logoIcon;
    
    /** Application Title. From resource file.
        */
    protected static String appTitle;
    
    /** Default entry size
        */
    protected static int entrySize = 60;
    
    /** Default frame width
        *  You can overwrite this value by using the
        *  Graphpad property FrameWidth
        */
    protected static int frameWidth = 640;
    
    /** Default frame height
        *  You can overwrite this value by using the
        *  Graphpad property FrameHeight
        */
    protected static int frameHeight = 480;
    
    /** Default value for the maximized frame.
        *  You can overwrite this value by using the
        *  Graphpad property FrameMaximize
        */
    protected static boolean frameMaximize;
    
    /** Boolean for the visible state of
        *  the toolbars
        */
    protected boolean toolBarsVisible = true;
    
    /** Global instance for some graph tool implementations.
        * */
    protected static GPGraphTools graphTools = new GPGraphTools();
    
    /** Log console for the System in and out
        *  messages
        */
    protected GPLogConsole logger =
        new GPLogConsole(appTitle, applicationIcon.getImage());
    
    /** Desktoppane for the internal frames
        */
    protected JDesktopPane desktop = new JDesktopPane();
    
    /** Contains the mapping between GPDocument objects
        *  and GPInternalFrames.
        *
        */
    protected Hashtable doc2InternalFrame = new Hashtable();
    
    /** A factory for the menu, tool and popup bars
        *
        */
    protected GPBarFactory barFactory = new GPBarFactory(this);
    
    /** The current Toolbar for this graphpad
        */
    protected JPanel toolBarMainPanel = new JPanel(new BorderLayout());
    
    /** The current Toolbar for this graphpad
        */
    protected JPanel toolBarInnerPanel;
    
    /** The current Menubar for this graphpad
        */
    protected JMenuBar menubar;
    
    /** The current Statusbar for this Graphpad instance
        */
    protected GPStatusBar statusbar;
    
    /** ActionMap contains all default Actions for
        *  this application
        *
        */
    protected ActionMap defaultActionMap = new ActionMap();
    
    /** The main Panel with the status bar
        *  and the desktop pane
        */
    protected JPanel mainPanel = new JPanel(new BorderLayout());
    
    /** ActionMap contains the current ActionMap.
        *
        */
    protected ActionMap currentActionMap = defaultActionMap;
    
    /** GPMarqueeHandler marqueeHandler
        */        
    protected GPMarqueeHandler marqueeHandler = new GPMarqueeHandler(this);
    
    /** GPConfiguration configuration
        */        
    protected GPConfiguration configuration = null;
    
    /** UMClipObj clip_obj
        */        
    public UMClipObj clip_obj;
    
    public String fontStyle;  // indicates what current font the text is in, starts off as normal
    
    public boolean rulerSelected;  // indicates whether ruler option is selected for this graphpad, starts false
    
    private TPGPDocTransferHandler tpgpTransferHandler = new TPGPDocTransferHandler( this );
    
    /** Creates a new default instance
        */
    public GPGraphpad() {
        this(null);
        fontStyle = "normal";
        rulerSelected = false;
    }
    
    /** Creates a new instance with the
        *  configuration from the config object.
        *
        *  @param config The configuration object.
        */
    public GPGraphpad(GPConfiguration config) {
        super(true);
        
        // procedure the configuration object
        configuration = config;
        
        // set the user actionmap if possible
        if (config != null && config.getActionMap() != null) {
            currentActionMap = config.getActionMap();
            currentActionMap.setParent(defaultActionMap);
        }
        
        // set the look and feel
        try {
            // Kunststoff makes scrolling extremely sluggish
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            
        } catch (Exception exc) {
            try {
                UIManager.setLookAndFeel(
                                         UIManager.getCrossPlatformLookAndFeelClassName());
            } catch (Exception exc2) {
                System.err.println("Error loading L&F: " + exc);
            }
        }
        
        setBorder(BorderFactory.createEtchedBorder());
        setLayout(new BorderLayout());
        
        // create the statusbar
        createStatusBar();
        
        // use and fill the action map
        fillDefaultActionMap();
        
        // set my reference to the actions
        setMe4Actions();
        
        // build the menu and the toolbar
        menubar = barFactory.createMenubar();
        
        toolBarInnerPanel = barFactory.createToolBars(toolBarMainPanel);
        setToolBarsVisible(true);
        
        add(BorderLayout.NORTH, menubar);
        add(BorderLayout.CENTER, mainPanel);
        add(BorderLayout.SOUTH, statusbar);
        
        //  JFrame frame = createFrame();
        //  frame.setIconImage(applicationIcon.getImage());
        //  frame.setTitle(appTitle);
        //  frame.getContentPane().add(this);
        //  frame.addWindowListener(new AppCloser());
        //
        //  frame.pack();
        //  frame.show();
        //
        //  frame.setSize(frameWidth, frameHeight);
        //  frame.setLocation(20,20);
        //  if (frameMaximize) {
        //   frame.setLocation(0, 0);
        //   frame.setSize(Toolkit.getDefaultToolkit().getScreenSize());
        //  }
        //
        //  update();
        
    }
    
    /** GPGraphpad uses the static initializer now.
        *  Please do not use this method.
        *  We will delete the method in one of the
        *  next versions.
        *
        * @deprecated dp
        */
    public static void init() {
    }
    /**
        *  The initializes the common values
     *
     */
    static {
        try {
            String vers = System.getProperty("java.version");
            if (vers.compareTo("1.4") < 0) {
                System.out.println(
                                   "!!!WARNING: JGraphpad must be run with a "
                                   + "1.4 or higher version VM!!!");
            }
            appTitle = Translator.getString("Title");
            
            // Logo
            String iconName = Translator.getString("Logo");
            logoIcon = ImageLoader.getImageIcon(iconName);
            
            // Icon
            iconName = Translator.getString("Icon");
            applicationIcon = ImageLoader.getImageIcon(iconName);
            
            frameWidth =
                new Integer(Translator.getString("FrameWidth")).intValue();
            frameHeight =
                new Integer(Translator.getString("FrameHeight")).intValue();
            frameMaximize =
                new Boolean(Translator.getString("FrameMaximize"))
                .booleanValue();
            
        } catch (Throwable t) {
            System.out.println("uncaught exception: " + t);
            t.printStackTrace();
        }
    }
    
    /** Creates a frame for this Graphpad panel
        * @return JFrame
        */
    protected JFrame createFrame() 
    {
        JFrame frame = new JFrameP() 
    {
            public String getInstanceID() {
                return this.getClass().getName();
            }
            public String getNameSpace() {
                return "JGraphPad";
            }
    };
        return frame;
    }
    
    /** fillDefaultActionMap
        */
    public void fillDefaultActionMap() {
        for (int i = 0; i < defaultActions.length; i++) {
            defaultActionMap.put(
                                 defaultActions[i].getValue(Action.NAME),
                                 defaultActions[i]);
        }
    }
    
    /** setMe4Actions
        */        
    protected void setMe4Actions() {
        Object[] keys;
        keys = defaultActionMap.allKeys();
        for (int i = 0; i < keys.length; i++) {
            Action a = defaultActionMap.get(keys[i]);
            if (a instanceof AbstractActionDefault) {
                AbstractActionDefault ad = (AbstractActionDefault) a;
                if (ad.getGraphpad() == null)
                    ad.setGraphpad(this);
            }
        }
        
        if (currentActionMap == defaultActionMap)
            return;
        
        keys = currentActionMap.allKeys();
        for (int i = 0; i < keys.length; i++) {
            Action a = currentActionMap.get(keys[i]);
            if (a instanceof AbstractActionDefault) {
                AbstractActionDefault ad = (AbstractActionDefault) a;
                if (ad.getGraphpad() == null)
                    ad.setGraphpad(this);
            }
        }
    }
    
    /** Returns the current Action Map
        * @return ActionMap
        */
    public ActionMap getCurrentActionMap() {
        return currentActionMap;
    }
    
    /** creates a AppCloser instance and returns this
        *  instance
        * @return WindowAdapter
        */
    protected WindowAdapter createAppCloser() {
        return new AppCloser();
    }
    
    /**
     * To shutdown when run as an application.  
     * If a save is needed a dialog is shown to save the file.
     */
    protected final class AppCloser extends WindowAdapter {
        /** The event when a window is closed.
        * @param e window event
        */            
        public void windowClosing(WindowEvent e)
				{
        	// The FileExit class will close the application if the user decides not to save or to save and
        	//exit. If the user says cancel, nothing will happen.
        	currentActionMap.get(
        			Utilities.getClassNameWithoutPackage(FileExit.class)).actionPerformed(null);
        }
    }
    
    /** Find the hosting frame, for the file-chooser dialog.
        * @return Frame
        */
    public Frame getFrame() {
        for (Container p = getParent(); p != null; p = p.getParent()) {
            if (p instanceof Frame) {
                return (Frame) p;
            }
        }
        return null;
    }
    
    /** Retrieves the Menu bar system.
        * @return JMenuBar
        */        
    public JMenuBar getMenubar() {
        return menubar;
    }
    
    /** Create a status bar
        * @return GPStatusBar
        */
    protected GPStatusBar createStatusBar() {
        statusbar = new GPStatusBar();
        return statusbar;
    }
    
    /** Retrieves the current Status Bar.
        * @return GPStatusBar
        */        
    public GPStatusBar getStatusBar() {
        return statusbar;
    }
    
    public TPGPDocTransferHandler getTPGPDocTransferHandler() {
        return tpgpTransferHandler;
    }
    
    /** Show a dialog with the given error message.
        * @param message String
        */
    public void error(String message) {
        JOptionPane.showMessageDialog(
                                      this,
                                      message,
                                      appTitle,
                                      JOptionPane.ERROR_MESSAGE);
    }
    
    // --- actions -----------------------------------
    
    /**
        * Actions defined by the Graphpad class
     */
    protected Action[] defaultActions =
    {
        new EditCell(this),
        new EditCopy(this),
        new EditCut(this),
        new EditDelete(this),
        new EditFind(this),
        new EditFindAgain(this),
        new EditInsertIntoLibrary(this),
        new EditPaste(this),
        new EditRedo(this),
        new EditUndo(this),
        new FileClose(this),
        new FileExit(this),
        new FileImportGXL(this),
        new FileImportSimple(this),
        new FileExportGraphviz(this),
        new FileExportImageMap(this),
        new FileExportJPG(this),
        new FileExportPNG(this),
        new FileExportGXL(this),
        new FileLibraryClose(this),
        new FileLibraryNew(this),
        new FileLibraryOpen(this),
        new FileLibraryRename(this),
        new FileLibrarySaveAs(this),
        new FileNew(this),
        new FileNewView(this),
        new FileOpen(this),
        new FilePageFormat(this),
        new FilePrint(this),
        new FileRecent1(this), //Recent files feature added 4/5/2003
        new FileRecent2(this), //Recent files feature added 4/5/2003
        new FileRecent3(this), //Recent files feature added 4/5/2003
        new FileRecent4(this), //Recent files feature added 4/5/2003
        new FileSave(this),
        new FileSaveAs(this),
        new FormatAlignMiddle(this),
        new FormatAlignBottom(this),
        new FormatAlignCenter(this),
        new FormatAlignLeft(this),
        new FormatAlignRight(this),
        new FormatAlignTop(this),
        new FormatBorderColor(this),
        new FormatBorderColorList(this),
        new FormatBorderNo(this),
        new FormatFillColor(this),
        new FormatFillColorList(this),
        new FormatFillNo(this),
        new FormatShapeImage(this),
        new FormatShapeNoImage(this),
        //new FormatLineBezier(this),
        //new FormatLineColor(this),
        //new FormatLineColorList(this),
        //new FormatLineEndList(this),
        //new FormatLineOrthogonal(this),
        //new FormatLinePattern(this),
        //new FormatLinePatternList(this),
        //new FormatLineQuadratic(this),
        //new FormatLineStartList(this),
        //new FormatLineWidth(this),
        //new FormatLineWidthList(this),
        //new FormatLockNotBendable(this),
        //new FormatLockNotConnectable(this),
        //new FormatLockNotDisconnectable(this),
        //new FormatLockPosition(this),
        //new FormatLockSize(this),
        //new FormatLockValue(this),
        new FormatReverse(this),
        //new FormatRoutingNo(this),
        //new FormatRoutingSimple(this),
        new FormatSizeAuto(this),
        new FormatSizeManual(this),
        //new FormatStyleEndSize(this),
        //new FormatStyleNoStartEnd(this),
        //new FormatStyleStartSize(this),
        new FormatTextFont(this),
        new FormatTextFontColor(this),
        new FormatTextFontColorList(this),
        new FormatTextFontSize(this),
        new FormatTextFontSizeList(this),
        new FormatTextFontStyleBold(this),
        new FormatTextFontStyleItalic(this),
        new FormatTextFontStyleNormal(this),
        new FormatTextFontStyleUnderline(this),
        new FormatTextPositionBottom(this),
        new FormatTextPositionCenter(this),
        new FormatTextPositionLeft(this),
        new FormatTextPositionMiddle(this),
        new FormatTextPositionRight(this),
        new FormatTextPositionTop(this),
        //new FormatUnlockBendable(this),
        //new FormatUnlockConnectable(this),
        //new FormatUnlockConnections(this),
        //new FormatUnlockDisconnectable(this),
        //new FormatUnlockPosition(this),
        //new FormatUnlockSize(this),
        //new FormatUnlockValue(this),
        
        //new GraphApplyLayoutAlgorithm(this),
        //new GraphArrange(this),
        new FormatBackgroundColor(this),
        new FormatBackgroundImage(this),
        new FormatBackgroundNoImage(this),
        //new GraphDragEnabled(this),
        //new GraphDropEnabled(this),
        //new GraphEditable(this),
        //new GraphEnabled(this),
        //new GraphLayout(this),
        //new GraphOptionsBendable(this),
        //new GraphOptionsCloneable(this),
        //new GraphOptionsConnectable(this),
        //new GraphOptionsDisconnectable(this),
        //new GraphOptionsDisconnectOnMove(this),
        //new GraphOptionsDoubleBuffered(this),
        //new GraphOptionsMoveable(this),
        //new GraphOptionsSizeable(this),
        //new GraphTilt(this),
        
        
        new HelpAbout(this),
        //ADDED by Will 4/4/03
        new HelpHelpTopics(this),
        new SelectAll(this),
        new SelectAllClear(this),
        
        //new SelectEdges(this),
        //new SelectEdgesClear(this),
        new SelectInverse(this),
        //new SelectMinimalSpanTree(this),
        //new SelectShortestPath(this),
        //new SelectVertices(this),
        //new SelectVerticesClear(this),
        
        new ShapeAlignBottom(this),
        new ShapeAlignCenter(this),
        new ShapeAlignLeft(this),
        new ShapeAlignMiddle(this),
        new ShapeAlignRight(this),
        new ShapeAlignTop(this),
        
        //new ShapeBestPorts(this),
        //new ShapeCloneAttributes(this),
        //new ShapeCloneLabel(this),
        //new ShapeCloneSize(this),
        //new ShapeConnect(this),
        //new ShapeDefaultPorts(this),
        //new ShapeDisconnect(this),
        
        new ShapeGroup(this),
        new ShapeToBack(this),
        new ShapeToFront(this),
        new ShapeUngroup(this),
        new ToolBoxEdge(this),
        new ToolBoxEllipse(this),
        new ToolBoxIcon(this),
        new ToolBoxLine(this),
        new ToolBoxRectangle(this),
        new ToolBoxSelect(this),
        new ToolBoxText(this),
        new ToolBoxZoomArea(this),
        new ToolsGridSize(this),
        new WindowLookAndFeel(this),
        new ToolsMetric(this),
        new ToolsPortSize(this),
        new ToolsShowOverview(this),
        new ViewShowStatusbar(this),
        new ViewShowToolbar(this),
        new ToolsSnapSize(this),
        new ToolsStatusbar(this),
        new ToolsTooltips(this),
        new ViewActualSize(this),
        new ViewAntiAlias(this),
        new ViewFit(this),
        new ViewGrid(this),
        new ViewLayout(this),
        new ViewPorts(this),
        new ViewRuler(this),
        new ViewScaleCustom(this),
        new ViewScaleZoomIn(this),
        new ViewScaleZoomOut(this),
        new ViewSlideShow(this), // Slide show feature added on 3/31/03
        new WindowCascade(this),
        new WindowLogConsole(this),
        new WindowMaximize(this),
        new WindowMinimize(this),
        new WindowWindows(this),
        
        new TPBookAddSlide( this ),
        new TPBookRemoveSlide( this ) };
    
    /**
        * Returns the current graph.
     * @return GPGraph
     */
    public GPGraph getCurrentGraph() {
        GPDocument doc = getCurrentDocument();
        if (doc == null)
            return null;
        return doc.getGraph();
    }
    
    /** Retrieves the current graph model provider.
        * @return GraphModelProvider
        */        
    public GraphModelProvider getCurrentGraphModelProvider() {
        GPDocument doc = getCurrentDocument();
        if (doc == null)
            return null;
        return doc.getGraphModelProvider();
    }
    
    /**
        * Returns the currently selected document.
     * If one is not selected, the first one encountered is selected.
     * @return GPDocument
     */
    public GPDocument getCurrentDocument() {
        GPInternalFrame internalFrame =
        (GPInternalFrame) desktop.getSelectedFrame();
        if (internalFrame == null) {
            JInternalFrame[] frames = desktop.getAllFrames();
            if (frames.length > 0) {
                try {
                    frames[0].setSelected(true);
                    internalFrame = (GPInternalFrame) frames[0];
                } catch (PropertyVetoException e) {
                    return null;
                }
            }
        }
        if (internalFrame == null)
            return null;
        return internalFrame.getDocument();
    }
    
    /**
        * Returns all of the documents.
     * @return GPGraphTools or <code>null</code> if no documents
     */
    public GPDocument[] getAllDocuments() {
        JInternalFrame[] frames = desktop.getAllFrames();
        
        if (frames != null && frames.length > 0) {
            Vector docs = new Vector();
            for (int i = 0; i < frames.length; i++) {
                // make sure to only pick up GPInternalFrame instances
                if (frames[i] instanceof GPInternalFrame) {
                    docs.add(((GPInternalFrame) frames[i]).getDocument());
                }
            }
            return (GPDocument[]) docs.toArray(new GPDocument[docs.size()]);
        } else
            return null;
    }
    
    /**
        * Returns the graphTools.
     * @return GPGraphTools
     */
    public static GPGraphTools getGraphTools() {
        return graphTools;
    }
    
    /**
        * Returns the undoAction.
     * @return UndoAction
     */
    public AbstractActionDefault getEditUndoAction() {
        return (AbstractActionDefault) currentActionMap.get(
                                                            Utilities.getClassNameWithoutPackage(
                                                                                                 EditUndo.class));
    }
    
    /**
        * Returns the redoAction.
     *
     * @return RedoAction
     */
    public AbstractActionDefault getEditRedoAction() {
        return (AbstractActionDefault) currentActionMap.get(
                                                            Utilities.getClassNameWithoutPackage(
                                                                                                 EditRedo.class));
    }
    
    /** Retrieves the current Log Console.
        * @return GPLogConsole
        */        
    public GPLogConsole getLogConsole() {
        return logger;
    }
    
    /** Verifies whether the tool bars are visible.
        * @return Boolean
        */        
    public boolean isToolBarsVisible() {
        return this.toolBarsVisible;
    }
    
    /** Sets the Toolbars such that they are now visible.
        * @param state boolean
        */        
    public void setToolBarsVisible(boolean state) {
        this.toolBarsVisible = state;
        
        if (state == true) {
            mainPanel.remove(desktop);
            toolBarInnerPanel.add(BorderLayout.CENTER, desktop);
            mainPanel.add(BorderLayout.CENTER, toolBarMainPanel);
        } else {
            mainPanel.remove(toolBarMainPanel);
            toolBarInnerPanel.remove(desktop);
            mainPanel.add(BorderLayout.CENTER, desktop);
        }
        desktop.repaint();
    }
    
    /** Adds a new Internal Frame to the Graphpad
        * @param f GPInternalFrame
        */
    public void addGPInternalFrame(GPInternalFrame f) {
        desktop.add(f);
        try {
            f.setSelected(true);
        } catch (Exception ex) {
        }
        doc2InternalFrame.put(f.getDocument(), f);
    }
    
    /** removes the specified Internal Frame from the Graphpad
        * @param f GPInternalFrame
        */
    public void removeGPInternalFrame(GPInternalFrame f) {
        if (f == null)
            return;
        f.setVisible(false);
        desktop.remove(f);
        doc2InternalFrame.remove(f.getDocument());
        JInternalFrame[] frames = desktop.getAllFrames();
        if (frames.length > 0) {
            try {
                frames[0].setSelected(true);
            } catch (PropertyVetoException e) {
            }
        }
    }
    
    /** Adds a new Document based on the GraphModelProvider.
        * @param graphModelProvider the current graph model provider
        */
    public void addDocument(GraphModelProvider graphModelProvider) {
        addDocument(null, graphModelProvider, null, null, null, null);
    }
    
    /** Opens a new document frame based on the parameters
     * @param filename String
     * @param graphModelProvider graph model provider
     * @param gpGraph graphpad graph
     * @param model GraphModel
     * @param undo GraphUndoManager
     */
    public void addDocument(
    		String filename,
				GraphModelProvider graphModelProvider,
				GPGraph gpGraph,
				GraphModel model,
				GraphUndoManager undo) {

    	if (graphModelProvider == null)
    		return;

    	if (model == null)
    		model = graphModelProvider.createCleanGraphModel();

    	if (gpGraph == null) {
    		gpGraph = graphModelProvider.createCleanGraph(model);
    	}

    	gpGraph.setSelectNewCells(true);
    	gpGraph.setInvokesStopCellEditing(true);
    	gpGraph.setMarqueeHandler(getMarqueeHandler());

    	GPDocument doc =
    		new GPDocument(
    				this,
						filename,
						graphModelProvider,
						gpGraph,
						model,
						null);

    	GPInternalFrame iframe = new GPInternalFrame(doc);

    	iframe.show();

    	addGPInternalFrame(iframe);

    }
    
    /** You can add a document by giving the filename.
        * Before you can add a document with the specified filename
        * you must add the corresponding GraphModelProvider and the
        * corresponding GraphModelFileFormat at the GraphModelProviderRegistry.
        *
        * @see com.jgraph.pad.GraphModelProviderRegistry
        * @see com.jgraph.pad.GraphModelProvider
        * @see com.jgraph.pad.GraphModelFileFormat
        * @param filename String
        */
    public void addDocument(String filename) {
        GraphModelProvider graphModelProvider =
        GraphModelProviderRegistry.getGraphModelProvider(filename);
        GraphModelFileFormat fileFormat =
            GraphModelProviderRegistry.getGraphModelFileFormat(filename);
        GraphModel model = graphModelProvider.createCleanGraphModel();
        GPGraph gpGraph = graphModelProvider.createCleanGraph(model);
        try {
            fileFormat.read(filename, null, gpGraph);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                                          getFrame(),
                                          e,
                                          Translator.getString("Error"),
                                          JOptionPane.ERROR_MESSAGE);
        }
        addDocument(filename, graphModelProvider, gpGraph, model, null, null );
    }
    
    /** Opens a new document frame based on the parameters
        * @param filename String
        * @param graphModelProvider graph model provider
        * @param gpGraph graphpad graph
        * @param model GraphModel
        * @param undo GraphUndoManager
        */
    public void addDocument(
                            String filename,
                            GraphModelProvider graphModelProvider,
                            GPGraph gpGraph,
                            GraphModel model,
                            GraphUndoManager undo, List gpGraphList ) {
        
        if (graphModelProvider == null)
            return;
        
        if (model == null)
            model = graphModelProvider.createCleanGraphModel();
        
        if (gpGraph == null) {
            gpGraph = graphModelProvider.createCleanGraph(model);
        }
        
        if( gpGraphList == null ) {
            gpGraphList = new LinkedList();
            gpGraphList.add( graphModelProvider.createCleanGraph( model ) );
        }
        
        gpGraph.setSelectNewCells(true);
        gpGraph.setInvokesStopCellEditing(true);
        gpGraph.setMarqueeHandler(getMarqueeHandler());
        
        GPDocument doc =
            new TPDocument(
                           this,
                           filename,
                           graphModelProvider,
                           gpGraph,
                           model,
                           null, gpGraphList );
        
        GPInternalFrame iframe = new GPInternalFrame(doc);
        
        iframe.show();
        
        addGPInternalFrame(iframe);
        
    }
    
    /** Removes the Document
        * @param doc GPDocument
        */        
    public void removeDocument(GPDocument doc) {
        GPInternalFrame iFrame = (GPInternalFrame) doc2InternalFrame.get(doc);
        removeGPInternalFrame(iFrame);
    }
    
    /** Updates the graphpad.
        */        
    public void update() {
        GPDocument currentDoc = getCurrentDocument();
        
        Object[] keys = currentActionMap.keys();
        for (int i = 0; i < keys.length; i++) {
            Action a = currentActionMap.get(keys[i]);
            if (a instanceof AbstractActionDefault) {
                ((AbstractActionDefault) a).update();
            } else {
                if (currentDoc == null) {
                    a.setEnabled(false);
                } else {
                    a.setEnabled(true);
                }
            }
        }
        
        if (currentActionMap == defaultActionMap)
            return;
        
        keys = defaultActionMap.keys();
        for (int i = 0; i < keys.length; i++) {
            Action a = defaultActionMap.get(keys[i]);
            if (a instanceof AbstractActionDefault) {
                ((AbstractActionDefault) a).update();
            }
        }
    }
    
    /**
        * Returns the barFactory.
     * @return GPBarFactory
     */
    public GPBarFactory getBarFactory() {
        return barFactory;
    }
    
    /**
        * Sets the barFactory.
     * @param barFactory The barFactory to set
     */
    public void setBarFactory(GPBarFactory barFactory) {
        this.barFactory = barFactory;
    }
    
    /** Retrieves all frames from the graphpad.
        * @return JInternalFrame
        */        
    public JInternalFrame[] getAllFrames() {
        return desktop.getAllFrames();
    }
    
    /** Sets the parent's action map.
        * @param map ActionMap
        */        
    public void setParentActionMap(ActionMap map) {
        defaultActionMap.setParent(map);
    }
    /**
        * Returns the applicationIcon.
     * @return ImageIcon
     */
    public static ImageIcon getApplicationIcon() {
        return applicationIcon;
    }
    
    /**
        * Sets the applicationIcon.
     * @param applicationIcon The applicationIcon to set
     */
    public static void setApplicationIcon(ImageIcon applicationIcon) {
        GPGraphpad.applicationIcon = applicationIcon;
    }
    
    /**
        * Returns the marqueeHandler.
     * @return GPMarqueeHandler
     */
    public GPMarqueeHandler getMarqueeHandler() {
        return marqueeHandler;
    }
    
    /** Adds a Desktop Container Listener.
        * @param listener ContainerListener
        */        
    public void addDesktopContainerListener(ContainerListener listener) {
        desktop.addContainerListener(listener);
    }
    
    /** Removes the Desktop Container listener.
        * @param listener ContainerListener
        */        
    public void removeDesktopContainerListener(ContainerListener listener) {
        desktop.removeContainerListener(listener);
    }
    /**
        * Determines whether the frame inside which the TerpPresent application will be displayed is 
     * maximized or not.
     * 
     * @return Returns the frameMaximize. True if the frame is maxmized, false otherwise.
     * @version 3.0
     */
    public static boolean isFrameMaximize()
    {
        return frameMaximize;
    }
    
    /**
        * Returns the width of the frame inside which the TerpPresent application will be displayed.
     * 
     * @return Returns the frameWidth.
     * @version 3.0
     */
    public static int getFrameWidth()
    {
        return frameWidth;
    }
    /**
        * Returns the height of the frame inside which the TerpPresent application will be displayed.
     * 
     * @return Returns the frameHeight.
     * @version 3.0
     */
    public static int getFrameHeight()
    {
        return frameHeight;
    }

	/*************** Xun : User Defined Methods **************/
	/** getDebugMethod
	  * This function is to provide JavaGUIReplayer methods
	  * to get extra information of the graphpad
	  */
	public HashMap getDebugMethod() {
		try {
			HashMap hp = new HashMap();
			hp.put("GPGraphpadContent_FILE", (Method)(this.getClass().getMethod("getGPGraphpadContent", null)));
			return hp;
		} catch (NoSuchMethodException e) {
			return null;
		}
	}

	public Vector getGPGraphpadContent() {
       	Vector result = new Vector();
		if (getCurrentDocument().getModel().getRootCount() >= 0) {
			if(getCurrentGraph() != null){
        		BufferedImage outputBuffer = GPConverter.toImage(getCurrentGraph());
        		String line = new String();
        		if(outputBuffer != null){
					int imageHeight = outputBuffer.getHeight();
	        		int imageWidth = outputBuffer.getWidth();
	        		try{
		        		for(int i = 0; i < imageHeight; i++){
   		            		line = "";
							for(int j = 0; j < imageWidth; j++){
					    		line += outputBuffer.getRGB(j, i) + " ";
							}
							line += "\n";
							result.addElement(line);
				 		}
			 		} catch(Exception e){
			     		e.printStackTrace();
			     		System.out.println("Wrong with writing iconimage into file!");
			 		}
				}
			}		
		}

		return result;
	}

	/** getReplayableMethod
	  * This function is to provide JavaGUIRipper to get 
	  * the replayable method names. Then when replaying on
	  * this component, JavaGUIReplayer can directly call 
	  * these methods.
	  */
	public Vector getReplayableMethod() {
		Vector replayableMethods = new Vector();
		replayableMethods.add(new String("drawLine"));
		return replayableMethods;
	}

    //MouseDrag on this GPGraphpad whatever currentTool is
    public void drawLine() {
		int w = this.getCurrentDocument().getInternalFrame().getWidth();
		int h = this.getCurrentDocument().getInternalFrame().getHeight();
		System.out.println("WWW:w="+w+" h="+h);

		MouseEvent mouseMoveEvent = new MouseEvent((Component)(this.getCurrentGraph()), MouseEvent.MOUSE_MOVED, System.currentTimeMillis(), 16, 5, 10, 1, false, MouseEvent.BUTTON1);
		try {
		    //((Component)(this.getCurrentGraph())).dispatchEvent(mouseMoveEvent);
			this.getMarqueeHandler().mouseMoved(mouseMoveEvent);
		} catch (Exception e) {
			System.out.println("Center:drawLine failed!");
			e.printStackTrace();
		 }

        MouseEvent mousePressEvent = new MouseEvent((Component)(this.getCurrentGraph()), MouseEvent.MOUSE_PRESSED, System.currentTimeMillis(), 16, 5, 10, 1, false, MouseEvent.BUTTON1);
        try {
		    //this.getCurrentGraph().dispatchEvent(mousePressEvent);
			this.getMarqueeHandler().mousePressed(mousePressEvent);
        } catch (Exception e) {
            System.out.println("Center:drawLine failed!");
            e.printStackTrace();
        }

        MouseEvent mouseDragEvent = new MouseEvent((Component)(this.getCurrentGraph()), MouseEvent.MOUSE_DRAGGED, System.currentTimeMillis(), 16, w/5, h/4, 0, false, MouseEvent.BUTTON1);
        try {
		    //this.getCurrentGraph().dispatchEvent(mouseMoveEvent);
			this.getMarqueeHandler().mouseDragged(mouseDragEvent);
        } catch (Exception e) {
            System.out.println("Center:drawLine failed!");
            e.printStackTrace();
        }

        mouseDragEvent = new MouseEvent((Component)(this.getCurrentGraph()), MouseEvent.MOUSE_DRAGGED, System.currentTimeMillis(), 16, w/2, h/2, 0, false, MouseEvent.BUTTON1);
        try {
		    //this.getCurrentGraph().dispatchEvent(mouseMoveEvent);
			this.getMarqueeHandler().mouseDragged(mouseDragEvent);
		} catch (Exception e) {
		    System.out.println("Center:drawLine failed!");
		    e.printStackTrace();
		}

		MouseEvent mouseReleaseEvent = new MouseEvent((Component)(this.getCurrentGraph()), MouseEvent.MOUSE_RELEASED, System.currentTimeMillis(), 16, w/3, h/2, 1, false, MouseEvent.BUTTON1);
		try {
		    //this.getCurrentGraph().dispatchEvent(mouseMoveEvent);
			this.getMarqueeHandler().mouseReleased(mouseReleaseEvent);
		 } catch (Exception e) {
		     System.out.println("Center:drawLine failed!");
		     e.printStackTrace();
		 }

       	this.update();
	}

}
