import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;

//import javax.swing.Action;
import javax.swing.JInternalFrame;


/**
 * The graphpad internal frame is a container
 * is a container for one GPDocument.
 * There is a split between a document and
 * the joint internal frame, sothat you can use
 * the document without using internal frames.
 *
 * @author sven.luzar
 * @version 2.0 TerpPresent
 *
 */
public class GPInternalFrame extends JInternalFrame {

	/** A link to the Graphpad Document of this frame.
	 */
	GPDocument document;

	/** Constructor for GPInternalFrame.
         * @param document a GPDocument
         */
	public GPInternalFrame(GPDocument document) {
		super(document.getFilename(), true, true, true, true);
		this.setFrameIcon(GPGraphpad.getApplicationIcon());
		this.document = document;
		this.document.setInternalFrame(this);
		//changing the internal slide size from 600x400 to 976x636
		//slide can now hold a background of 800x600 by default
		//this.setPreferredSize(new Dimension(976, 636)); Xun
		this.setPreferredSize(new Dimension(150, 120));
		this.getContentPane().add(document);
		this.addVetoableChangeListener(new GPVetoableListner(document));
		this.pack();
	}

	/**
	 * Returns the document.
	 * @return GPDocument
	 */
	public GPDocument getDocument() {
		return document;
	}

	/**
	 * Sets the document.
	 * @param document The document to set
	 */
	public void setDocument(GPDocument document) {
		this.remove(this.document);
		this.document = document;
		this.add(this.document);
		this.pack();
	}

}
/** A listener for vetoable.
 */
class GPVetoableListner implements VetoableChangeListener {

    /** A GPDocument.
     */    
	GPDocument document;

        /** Constructor for GPVetoableListner.
         * @param doc a document
         */        
	GPVetoableListner(GPDocument doc) {
		this.document = doc;
	}
	/** Change a vetoable.
         * @see javax.swing.event.InternalFrameListener#internalFrameClosing(InternalFrameEvent)
         * @param evt an property changing event
         * @throws PropertyVetoException thrown when internal frame can't be closed
         */
	public void vetoableChange(PropertyChangeEvent evt)
		throws PropertyVetoException {
		if (evt.getPropertyName() != JInternalFrame.IS_CLOSED_PROPERTY)
			return;

		if (((Boolean)evt.getNewValue()).booleanValue() && document.close(true)){
				document.getGraphpad().removeDocument(document);
		} else {
			throw new PropertyVetoException("Can't close the Internal Frame", evt) ;
		}
	}

}
