/*
 * @(#)GPLibraryPanel.java	1.2 11/11/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.Serializable;
import java.util.Map;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.Border;

//@jdk1.4 removed
//import com.jgraph.plaf.basic.TransferHandler;

//@jdk1.4 removed
//public class GPLibraryPanel extends TransferHandler.JAdapterComponent {
//@jdk1.4 added

/** The library panel.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */
public class GPLibraryPanel extends JComponent {

    /** Maximum size of library entries.
     */    
	protected int MAX = 60;

        /** Scrollable panel.
         */        
	protected ScrollablePanel panel;
        /** Selected graph cell component.
         */        
	protected GraphCellsComponent selected = null;
        /** Old border.
         */        
	protected Border oldBorder = null;
        /** Boolean for draging.
         */        
	protected boolean dragging = false;
        /** A new Jcombobox.
         */        
	protected JComboBox combo = new JComboBox();

        /** Boolean flag for access right.
         */        
	public boolean fullLibraryAccess = true;


        /** Constructor for the GPLibraryPanel.
         * @param name name of the panel
         * @param files an array of file names
         * @param entrySize the entry size
         */        
	public GPLibraryPanel(String name, String[] files, int entrySize) {
		MAX = entrySize;
		setLayout(new BorderLayout());
		setPreferredSize(new Dimension(2 * MAX + 35, 0));
		// Don't forget to uncomment for the Java 1.3 release

		setTransferHandler(new DNDTransferHandler());
		add(combo, BorderLayout.NORTH);
		addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				setSelected(selected);
			}

			public void focusLost(FocusEvent e) {
				setSelected(selected);
			}

		});
		// Combo Box Handler (Library Switching)
		combo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Object item = combo.getSelectedItem();
				if (item instanceof ScrollablePanel) {
					if (panel != null) {
						remove(panel.getParent().getParent());
						panel.getParent().getParent().remove(panel);
					}
					panel = (ScrollablePanel) item;
					JScrollPane scrollPane = new JScrollPane(panel);
					scrollPane.getViewport().setBackground(Color.white);
					add(scrollPane, BorderLayout.CENTER);
					// Add to each Lib!
					panel.getParent().addMouseListener(new MouseAdapter() {
						public void mousePressed(MouseEvent e) {
							setSelected(null);
							requestFocus();
						}
						public void mouseReleased(MouseEvent e) {
							if (isPopupTrigger(e))
								showPopupMenu(panel, e.getPoint());
						}
					});
					validate();
				}
			}
		});
		if (files != null && files.length > 0) {
			for (int i = 0; i < files.length; i++)
				openLibrary(files[i]);
		}
		if (files == null && name == null)
			name = "No Name";
		if (name != null && name.length() > 0)
			addLibrary(name);

		fullLibraryAccess =
			new Boolean(Translator.getString("FullLibraryAccess"))
				.booleanValue();

	}

        /** Check if is a popup trigger.
         * @param e a mouse event
         * @return boolean for popup trigger
         */        
	protected boolean isPopupTrigger(MouseEvent e) {
		return SwingUtilities.isRightMouseButton(e);
	}

        /** Show the popup menu.
         * @param c a component
         * @param p the point where the menu pops up
         */        
	public void showPopupMenu(Component c, Point p) {
		Container parent = this;
		do {
			parent = parent.getParent();
		} while (parent != null && !(parent instanceof GPGraphpad));
		GPGraphpad pad = (GPGraphpad) parent;
		if (pad != null) {
			JPopupMenu pop = pad.getBarFactory() .createLibraryPopupMenu() ;
			pop.show(c, p.x, p.y);
		}
	}

        /** Adds a library.
         * @param name the name of the library panel
         * @return a scrollable panel
         */        
	public ScrollablePanel addLibrary(String name) {
		ScrollablePanel tmp = new ScrollablePanel(name);
		combo.addItem(tmp);
		combo.setSelectedItem(tmp);
		return tmp;
	}

        /** Opens a library.
         * @param state the state of the current library
         */        
	public void openLibrary(Object state) {
		if (state instanceof Vector) {
			Vector v = (Vector) state;
			ScrollablePanel p = addLibrary(v.get(0).toString());
			for (int i = 1; i < v.size(); i++) {
				Object t = v.get(i);
				if (t instanceof Transferable)
					getTransferHandler().importData(this, (Transferable) t);
			}
		}
	}

        /** Close the current library.
         */        
	public void closeLibrary() {
		if (combo.getItemCount() > 1) {
			combo.removeItem(panel);
			combo.setSelectedIndex(0);
		}
	}

        /** Constructor for a scrollable panel.
         */        
	public class ScrollablePanel extends JPanel implements Scrollable {

            /** A GPGraph.
             */            
		protected GPGraph graph = new GPGraph();
                /** Name of the panel.
                 */                
		protected String name;

                /** Constructor for a scrollable panel.
                 * @param name name of the panel
                 */                
		public ScrollablePanel(String name) {
			super(new FixedWidthFlowLayout());
			setBackground(Color.white);
			this.name = name;
		}

                /** Get the preferred scrollable vew port size.
                 * @return the dimension of the view port
                 */                
		public Dimension getPreferredScrollableViewportSize() {
			return getPreferredSize();
		}

                /** Returns the scroll speed.
                 * @param visibleRect a visible rectangle
                 * @param orientation the orientation of the scroll
                 * @param direction the direction of the scroll
                 * @return the increment of the scroll
                 */                
		public int getScrollableBlockIncrement(
			Rectangle visibleRect,
			int orientation,
			int direction) {
			return 10;
		}

                /** Get the track port height.
                 * @return always return false
                 */                
		public boolean getScrollableTracksViewportHeight() {
			return false;
		}

                /** Get the track port width.
                 * @return always return true
                 */                
		public boolean getScrollableTracksViewportWidth() {
			return true;
		}

                /** Get the scroll unit increment.
                 * @param visibleRect the visible rectangle
                 * @param orientation the scroll orientation
                 * @param direction the scroll direction
                 * @return the increment
                 */                
		public int getScrollableUnitIncrement(
			Rectangle visibleRect,
			int orientation,
			int direction) {
			return 10;
		}

                /** Get the graph.
                 * @return a GPGraph object
                 */                
		public GPGraph getGraph() {
			return graph;
		}

                /** Set the name.
                 * @param name the name to be set to
                 */                
		public void setName(String name) {
			this.name = name;
		}

                /** Convert to string.
                 * @return the name of the current object
                 */                
		public String toString() {
			return name;
		}

                /** Get the archivable state.
                 * @return the serializable state
                 */                
		public Serializable getArchiveableState() {
			Vector v = new Vector();
			v.add(name);
			Component[] comps = getComponents();
			for (int i = 0; i < comps.length; i++)
				if (comps[i] instanceof GraphCellsComponent)
					v.add(((GraphCellsComponent) comps[i]).getTransferable());
			return v;
		}

	}

        /** Set the selected component.
         * @param comp the graph cells componenet
         */        
	public void setSelected(GraphCellsComponent comp) {
		if (selected != null && oldBorder != null)
			selected.setBorder(oldBorder);
		selected = comp;
		if (selected != null && hasFocus()) {
			oldBorder = selected.getBorder();
			selected.setBorder(BorderFactory.createLoweredBevelBorder());
		}
	}

        /** Delete from the libaray.
         */        
	public void delete() {
    	if (selected != null && fullLibraryAccess) {
    		GraphCellsComponent tmp = selected;
    		int r = JOptionPane.showConfirmDialog(null, "Delete from Library", "Really delete?", JOptionPane.YES_NO_OPTION);
    		if (r == JOptionPane.YES_OPTION) {
	    		GPTransferable t = tmp.getTransferable();
	    		panel.getGraph().getModel().remove(t.getCells());
	    		panel.remove(tmp);
	    		setSelected(null);
				  panel.revalidate();
				  repaint();
	    	}
	    }
	}

        /** Get the panel.
         * @return a scrollable panel
         */        
	public ScrollablePanel getPanel() {
		return panel;
	}

        /** Constructor for DNDTransferHandler.
         */        
	class DNDTransferHandler extends TransferHandler {

		/** Create a Transferable to use as the source for a data transfer.
                 *
                 * @return The representation of the data to be transfered.
                 * @param c a JComponent
                 */
		protected Transferable createTransferable(JComponent c) {
			if (selected != null) {
				dragging = true;
				return selected.getTransferable();
			}
			return null;
		}

                /** Get the source actions.
                 * @param c a JComponent
                 * @return copy or move boolean
                 */                
		public int getSourceActions(JComponent c) {
			return COPY_OR_MOVE;
		}

                /** Check if can be imported or not.
                 * @param comp the JComponent
                 * @param flavors an array of data flavor
                 * @return always returns true
                 */                
		public boolean canImport(JComponent comp, DataFlavor[] flavors) {
			return true;
		}

                /** Finishes export.
                 * @param source the source JComponent
                 * @param data the transferable data
                 * @param action the integer representation of an action
                 */                
		protected void exportDone(
			JComponent source,
			Transferable data,
			int action) {
			dragging = false;
		}

                /** Import data.
                 * @param comp a JComponent
                 * @param t transferable object
                 * @return status of the import
                 */                
		public boolean importData(JComponent comp, Transferable t) {
			if (t.isDataFlavorSupported(GraphTransferable.dataFlavor)
				&& !dragging) {
				try {
					Object obj =
						t.getTransferData(GraphTransferable.dataFlavor);
					if (obj instanceof GraphTransferable && !dragging) {
						GraphTransferable gt = (GraphTransferable) obj;
						Rectangle bounds = gt.getBounds();
						Map clones = panel.getGraph().cloneCells(gt.getCells());
						ConnectionSet cs = gt.getConnectionSet().clone(clones);
						Map attributes = gt.getAttributeMap();
						attributes =
							GraphConstants.replaceKeys(clones, attributes);
						Object[] cells = clones.values().toArray();
						panel.getGraph().getModel().insert(
							cells,
							attributes,
							cs,
							null,
							null);
						// Translate to (0,0)
						for (int i = 0; i < cells.length; i++) {
							CellView view =
								panel
									.getGraph()
									.getGraphLayoutCache()
									.getMapping(
									cells[i],
									true);
							if (attributes != null) {
								Map map = (Map) attributes.get(cells[i]);
								if (map != null) {
									map = GraphConstants.cloneMap(map);
									GraphConstants.translate(
										map,
										-bounds.x,
										-bounds.y);
									view.setAttributes(map);
								}
							}
						}
						// Create new Transferable
						Object[] orig = gt.getCells();
						cells = new Object[orig.length];
						for (int i = 0; i < orig.length; i++)
							cells[i] = clones.get(orig[i]);
						String text = gt.getPlainData();
						GPTransferable newt =
							new GPTransferable(
								text,
								cells,
								cs,
								attributes,
								bounds);
						panel.add(
							new GraphCellsComponent(
								panel.getGraph(),
								cells,
								newt));
						panel.revalidate();
					}
				} catch (Exception ex) {
					System.out.println("Error: " + ex.getMessage());
				}
			}
			return false;
		}
	}

        /** A Fixed Width Flow Layout.
         */        
	public static class FixedWidthFlowLayout extends FlowLayout {

            /** Constructor for FixedWidthFlowLayout.
             */            
		public FixedWidthFlowLayout() {
			super(FlowLayout.LEFT);
		}

                /** Get the preferred layout size.
                 * @param target the target container
                 * @return the dimension of the target container
                 */                
		public Dimension preferredLayoutSize(Container target) {
			synchronized (target.getTreeLock()) {
				int maxw = target.getParent().getSize().width;
				int vgap = getVgap();
				int hgap = getHgap();
				Dimension dim = new Dimension(maxw, 0);
				int nmembers = target.getComponentCount();
				int w = 0, h = 0;
				for (int i = 0; i < nmembers; i++) {
					Component m = target.getComponent(i);
					if (m.isVisible()) {
						Dimension d = m.getPreferredSize();
						w += d.width + hgap;
						if (w >= maxw) {
							dim.height += h + vgap;
							w = d.width + hgap;
							h = 0;
						}
						h = Math.max(h, d.height);
					}
				}
				Insets insets = target.getInsets();
				dim.height += h + insets.top + insets.bottom + vgap * 2;
				return dim;
			}
		}

	}

        /** A Graph Cells Component.
         */        
	public class GraphCellsComponent extends JComponent {

            /** A GPGraph.
             */            
		protected GPGraph graph;
                /** A GPTransferable.
                 */                
		protected GPTransferable trans;
                /** An array of cells.
                 */                
		protected Object[] cells;
                /** A real graph cell renderer.
                 */                
		protected RealGraphCellRenderer renderer;
                /** The scale.
                 */                
		protected double scale = 0.5;

                /** Constructor for GraphCellsComponent.
                 * @param graph the GPGraph
                 * @param cells the array of cells
                 * @param trans the GPTransferable
                 */                
		public GraphCellsComponent(
			GPGraph graph,
			Object[] cells,
			GPTransferable trans) {
			this.trans = trans;
			this.graph = graph;
			// Insert GraphCellRenderer
			setBorder(BorderFactory.createLineBorder(graph.getMarqueeColor()));
			CellView[] views =
				panel.getGraph().getGraphLayoutCache().getMapping(cells);
			renderer = new RealGraphCellRenderer(graph, views);
			renderer.setLocation(new Point(4, 4));
			Rectangle b = AbstractCellView.getBounds(views);
			int tmp = Math.max(b.width, b.height);
			if (tmp > MAX) {
				scale = (double) MAX / Math.max(b.width, b.height);
				renderer.setScale(scale);
				b.x *= scale;
				b.y *= scale;
				b.width *= scale;
				b.height *= scale;
			}
			setPreferredSize(new Dimension(b.width + 8, b.height + 8));
			renderer.setSize(getPreferredSize());
			add(renderer);
			addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					if (!isPopupTrigger(e)) {
						GPLibraryPanel.this.requestFocus();
						GPLibraryPanel.this.setSelected(
							GraphCellsComponent.this);
						GPLibraryPanel.this.getTransferHandler().exportAsDrag(
							GPLibraryPanel.this,
							e,
							TransferHandler.COPY);
					}
				}
				public void mouseReleased(MouseEvent e) {
					if (isPopupTrigger(e))
						showPopupMenu(panel, e.getPoint());
				}
			});
		}

                /** Returns the transferable.
                 * @return the transferable object
                 */                
		public GPTransferable getTransferable() {
			return trans;
		}

	}

}