/*
 * @(#)GPMarqueeHandler.java	1.2 05.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JPopupMenu;
import javax.swing.JToggleButton;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;


/**
 * MarqueeHandler that can insert cells.
 * 
 * @author sven.luzar
 * @version 2.0 TerpPresent
 *
 */
public class GPMarqueeHandler extends BasicMarqueeHandler {

	/** A reference to the graphpad object.
	 */
	GPGraphpad graphpad;

	/** The default color for borders.
	 */
	protected transient Color defaultBorderColor = Color.black;

        /** Button select.
         */        
	protected transient JToggleButton buttonSelect = new JToggleButton();
        /** Button rectangle.
         */        
	protected transient JToggleButton buttonRectangle = new JToggleButton();
        /** Button circle.
         */        
	protected transient JToggleButton buttonCircle = new JToggleButton();
        /** Button text.
         */        
	protected transient JToggleButton buttonText = new JToggleButton();
        /** Button image.
         */        
	protected transient JToggleButton buttonImage = new JToggleButton();
        /** Button line.
         */        
	protected transient JToggleButton buttonLine = new JToggleButton();
        /** Button edge.
         */        
	protected transient JToggleButton buttonEdge = new JToggleButton();
        /** Button zoom area.
         */        
	protected transient JToggleButton buttonZoomArea = new JToggleButton();

        /** Starting point.
         */
        protected Point start;
        
        /** Current point.
         */        
	protected Point  current;

        /** The rectangle bounds.
         */        
	protected Rectangle bounds;

        /** The port view.
         */
        protected PortView port;
        
        /** The first port view.
         */
        protected PortView firstPort;
        
        /** The second port view.
         */        
	protected PortView   lastPort;

	/** Constructor for GPMarqueeHandler.
         * @param graphpad the GPGraphpad
         */
	public GPMarqueeHandler(GPGraphpad graphpad) {
		super();
		this.graphpad = graphpad;
		ButtonGroup grp = new ButtonGroup();
		grp.add(buttonSelect);
		grp.add(buttonRectangle);
		grp.add(buttonCircle);
		grp.add(buttonText);
		grp.add(buttonImage);
		grp.add(buttonLine);
		grp.add(buttonEdge);
		grp.add(buttonZoomArea);
	}

	/* Return true if this handler should be preferred over other handlers. */
        /** Check if event is force marquee.
         * @param e a mouse event
         * @return boolean for force marquee
         */        
	public boolean isForceMarqueeEvent(MouseEvent e) {
		return !buttonSelect.isSelected()
			|| isPopupTrigger(e)
			|| super.isForceMarqueeEvent(e);
	}

        /** Check if is a popup trigger.
         * @param e a mouse event
         * @return boolean for popup trigger
         */        
	protected boolean isPopupTrigger(MouseEvent e) {
		return SwingUtilities.isRightMouseButton(e) && !e.isShiftDown();
	}

        /** Check if mouse botton is pressed.
         * @param event a mouse event
         */        
	public void mousePressed(MouseEvent event) {
		if (!isPopupTrigger(event)
			&& !event.isConsumed()
			&& !buttonSelect.isSelected()) {
			start = graphpad.getCurrentGraph().snap(event.getPoint());
			firstPort = port;
			if (buttonEdge.isSelected() && firstPort != null)
				start =
					graphpad.getCurrentGraph().toScreen(
						firstPort.getLocation(null));
			event.consume();
		}
		if (!isPopupTrigger(event))
			super.mousePressed(event);
		else {
			boolean selected = false;
			Object[] cells = graphpad.getCurrentGraph().getSelectionCells();
			for (int i = 0; i < cells.length && !selected; i++)
				selected =
					graphpad.getCurrentGraph().getCellBounds(
						cells[i]).contains(
						event.getPoint());
			if (!selected)
				graphpad.getCurrentGraph().setSelectionCell(
					graphpad.getCurrentGraph().getFirstCellForLocation(
						event.getX(),
						event.getY()));
			event.consume();
		}
	}

        /** Mouse drag event.
         * @param event a mouse event
         */        
	public void mouseDragged(MouseEvent event) {
		if (!event.isConsumed() && !buttonSelect.isSelected()) {
			Graphics g = graphpad.getCurrentGraph().getGraphics();
			Color bg = graphpad.getCurrentGraph().getBackground();
			Color fg = Color.black;
			g.setColor(fg);
			g.setXORMode(bg);
			overlay(g);
			current = graphpad.getCurrentGraph().snap(event.getPoint());
			if (buttonEdge.isSelected() || buttonLine.isSelected()) {
				port =
					getPortViewAt(
						event.getX(),
						event.getY(),
						!event.isShiftDown());
				if (port != null)
					current =
						graphpad.getCurrentGraph().toScreen(
							port.getLocation(null));
			}
			bounds = new Rectangle(start).union(new Rectangle(current));
			g.setColor(bg);
			g.setXORMode(fg);
			overlay(g);
			event.consume();
		}
		super.mouseDragged(event);
	}

	// Default Port is at index 0
        /** Get the port view at specified coordinates.
         * @param x x coor
         * @param y y coor
         * @param jump boolean for jump
         * @return the PortView at specified place
         */        
	public PortView getPortViewAt(int x, int y, boolean jump) {
		Point sp = graphpad.getCurrentGraph().fromScreen(new Point(x, y));
		PortView port = graphpad.getCurrentGraph().getPortViewAt(sp.x, sp.y);
		// Shift Jumps to "Default" Port (child index 0)
		if (port == null && jump) {
			Object cell =
				graphpad.getCurrentGraph().getFirstCellForLocation(x, y);
			if (graphpad.getCurrentGraph().isVertex(cell)) {
				Object firstChild =
					graphpad.getCurrentGraph().getModel().getChild(cell, 0);
				CellView firstChildView =
					graphpad
						.getCurrentGraph()
						.getGraphLayoutCache()
						.getMapping(
						firstChild,
						false);
				if (firstChildView instanceof PortView)
					port = (PortView) firstChildView;
			}
		}
		return port;
	}

        /** When mouse botton is released.
         * @param event a mouse event
         */        
	public void mouseReleased(MouseEvent event) {
		
		GraphModelProvider gmp = graphpad.getCurrentGraphModelProvider();
		GraphModel model = graphpad.getCurrentGraph().getModel() ;
		
		if (isPopupTrigger(event)) {
			Object cell =
				graphpad.getCurrentGraph().getFirstCellForLocation(
					event.getX(),
					event.getY());
			if (cell == null)
				graphpad.getCurrentGraph().clearSelection();
			Container parent = graphpad.getCurrentGraph();
			do {
				parent = parent.getParent();
			} while (parent != null && !(parent instanceof GPGraphpad));

			GPGraphpad pad = (GPGraphpad) parent;
			if (pad != null) {
				JPopupMenu pop = pad.getBarFactory().createGraphPopupMenu();
				pop.show(
					graphpad.getCurrentGraph(),
					event.getX(),
					event.getY());
			}
			event.consume();
		} else if (
			event != null
				&& !event.isConsumed()
				&& bounds != null
				&& !buttonSelect.isSelected()) {
			graphpad.getCurrentGraph().fromScreen(bounds);
			bounds.width++;
			bounds.height++;
			if (buttonZoomArea.isSelected()) {
				Rectangle view = graphpad.getCurrentGraph().getBounds();
				if (graphpad.getCurrentGraph().getParent()
					instanceof JViewport)
					view =
						((JViewport) graphpad.getCurrentGraph().getParent())
							.getViewRect();
				if (bounds.width != 0
					&& bounds.height != 0
					&& SwingUtilities.isLeftMouseButton(event)) {
					double scale =
						Math.min(
							(double) view.width / (double) bounds.width,
							(double) view.height / (double) bounds.height);
					if (scale > 0.1) {
						Rectangle unzoomed =
							graphpad.getCurrentGraph().fromScreen(bounds);
						graphpad.getCurrentGraph().setScale(scale);
						graphpad.getCurrentGraph().scrollRectToVisible(
							graphpad.getCurrentGraph().toScreen(unzoomed));
					}
				} else
					graphpad.getCurrentGraph().setScale(1);
				// FIX: Set ResizeAction to null!
			} else if (buttonRectangle.isSelected())
				addVertex(
					GraphModelProvider.CELL_VERTEX_DEFAULT,
					"",
					bounds,
					false,
					Color.black);
			else if (buttonCircle.isSelected())
				addEllipse("", bounds);
			else if (buttonText.isSelected()) {
				Color borderColor =
					(new Boolean(Translator.getString("TextBorder"))
						.booleanValue())
						? defaultBorderColor
						: null;
				Object cell =
					addVertex(
						GraphModelProvider.CELL_VERTEX_TEXT,
						"Text",
						bounds,
						false,
						Color.black);
				graphpad.getCurrentGraph().startEditingAtCell(cell);
			} else if (buttonImage.isSelected())
				addVertex(
					GraphModelProvider.CELL_VERTEX_IMAGE,
					GPGraphpad.applicationIcon,
					bounds,
					false,
					null);
			else if (buttonEdge.isSelected()) {
				Point p =
					graphpad.getCurrentGraph().fromScreen(new Point(start));
				Point p2 =
					graphpad.getCurrentGraph().fromScreen(new Point(current));
				ArrayList list = new ArrayList();
				list.add(p);
				list.add(p2);
				Map map = GraphConstants.createMap();
				GraphConstants.setPoints(map, list);
				GraphConstants.setRouting(map, GraphConstants.ROUTING_SIMPLE);
				Map viewMap = new Hashtable();
				
				
				//DefaultEdge cell = new DefaultEdge("");
				//Object cell = gmp.createCell(model, gmp.CELL_EDGE_DEFAULT , "", map);
				Object cell = gmp.createCell(model, GraphModelProvider.CELL_EDGE_DEFAULT , "", map);
				
				viewMap.put(cell, map);
				Object[] insert = new Object[] { cell };
				ConnectionSet cs = new ConnectionSet();
				if (firstPort != null)
					cs.connect(cell, firstPort.getCell(), true);
				if (port != null)
					cs.connect(cell, port.getCell(), false);
				graphpad.getCurrentGraph().getModel().insert(
					insert,
					viewMap,
					cs,
					null,
					null);
			} else if (buttonLine.isSelected()) {
				Point p =
					graphpad.getCurrentGraph().fromScreen(new Point(start));
				Point p2 =
					graphpad.getCurrentGraph().toScreen(new Point(current));
				ArrayList list = new ArrayList();
				list.add(p);
				list.add(p2);
				Map map = GraphConstants.createMap();
				GraphConstants.setPoints(map, list);
				GraphConstants.setLineEnd(map, GraphConstants.ARROW_CLASSIC);
				GraphConstants.setEndFill(map, true);
				//GraphConstants.setConnectable(map, false);
				Map viewMap = new Hashtable();
				//DefaultEdge cell = new DefaultEdge("");
				//Object cell = gmp.createCell(model, gmp.CELL_EDGE_DEFAULT, "", map);
				Object cell = gmp.createCell(model, GraphModelProvider.CELL_EDGE_DEFAULT, "", map);
				viewMap.put(cell, map);
				Object[] insert = new Object[] { cell };
				ConnectionSet cs = new ConnectionSet();
				if (firstPort != null)
					cs.connect(cell, firstPort.getCell(), true);
				if (port != null)
					cs.connect(cell, port.getCell(), false);
				graphpad.getCurrentGraph().getModel().insert(
					insert,
					viewMap,
					cs,
					null,
					null);
			}
			event.consume();
		}
		buttonSelect.doClick();
		firstPort = null;
		port = null;
		start = null;
		current = null;
		bounds = null;
		super.mouseReleased(event);
	}

        /** When mouse is being moved.
         * @param event a mouse event
         */        
	public void mouseMoved(MouseEvent event) {
		if (!buttonSelect.isSelected() && !event.isConsumed()) {
			graphpad.getCurrentGraph().setCursor(
				new Cursor(Cursor.CROSSHAIR_CURSOR));
			event.consume();
			if (buttonEdge.isSelected() || buttonLine.isSelected()) {
				PortView oldPort = port;
				PortView newPort =
					getPortViewAt(
						event.getX(),
						event.getY(),
						!event.isShiftDown());
				if (oldPort != newPort) {
					Graphics g = graphpad.getCurrentGraph().getGraphics();
					Color bg = graphpad.getCurrentGraph().getBackground();
					Color fg = graphpad.getCurrentGraph().getMarqueeColor();
					g.setColor(fg);
					g.setXORMode(bg);
					overlay(g);
					port = newPort;
					g.setColor(bg);
					g.setXORMode(fg);
					overlay(g);
				}
			}
		}
		super.mouseMoved(event);
	}

        /** Over lay the graphics.
         * @param g the graphics
         */        
	public void overlay(Graphics g) {
		super.overlay(g);
		paintPort(graphpad.getCurrentGraph().getGraphics());
		if (bounds != null && start != null) {
			if (buttonImage.isSelected() || buttonZoomArea.isSelected())
				 ((Graphics2D) g).setStroke(GraphConstants.SELECTION_STROKE);
			if (buttonCircle.isSelected())
				g.drawOval(bounds.x, bounds.y, bounds.width, bounds.height);
			else if (
				(buttonLine.isSelected() || buttonEdge.isSelected())
					&& current != null)
				g.drawLine(start.x, start.y, current.x, current.y);
			else if (!buttonSelect.isSelected())
				g.drawRect(bounds.x, bounds.y, bounds.width, bounds.height);
		}
	}

        /** Paint the port.
         * @param g the graphics
         */        
	protected void paintPort(Graphics g) {
		if (port != null) {
			boolean offset =
				(GraphConstants.getOffset(port.getAllAttributes()) != null);
			Rectangle r =
				(offset) ? port.getBounds() : port.getParentView().getBounds();
			r = graphpad.getCurrentGraph().toScreen(new Rectangle(r));
			int s = 3;
			r.translate(-s, -s);
			r.setSize(r.width + 2 * s, r.height + 2 * s);
			GPGraphUI ui = (GPGraphUI) graphpad.getCurrentGraph().getUI();
			ui.paintCell(g, port, r, true);
		}
	}

	//
	// Cell Creation
	//

  /** Adding a vertex.
   * @param type the specified type
   * @param userObject the user object
   * @param bounds the rectangle bounds
   * @param autosize boolean for autosize
   * @param border the border color
   * @return the vertex object
   */        
	public Object addVertex(int type,Object userObject,Rectangle bounds,boolean autosize,
			Color border) 
	{
		Map viewMap = new Hashtable();
		Map map;
		GraphModelProvider gmp = graphpad.getCurrentGraphModelProvider();
		GraphModel model = graphpad.getCurrentGraph().getModel() ;

		// Create Vertex
		Object obj = (userObject instanceof String) ? userObject : "";
		//DefaultGraphCell cell;
		/*
		if (userObject instanceof ImageIcon)
			cell = new ImageCell(obj);
		else if (userObject.toString().equals("Text"))
			cell = new TextCell("Type here");
		else
			cell = new DefaultGraphCell(obj);
		*/
		map = GraphConstants.createMap();
		GraphConstants.setBounds(map, bounds);
		GraphConstants.setOpaque(map, false);
		if (border != null)
			GraphConstants.setBorderColor(map, border);

		GraphConstants.setFontName(map, Translator.getString("FontName"));
		GraphConstants.setFontSize(
			map,
			new Integer(Translator.getString("FontSize")).intValue());
		GraphConstants.setFontStyle(
			map,
			new Integer(Translator.getString("FontStyle")).intValue());

		if (autosize)
			GraphConstants.setAutoSize(map, true);

		Object cell;
		switch (type) {
			case GraphModelProvider.CELL_VERTEX_IMAGE :
				//cell = gmp.createCell(model, gmp.CELL_VERTEX_IMAGE, obj, map);
				cell = gmp.createCell(model, GraphModelProvider.CELL_VERTEX_IMAGE, obj, map);
				break;
			case GraphModelProvider.CELL_VERTEX_TEXT :
				//cell = gmp.createCell(model, gmp.CELL_VERTEX_TEXT, "Type here", map);
				cell = gmp.createCell(model, GraphModelProvider.CELL_VERTEX_TEXT, "Type here", map);
				break;
			default :
				//cell = gmp.createCell(model, gmp.CELL_VERTEX_DEFAULT, userObject, map);
				cell = gmp.createCell(model, GraphModelProvider.CELL_VERTEX_DEFAULT, userObject, map);
				break;
		}

		viewMap.put(cell, map);

		// Create Ports
		int u = GraphConstants.PERCENT;
		//DefaultPort port;
		Object port;

		// Floating Center Port (Child 0 is Default)
		// port = new DefaultPort("Center");
		// cell.add(port);
		//port = gmp.createCell(model, gmp.CELL_PORT_DEFAULT, "Center", null);
		port = gmp.createCell(model, GraphModelProvider.CELL_PORT_DEFAULT, "Center", null);
		gmp.addPort(cell, port);

		if (userObject instanceof ImageIcon) {
			GraphConstants.setIcon(map, (ImageIcon) userObject);

			// Single non-floating central-port
			map = GraphConstants.createMap();
			GraphConstants.setOffset(
				map,
				new Point((int) (u / 2), (int) (u / 2)));
			viewMap.put(port, map);
		} else {
			// Top Left
			//port = new DefaultPort("Topleft");
			//cell.add(port);
			map = GraphConstants.createMap();
			GraphConstants.setOffset(map, new Point(0, 0));
			//port = gmp.createCell(model, gmp.CELL_PORT_DEFAULT, "Topleft", map);
			port = gmp.createCell(model, GraphModelProvider.CELL_PORT_DEFAULT, "Topleft", map);
			gmp.addPort(cell, port);
			viewMap.put(port, map);

			// Top Center
			//port = new DefaultPort("Topcenter");
			//cell.add(port);
			map = GraphConstants.createMap();
			GraphConstants.setOffset(map, new Point((int) (u / 2), 0));
			//port = gmp.createCell(model, gmp.CELL_PORT_DEFAULT, "Topcenter", map);
			port = gmp.createCell(model, GraphModelProvider.CELL_PORT_DEFAULT, "Topcenter", map);
			gmp.addPort(cell, port);
			viewMap.put(port, map);

			// Top Right
			//port = new DefaultPort("Topright");
			//cell.add(port);
			map = GraphConstants.createMap();
			GraphConstants.setOffset(map, new Point(u, 0));
			//port = gmp.createCell(model, gmp.CELL_PORT_DEFAULT, "Topright", map);
			port = gmp.createCell(model, GraphModelProvider.CELL_PORT_DEFAULT, "Topright", map);
			gmp.addPort(cell, port);
			viewMap.put(port, map);

			// Top Center
			//port = new DefaultPort("Middleleft");
			//cell.add(port);
			map = GraphConstants.createMap();
			GraphConstants.setOffset(map, new Point(0, (int) (u / 2)));
			//port = gmp.createCell(model, gmp.CELL_PORT_DEFAULT, "Middleleft", map);
			port = gmp.createCell(model, GraphModelProvider.CELL_PORT_DEFAULT, "Middleleft", map);
			gmp.addPort(cell, port);
			viewMap.put(port, map);

			// Middle Right
			//port = new DefaultPort("Middleright");
			//cell.add(port);
			map = GraphConstants.createMap();
			GraphConstants.setOffset(map, new Point(u, (int) (u / 2)));
			//port = gmp.createCell(model, gmp.CELL_PORT_DEFAULT, "Middleright", map);
			port = gmp.createCell(model, GraphModelProvider.CELL_PORT_DEFAULT, "Middleright", map);
			gmp.addPort(cell, port);
			viewMap.put(port, map);

			// Bottom Left
			//port = new DefaultPort("Bottomleft");
			//cell.add(port);
			map = GraphConstants.createMap();
			GraphConstants.setOffset(map, new Point(0, u));
			//port = gmp.createCell(model, gmp.CELL_PORT_DEFAULT, "Bottomleft", map);
			port = gmp.createCell(model, GraphModelProvider.CELL_PORT_DEFAULT, "Bottomleft", map);
			gmp.addPort(cell, port);
			viewMap.put(port, map);

			// Bottom Center
			//port = new DefaultPort("Bottomcenter");
			//cell.add(port);
			map = GraphConstants.createMap();
			GraphConstants.setOffset(map, new Point((int) (u / 2), u));
			//port = gmp.createCell(model, gmp.CELL_PORT_DEFAULT, "Bottomcenter", map);
			port = gmp.createCell(model, GraphModelProvider.CELL_PORT_DEFAULT, "Bottomcenter", map);
			gmp.addPort(cell, port);
			viewMap.put(port, map);

			// Bottom Right
			//port = new DefaultPort("Bottomright");
			//cell.add(port);
			map = GraphConstants.createMap();
			GraphConstants.setOffset(map, new Point(u, u));
			//port = gmp.createCell(model, gmp.CELL_PORT_DEFAULT, "Bottomright", map);
			port = gmp.createCell(model, GraphModelProvider.CELL_PORT_DEFAULT, "Bottomright", map);
			gmp.addPort(cell, port);
			viewMap.put(port, map);
		}

		Object[] insert = new Object[] { cell };
		graphpad.getCurrentGraph().getModel().insert(
			insert,
			viewMap,
			null,
			null,
			null);
		return cell;
	}

	/** Adding an ellipse.
	 * @param userObject the user object
	 * @param bounds the rectangle bounds
	 */        
	public void addEllipse(Object userObject, Rectangle bounds) {
		Map viewMap = new Hashtable();
		Map map;
		GraphModelProvider gmp = graphpad.getCurrentGraphModelProvider();
		GraphModel model = graphpad.getCurrentGraph().getModel() ;

		// Create Vertex
		// EllipseCell cell = new EllipseCell(userObject);
		map = GraphConstants.createMap();
		GraphConstants.setBounds(map, bounds);
		GraphConstants.setOpaque(map, false);
		GraphConstants.setBorderColor(map, Color.black);
		Object cell =
			gmp.createCell(model, 
				GraphModelProvider.CELL_VERTEX_ELLIPSE,
				userObject,
				map);

		viewMap.put(cell, map);

		// Create Ports
		int u = GraphConstants.PERCENT;
		Object port;

		// Floating Center Port
		port =
			gmp.createCell(model, 
				GraphModelProvider.CELL_PORT_DEFAULT,
				"Center",
				null);
		gmp.addPort(cell, port);
		//port = new DefaultPort("Center");
		//cell.add(port);

		// Top Left
		//port = new DefaultPort("Topleft");
		//cell.add(port);
		map = GraphConstants.createMap();
		GraphConstants.setOffset(
			map,
			new Point((int) (u * 0.15), (int) (u * 0.15)));
		port =
			gmp.createCell(model, 
				GraphModelProvider.CELL_PORT_DEFAULT,
				"Topleft",
				map);
		gmp.addPort(cell, port);
		viewMap.put(port, map);

		// Top Center
		// port = new DefaultPort("Topcenter");
		//cell.add(port);
		map = GraphConstants.createMap();
		GraphConstants.setOffset(map, new Point((int) (u / 2), 0));
		port =
			gmp.createCell(model, 
				GraphModelProvider.CELL_PORT_DEFAULT,
				"Topcenter",
				map);
		gmp.addPort(cell, port);
		viewMap.put(port, map);

		// Top Right
		// port = new DefaultPort("Topright");
		// cell.add(port);
		map = GraphConstants.createMap();
		GraphConstants.setOffset(
			map,
			new Point((int) (u * 0.85), (int) (u * 0.15)));
		port =
			gmp.createCell(model, 
				GraphModelProvider.CELL_PORT_DEFAULT,
				"Topright",
				map);
		gmp.addPort(cell, port);
		viewMap.put(port, map);

		// Top Center
		//port = new DefaultPort("Middleleft");
		//cell.add(port);
		map = GraphConstants.createMap();
		GraphConstants.setOffset(map, new Point(0, (int) (u / 2)));
		port =
			gmp.createCell(model, 
				GraphModelProvider.CELL_PORT_DEFAULT,
				"Middleleft",
				map);
		gmp.addPort(cell, port);
		viewMap.put(port, map);

		// Middle Right
		//port = new DefaultPort("Middleright");
		//cell.add(port);
		map = GraphConstants.createMap();
		GraphConstants.setOffset(map, new Point(u, (int) (u / 2)));
		port =
			gmp.createCell(model, 
				GraphModelProvider.CELL_PORT_DEFAULT,
				"Middleright",
				map);
		gmp.addPort(cell, port);
		viewMap.put(port, map);

		// Bottom Left
		//port = new DefaultPort("Bottomleft");
		//cell.add(port);
		map = GraphConstants.createMap();
		GraphConstants.setOffset(
			map,
			new Point((int) (u * 0.15), (int) (u * 0.85)));
		port =
			gmp.createCell(model, 
				GraphModelProvider.CELL_PORT_DEFAULT,
				"Bottomleft",
				map);
		gmp.addPort(cell, port);
		viewMap.put(port, map);

		// Bottom Center
		//port = new DefaultPort("Bottomcenter");
		//cell.add(port);
		map = GraphConstants.createMap();
		GraphConstants.setOffset(map, new Point((int) (u / 2), u));
		port =
			gmp.createCell(model, 
				GraphModelProvider.CELL_PORT_DEFAULT,
				"Bottomcenter",
				map);
		gmp.addPort(cell, port);
		viewMap.put(port, map);

		// Bottom Right
		//port = new DefaultPort("Bottomright");
		//cell.add(port);
		map = GraphConstants.createMap();
		GraphConstants.setOffset(
			map,
			new Point((int) (u * 0.85), (int) (u * 0.85)));
		port =
			gmp.createCell(model, 
				GraphModelProvider.CELL_PORT_DEFAULT,
				"Bottomright",
				map);
		gmp.addPort(cell, port);
		viewMap.put(port, map);

		Object[] insert = new Object[] { cell };
		graphpad.getCurrentGraph().getModel().insert(
			insert,
			viewMap,
			null,
			null,
			null);
	}

	/**
	 * Returns the buttonCircle.
	 * @return JToggleButton
	 */
	public JToggleButton getButtonCircle() {
		return buttonCircle;
	}

	/**
	 * Returns the buttonEdge.
	 * @return JToggleButton
	 */
	public JToggleButton getButtonEdge() {
		return buttonEdge;
	}

	/**
	 * Returns the buttonImage.
	 * @return JToggleButton
	 */
	public JToggleButton getButtonImage() {
		return buttonImage;
	}

	/**
	 * Returns the buttonLine.
	 * @return JToggleButton
	 */
	public JToggleButton getButtonLine() {
		return buttonLine;
	}

	/**
	 * Returns the buttonRectangle.
	 * @return JToggleButton
	 */
	public JToggleButton getButtonRectangle() {
		return buttonRectangle;
	}

	/**
	 * Returns the buttonSelect.
	 * @return JToggleButton
	 */
	public JToggleButton getButtonSelect() {
		return buttonSelect;
	}

	/**
	 * Returns the buttonText.
	 * @return JToggleButton
	 */
	public JToggleButton getButtonText() {
		return buttonText;
	}

	/**
	 * Returns the buttonZoomArea.
	 * @return JToggleButton
	 */
	public JToggleButton getButtonZoomArea() {
		return buttonZoomArea;
	}

}
