/*
 * @(#)GPOverview.java	1.2 11/11/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BorderFactory;
import javax.swing.JPanel;


/** This is the GPOverviewPanel class.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */
public class GPOverviewPanel
	extends JPanel
	implements ComponentListener, GraphModelListener, Observer {
            /** This is the GPGraph that is used.
             */            
	protected GPGraph graph;

        /** The GPOverviewPanel constructor.
         * @param g This is a JGraph used to create the overview panel
         */        
	public GPOverviewPanel(JGraph g) {
		GraphLayoutCache view = new ViewRedirector(g, g.getGraphLayoutCache());
		graph = new GPGraph(g.getModel(), view);
		graph.setAntiAliased(true);
		setBorder(BorderFactory.createEtchedBorder());
		graph.getModel().addGraphModelListener(this);
		graph.setEnabled(false);
		this.addComponentListener(this);
		g.getGraphLayoutCache().addObserver(this);
		setLayout(new BorderLayout());
		add(graph, BorderLayout.CENTER);
	}

	//
	// Observer
	//

        /** This updates the component.
         * @param o This is the component
         * @param arg This is the argument to the component
         */        
	public void update(Observable o, Object arg) {
		componentResized(null);
	}

	//
	// GraphModelListener
	//

        /** This changes the graph.
         * @param e This event is what causes the graph to change
         */        
	public void graphChanged(GraphModelEvent e) {
		componentResized(null);
	}

	//
	// Component Listener
	//

        /** This resizes the component.
         * @param e This event is what causes the component to be resized
         */        
	public void componentResized(ComponentEvent e) {
		Rectangle r = AbstractCellView.getBounds(graph.getGraphLayoutCache().getRoots());
		double scale = 0.5;
		if (r != null) {
			Dimension d = new Dimension(r.x + r.width, r.y + r.height);
			Dimension s = getSize();
			double sx = s.getWidth() * 0.95 / d.getWidth();
			double sy = s.getHeight() * 0.95 / d.getHeight();
			scale = Math.min(Math.max(Math.min(sx, sy), 0.05), 0.5);
		}
		graph.setScale(scale);
		repaint();
	}

        /** This causes the component to be shown.
         * @param e This component event is shown 
         */        
	public void componentShown(ComponentEvent e) {
		componentResized(e);
	}

        /** This causes the component to be hidden.
         * @param e This component event is hidden
         */        
	public void componentHidden(ComponentEvent e) {
	}

        /** This causes the component to be moved.
         * @param e This component event is moved
         */        
	public void componentMoved(ComponentEvent e) {
	}

	//
	// View Redirector
	//

        /** This class changes the view.
         */        
	public class ViewRedirector extends GraphLayoutCache {

            /** This is the cache of the old layout parameters.
             */            
		protected GraphLayoutCache realView;

                /** This is the ViewRedirector constructor.
                 * @param graph This is the JGraph
                 * @param realView This is the layout cache */                
		public ViewRedirector(JGraph graph, GraphLayoutCache realView) {
			super(null, graph);
			this.realView = realView;
			setModel(graph.getModel());
		}

                /** This gets the roots of the GraphLayoutCache.
                 * @return This returns the roots
                 */                
		public CellView[] getRoots() {
			return realView.getRoots();
		}

                /** This gets the map of the cell.
                 * @param cell This is the cell that we are getting the mapping of
                 * @param create This is passed to GraphLayoutCache.getMapping(Object, boolean)
                 * @return This returns the mapping of the cell
                 */                
		public CellView getMapping(Object cell, boolean create) {
			if (realView != null)
				return realView.getMapping(cell, create);
			return null;
		}

                /** This creates a new mapping for the cell.
                 * @param cell This is the cell that we are recording the mapping of
                 * @param view This is the view that corresponds to the cell
                 */                
		public void putMapping(Object cell, CellView view) {
			if (realView != null)
				realView.putMapping(cell, view);
		}

	}

}