/*
 * @(#)GraphModelFileFormat.java	1.0 17.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//import java.io.IOException;
import java.util.Hashtable;

import javax.swing.JComponent;
import javax.swing.filechooser.FileFilter;


/**This is the interface to implement a file format
 * for a specific graph model provider.<br>
 * <br>
 * You must implement all methods and the corresponding 
 * graph model provider must return this file format.
 * 
 * @see GraphModelProvider#getGraphModelFileFormats()
 * @author luzar
 * @version 2.0 TerpPresent
 */
public interface GraphModelFileFormat {

	/** Returns the JFileChooser file filter
         *  for this file format.
         * @return Returns the file filter
         */
	public FileFilter getFileFilter();

	/** Returns the File Extension for this
         *  file format (without a point).
         * @return Returns the file extension
         */
	public String getFileExtension();

	/** Writes the graph model with this file format.
         *
         * @param filename The filename for the file (Absolut Path and with extension)
         * @param properties The write properties for this file format.
         * @param gpGraph The current graph.
         * @param graphModel The current graph model
         * @throws Exception This exception is thrown if needed
         */
	public void write(
		String filename,
		Hashtable properties,
		GPGraph gpGraph,
		GraphModel graphModel)
		throws Exception;

	/** Gets a accessory component for the
         *  JFileChooser. If the return value is
         *  <tt>null</tt> no accessory will show.
         *  The component was used while the write operation.
         * @return Returns the write accessory
         */
	public JComponent getWriteAccessory();

	/** Returns a Hashtable with write properties.
         *  The source for this properties
         *  can be the accessory or other
         *  Frames which were shown while calling this method.
         *  All Properties in this Hashtable should be
         *  serializable.
         * @param accessory This is required
         * @return Returns a hashtable with write properties
         */
	public Hashtable getWriteProperties(JComponent accessory);

	/** Reads the graph from the filename and returns the
         *  new model for the graph.
         *
         * @return The new Graph model.
         * @param filename The filename for the file
         * @param properties The properties for the read process
         * @param gpGraph The current Graph
         * @throws Exception An exception is thrown if needed
         */
	public GraphModel read(
		String filename,
		Hashtable properties,
		GPGraph gpGraph)
		throws Exception;

	/** Gets a accessory component for the
         *  JFileChooser. If the return value is
         *  null no accessory will show.
         *  The component was used while the read operation.
         * @return Returns an accessory component
         */
	public JComponent getReadAccessory();

	/** Returns a Hashtable with read properties.
         *  The source for this properties
         *  can be the accessory or other
         *  Frames which were shown while calling this method.
         * @param accessory This is what we are reading the properties of
         * @return Returns a hashtable with read properties
         */
	public Hashtable getReadProperties(JComponent accessory);

}
