/*
 * @(#)GraphUI.java	1.0 1/1/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.*;
import java.awt.event.MouseEvent;
import javax.swing.plaf.ComponentUI;

/**
 * Pluggable look and feel interface for JGraph.
 *
 * @version 2.0 TerpPresent.
 * @author Gaudenz Alder
 */
public abstract class GraphUI extends ComponentUI
{

  /**
   * Paints the renderer of <code>view</code> to <code>g</code>
   * at <code>bounds</code>.
   */
    public GraphUI()
    {
    }

    /** The abstract method paintCell. This paints (draws to screen) a cell.
     *
     * @param g This is the Graphics object for the cell(s) you want to paint.
     * @param cellview This is the object holding the selected cell(s).
     * @param rectangle This is the Rectangle object holding the information such as x, y, width, and height.
     * @param flag This will be passed as the fourth parameter to Cellview.getRendererComponent*/
    public abstract void paintCell(Graphics g, CellView cellview, Rectangle rectangle, boolean flag);

    /** The abstract method paintPorts. This paints the specified ports.
     *
     * @param g This is the Graphics object for the port(s) you want to paint.
     * @param aportview This is the object holding the selected port(s).  */
    public abstract void paintPorts(Graphics g, PortView aportview[]);

    /** The abstract method selectCellsForEvent. Selects which cells should be associated with the particular event.
     *
     * @param jgraph This is the JGraph object with which you are working.
     * @param aobj This is an array of Objects that will be affected by the MouseEvent.
     * @param mouseevent either AddToSelectionEvent or ToggleSelectionEvent */
    public abstract void selectCellsForEvent(JGraph jgraph, Object aobj[], MouseEvent mouseevent);

    /** The abstract method getPreferredSize
     * @param jgraph This is the JGraph object with which you are working.
     * @param cellview This is the object that you wish to get the size of.
     * @return Returns the size of the CellView object */
    public abstract Dimension getPreferredSize(JGraph jgraph, CellView cellview);

    /** The abstract method getHandle
     * @param jgraph This is the JGraph object with which you are working.
     * @return Returns the handle of the JGraph object. */
    public abstract CellHandle getHandle(JGraph jgraph);

    /** The abstract method isEditing
     * @param jgraph This is the JGraph object with which you are working.
     * @return  Returns true if editingComponent != null */
    public abstract boolean isEditing(JGraph jgraph);

    /** Returns true or false depending on whether cell editing should end.
     *
     * @param jgraph This is the JGraph object with which you are working.
     * @return Returns true if it is time to stop the cell editing.  */
    public abstract boolean stopEditing(JGraph jgraph);

    /**
     * This method cancels all the changes that have been made to a cell.
     *
     * @param jgraph This is the JGraph object with which you are working.  */
    public abstract void cancelEditing(JGraph jgraph);

    /**
     * This method start when a cell starts to get edited.
     *
     * @param jgraph This is the JGraph object with which you are working.
     * @param obj This is the cell at which to start editing.  */
    public abstract void startEditingAtCell(JGraph jgraph, Object obj);

    /**
     * Returns the current cell being edited.
     *
     * @param jgraph This is the JGraph object with which you are working.
     * @return Returns the Object associated with the cell that is being edited. */
    public abstract Object getEditingCell(JGraph jgraph);
}
