/*
 * @(#)HelpAbout.java 1.2 01.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.event.ActionEvent;
import java.util.MissingResourceException;

import javax.swing.ImageIcon;
import javax.swing.JDialog;


/** Opens an About dialogBox that tells more about
 * TerpPresent version 2.0.
 * @author sven.luzar
 * @version 2.0 TerpPresent
 */
public class HelpAbout extends AbstractActionDefault {

 /** The about dialogBox for jGraphPad and terpPresent.
         */
 protected JDialog aboutDlg;

 /** Constructor for HelpAbout.
         * This constructor is called from GPGraphPad.java
         * @param graphpad Passes the current state of jGraphPad.
         */
 public HelpAbout(GPGraphpad graphpad) {
  super(graphpad);
 }

 /**    Oopens an About dialogBox that tells more about
         * TerpPresent version 2.0, which includes credits to the developers, when an
         * action event is initiated.
         * @param e Called when a menu-item is clicked or an Action is performed.
         * @return void
         * @throws ex Throws an exception when the HelpAbout resource is missing.
         */
 public void actionPerformed(ActionEvent e) {
  if (aboutDlg == null) {
   // Create a frame containing an instance of
   // LibraryPanel.
   
   String iconName = Translator.getString("Logo");
   ImageIcon logoIcon = ImageLoader.getImageIcon(iconName);
   
   try {
    String title = Translator.getString("AboutFrameTitle");
    
    aboutDlg = new GPAboutDialog(graphpad.getFrame(), title, logoIcon);
   } catch (MissingResourceException mre) {
    aboutDlg =
     new GPAboutDialog(
      graphpad.getFrame(),
      "About TerpPresent",
      logoIcon);
   }
  }
  aboutDlg.show();
 }
 /** Empty implementation. 
  *  This Action should be available
  *  each time.
  */
 public void update(){};

}