/**
 *  Building Composer, http://bc.cecer.army.mil
 *
 *  ERDC - Engineering Research and Development Center
 *  USACERL - US Army Construction Engineering Research Laboratory
 *
 *  All rights reserved, 2002.  Please see included license.
 */

import java.awt.*;
//import java.io.*;
//import java.util.*;
import javax.swing.*;

/**
 * JDialogP allows for the extension of JDialog
 *
 * @author US Army Construction Engineering Research Laboratory
 * @created February 23, 2002
 * @version 2.0 TerpPresent
 */
public abstract class JDialogP extends JDialog {
	// set default property value
	// all properties should be prefixed with instanceID in order to
	// ensure uniqueness across properties in the shared pool of user properties
        /** Default property bounds.
         */
	String property_bounds = getInstanceID() + ".bounds";
        /** Default name space.
         */
	String nameSpace = "." + getNameSpace();
        /** Default Gui properties.
         */
        UserProperties guiProperties = UserProperties.getInstance(nameSpace);
        /** Boolean if object is visible at least once.
         */
	boolean hasBeenVisibleAtLeastOnce = false;


	/*
	 * @todo persist iconify/maximized attribute
	 */

	/**
	 * Constructor for JDialogP.
	 *
	 */
	public JDialogP() {
		super();
	}


	/**
	 * Constructor for JDialogP.
	 *
	 * @param owner Description of Parameter
	 */
	public JDialogP(Dialog owner) {
		super(owner);
	}


	/**
	 * Constructor for JDialogP.
	 *
	 * @param owner Description of Parameter
	 * @param modal Description of Parameter
	 */
	public JDialogP(Dialog owner, boolean modal) {
		super(owner, modal);
	}


	/**
	 * Constructor for JDialogP.
	 *
	 * @param owner Description of Parameter
	 * @param title Description of Parameter
	 */
	public JDialogP(Dialog owner, String title) {
		super(owner, title);
	}


	/**
	 * Constructor for JDialogP.
	 *
	 * @param owner Description of Parameter
	 * @param title Description of Parameter
	 * @param modal Description of Parameter
	 */
	public JDialogP(Dialog owner, String title, boolean modal) {
		super(owner, title, modal);
	}


	/**
	 * Constructor for JDialogP.
	 *
	 * @param owner Description of Parameter
	 */
	public JDialogP(Frame owner) {
		super(owner);
	}


	/**
	 * Constructor for JDialogP.
	 *
	 * @param owner Description of Parameter
	 * @param modal Description of Parameter
	 */
	public JDialogP(Frame owner, boolean modal) {
		super(owner, modal);
	}


	/**
	 * Constructor for JDialogP.
	 *
	 * @param owner Description of Parameter
	 * @param title Description of Parameter
	 */
	public JDialogP(Frame owner, String title) {
		super(owner, title);
	}


	/**
	 * Constructor for JDialogP.
	 *
	 * @param owner Description of Parameter
	 * @param title Description of Parameter
	 * @param modal Description of Parameter
	 */
	public JDialogP(Frame owner, String title, boolean modal) {
		super(owner, title, modal);
	}


	/**
	 * Sets the bounds attribute of the JDialogP object.
	 *
	 * @param x The new x value
	 * @param y The new y value
	 * @param width The new width value
	 * @param height The new height value
	 */
	public void setBounds(int x, int y, int width, int height) {
		// if persisted, and already displayed at least once, then allow programmatic change
		if (hasBeenVisibleAtLeastOnce) {
			super.setBounds(x, y, width, height);
			guiProperties.setRect(property_bounds, this.getBounds());
		}
		// else, if not been visible at least once, and not yet ever persisted then allow programmatic change to set initial location
		else if (guiProperties == null || guiProperties.getRect(property_bounds) == null) {
			super.setBounds(x, y, width, height);
		}
		// else, ignore request and set to the previously persisted value
		else {
			Rectangle r = guiProperties.getRect(property_bounds);
			super.setBounds(r.x, r.y, r.width, r.height);
		}
	}


	/** Sets the bounds attribute of the JDialogP object.
         *
         * @param r This rectangle contains the bounds that need to be set */
	public void setBounds(Rectangle r) {
		this.setBounds(r.x, r.y, r.width, r.height);
	}


	/**
	 * Sets the location attribute of the JDialogP object.
	 *
	 * @param x The new x value
	 * @param y The new y value
	 */
	public void setLocation(int x, int y) {
		// if persisted, and already displayed at least once, then allow programmatic move
		if (hasBeenVisibleAtLeastOnce) {
			super.setLocation(x, y);
			guiProperties.setRect(property_bounds, this.getBounds());
		}
		// else, if not been visible at least once, and not yet ever persisted then allow programmatic move to set initial location
		else if (guiProperties == null || guiProperties.getRect(property_bounds) == null) {
			super.setLocation(x, y);
		}
		// else, ignore request and set to the previously persisted value
		else {
			Rectangle r = guiProperties.getRect(property_bounds);
			super.setLocation(r.x, r.y);
		}
	}


	/**
	 * Sets the location attribute of the JDialogP object.
	 *
	 * @param p The new point value
	 */
	public void setLocation(Point p) {
		this.setLocation(p.x, p.y);
	}


	/**
	 * Sets the size attribute of the JDialogP object.
	 *
	 * @param d The new Dimension value
	 */
	public void setSize(Dimension d) {
		this.setSize(d.width, d.height);
	}


	/**
	 * Sets the size attribute of the JDialogP object.
	 *
	 * @param width The new height value
	 * @param height The new width value
	 */
	public void setSize(int width, int height) {
		// if persisted, and already displayed at least once, then allow programmatic resize
		if (hasBeenVisibleAtLeastOnce) {
			super.setSize(width, height);
			guiProperties.setRect(property_bounds, this.getBounds());
		}
		// else, if not been visible at least once, and not yet ever persisted then allow programmatic resize to set initial location
		else if (guiProperties == null || guiProperties.getRect(property_bounds) == null) {
			super.setSize(width, height);
		}
		// else, ignore request and set to the previously persisted value
		else {
			Rectangle r = guiProperties.getRect(property_bounds);
			super.setSize(r.width, r.height);
		}
	}


	// changing forces a recreation with new directory
	// prepends '.'
	/**
	 * Gets the name attribute of the JDialogP object.
	 *
	 * @return The name value
	 */
	public abstract String getNameSpace();


	// prefix to key, doesn'buttonText create a new directory
	// automatically prepends full class name to ensure uniqueness
	// this.getClass().getName()
	// recommend implementors implement a setInstanceID() method if unique instance location is desired.  as it currently stands every instance of the implemented class will share the persisted bounds.
	/**
	 * Gets the instance ID attribute of the JDialogP object.
	 *
	 * @return The id value
	 */
	public abstract String getInstanceID();


	/**
	 * Show the JDialogP object.
	 */
	public void show() {
		init();

		if (hasBeenVisibleAtLeastOnce == false) {
			// set default location to be centered
			// this'buttonSelect implementation of setBounds will deal with checking for persisted value
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			Dimension componentSize = this.getSize();
			int x = (screenSize.width - this.getWidth()) / 2;
			int y = (screenSize.height - this.getHeight()) / 2;
			this.setBounds(x, y, this.getWidth(), this.getHeight());
		}

		hasBeenVisibleAtLeastOnce = true;
		super.show();
	}


	/*
	 * Set the bounds of the JDialogP object.
	 *
	 * @param x x value of the bounds
	 * @param y y value of the bounds
	 * @param width width of the bounds
	 * @param height height of the bounds
	 */
	// not needed internal setBounds(x, y, width, height)
	// was called
	/*
	public void reshape(int x, int y, int width, int height) {
		// if persisted, and already displayed at least once, then allow programmatic change
		if (hasBeenVisibleAtLeastOnce) {
			super.reshape(x, y, width, height);
			guiProperties.setRect(property_bounds, this.getBounds());
		}
		// else, if not been visible at least once, and not yet ever persisted then allow programmatic change to set initial location
		else if (guiProperties == null || guiProperties.getRect(property_bounds) == null) {
			super.reshape(x, y, width, height);
		}
		// else, ignore request and set to the previously persisted value
		else {
			Rectangle buttonRectangle = guiProperties.getRect(property_bounds);
			super.reshape(buttonRectangle.x, buttonRectangle.y, buttonRectangle.width, buttonRectangle.height);
		}
	}*/


	/**
	 * Move the JDialogP object
	 *
	 * @param x x movement distance
	 * @param y y movement distance
	 */
	// not needed internal setBounds(x, y, width, height)
	// was called
	/*	
	public void move(int x, int y) {
//		this.setLocation(x, y);
		// if persisted, and already displayed at least once, then allow programmatic move
		if (hasBeenVisibleAtLeastOnce) {
			super.move(x, y);
			guiProperties.setRect(property_bounds, this.getBounds());
		}
		// else, if not been visible at least once, and not yet ever persisted then allow programmatic move to set initial location
		else if (guiProperties == null || guiProperties.getRect(property_bounds) == null) {
			super.move(x, y);
		}
		// else, ignore request and set to the previously persisted value
		else {
			Rectangle buttonRectangle = guiProperties.getRect(property_bounds);
			super.move(buttonRectangle.x, buttonRectangle.y);
		}
	}*/


	/**
	 * Resize the JDialogP object
	 *
	 * @param d the dimensions of the object
	 */
	// not needed internal setBounds(x, y, width, height)
	// was called
	/*	
	public void resize(Dimension d) {
		this.resize(d.width, d.height);
	}*/


	/**
	 * Resize the JDialogP object
	 *
	 * @param width the width of the object
	 * @param height the height of the object
	 */
	//not needed internal the method setBounds was called
	/*
	public void resize(int width, int height) {
//		this.setSize(width, height);
		// if persisted, and already displayed at least once, then allow programmatic resize
		if (hasBeenVisibleAtLeastOnce) {
			super.setBounds(super.getX(), super.getY(), width, height);
			guiProperties.setRect(property_bounds, this.getBounds());
		}
		// else, if not been visible at least once, and not yet ever persisted then allow programmatic resize to set initial location
		else if (guiProperties == null || guiProperties.getRect(property_bounds) == null) {
			super.setBounds(super.getX(), super.getY(), width, height);
		}
		// else, ignore request and set to the previously persisted value
		else {
			Rectangle buttonRectangle = guiProperties.getRect(property_bounds);
			super.setBounds(super.getX(), super.getY(), width, height);
		}
	}
	*/


	/**
	 * Hide the JDialogP object.
	 */
	public void hide() {
		init();
		guiProperties.setRect(property_bounds, this.getBounds());
		guiProperties.save();
		super.hide();
	}


	/**
	 * Initialize the JDialogP object.
	 */
	protected void init() {
		// check if not already init'd
		if (guiProperties == null) {
			guiProperties = UserProperties.getInstance(nameSpace);
		}
	}

// NOT guaranteed to be called prior to exit!!!! need to find javaworld article on how to guarantee finalize...and if it doesn'buttonText require implementor intervention then use it rather than requiring explicit call to save().
//	/**
//	 * Description of the Method
//	 *
//	 * @exception Throwable Description of Exception
//	 */
//	protected void finalize()
//		throws Throwable {
//		init();
//		System.out.println("finalize");
//		guiProperties.save();
//		super.finalize();
//	}
}
