import java.util.Iterator;

/**
 * Defines the requirements for an object that
 * represens a port in a graph model.
 *
 * Referenced class of package com.jgraph.graph is
 * GraphCell.
 *
 * @author Gaudenz Alder
 * @version 1.0 1/1/02
 *
 * @see GraphCell
 */

public interface Port
  extends GraphCell
{
  
  /** 
   * Returns an iterator of the edges connected
   * to the port.
   * @return An Iterator data type.
   * @see Iterator
   */
  public abstract Iterator edges();
  
  /** 
   * Adds an edge to the list of ports. Returns an boolean saying 
   * whether the edge has been added or not
   * @param obj A variable of Object data type
   * @return A boolean data type.
   * @see Object
   */
  public abstract boolean addEdge(Object obj);
  
  /** 
   * Removes edge from the list of ports. Returns an boolean saying
   * whether the edge has been removed or not
   * @param obj A variable of Object data type
   * @return A boolean data type
   * see Object
   */
  public abstract boolean removeEdge(Object obj);
  
  
  /**
   * Retrieves the anchor of the port.
   * @return A Port data type.
   * @see Port
   */
  public abstract Port getAnchor();
  
  
  /** 
   * Sets the anchor of the port.
   * @param port A Port data type
   * @see Port
   */
  public abstract void setAnchor(Port port);
}
