/*
 * @(#)RealGraphCellRenderer.java	1.2 11/11/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;

import javax.swing.CellRendererPane;
import javax.swing.JComponent;


/** Allows for RealGraphCellRenderer to extend JComponent.
 *
 * @author TerpPresent
 * @version 2.0 TerpPresent
 */
public class RealGraphCellRenderer extends JComponent {

    /** This is a rendererPane.
     */    
	protected CellRendererPane rendererPane;
        /** This holds the different views for rendering.
         */        
	protected CellView[] views;
        /** This is the graph in which all work is done.
         */        
	protected GPGraph graph;
        /** This is the scale for rendering.
         */        
	protected double scale = 1.0;

        /** Constructor.
         * @param graph the graph to do the rendering in
         * @param views the list of views for rendering 
         */        
	public RealGraphCellRenderer(GPGraph graph, CellView[] views) {
		add(rendererPane = new CellRendererPane());
		this.views = views;
		this.graph = graph;
	}

        /** This sets the scale of rendering.
         * @param scale this is the scale 
         */        
	public void setScale(double scale) {
		this.scale = scale;
	}

        /** This gets the size to fit everything on screen.
         * @return This returns the dimensions of the size
         */        
	public Dimension getPreferredSize() {
		if (views != null) {
			Rectangle r = AbstractCellView.getBounds(views);
			r.width *= scale;
			r.height *= scale;
			return new Dimension(r.width + 2, r.height + 2);
		}
		return new Dimension(10, 10);
	}

        /** This paints the graphics to the rendering.
         * @param g these are the graphics to be painted
         */        
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		AffineTransform at = g2.getTransform();
		g2.scale(scale, scale);
		if (views != null) {
			Rectangle r = AbstractCellView.getBounds(views);
			g.translate(-r.x, -r.y);
			for (int i = 0; i < views.length; i++) {
				Rectangle b = views[i].getBounds();
				Component c;
				if (graph.isGroup(views[i].getCell()))
					c =
						new RealGraphCellRenderer(
							graph,
							((CellView) views[i]).getChildViews());
				else
					c =
						views[i].getRendererComponent(
							graph,
							false,
							false,
							false);
				rendererPane.paintComponent(
					g,
					c,
					this,
					b.x,
					b.y,
					b.width,
					b.height);
			}
		}
		g2.setTransform(at);
	}

}