/*
 * @(#)ShapeUngroup.java 1.2 01.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.event.ActionEvent;
import java.util.ArrayList;


/** Action that ungroups all grouped items in the current selection.
 * 
 * @author sven.luzar
 * @version 2.0 TerpPresent
 * @see com.jgraph.pad.actions.AbstractActionDefault
 *
 */
public class ShapeUngroup extends AbstractActionDefault {

 /** Constructor for ShapeUngroup.
         * @param graphpad The parameter refers to the current graph.
         * @see com.jgraph.GPGraphpad
         */
 public ShapeUngroup(GPGraphpad graphpad) {
  super(graphpad);
 }

 /** This function will ungroup the selected object, if previously grouped.
         * @param e - This parameter is called from selecting the corresponding menu item.
         * @see java.awt.event.ActionEvent
         */
 public void actionPerformed(ActionEvent e) {
  Object[] cells = getCurrentGraph().getSelectionCells();
  if (cells != null) {
   ArrayList groups = new ArrayList();
   ArrayList children = new ArrayList();
   for (int i = 0; i < cells.length; i++) {
    if (getCurrentGraph().isGroup(cells[i])) {
     groups.add(cells[i]);
     for (int j = 0;
      j < getCurrentGraph().getModel().getChildCount(cells[i]);
      j++) {
      Object child = getCurrentGraph().getModel().getChild(cells[i], j);
      if (!(child instanceof Port))
       children.add(child);
     }
    }
   }
   getCurrentGraph().getModel().remove(groups.toArray());
   getCurrentGraph().setSelectionCells(children.toArray());
  }
 }

}
