/*
 * TPDocument.java
 *
 * Created on March 19, 2004, 10:03 PM
 */

import java.awt.*;
import java.util.*;
import javax.swing.*;

/** A Document that contains multiple slides.  Each slides is
 * is represented with a GPDocument.
 *
 * @author TerpPresent
 * @author vxn
 * @version 3.0 TerpPresent
 */
public class TPDocument extends GPDocument {
    
    //private JPanel tpDocPanel;
    //private JSplitPane tpDocPanel;
    //private JScrollPane tpScrollPane;


    /** A map that stores the GPDocument(s) making up the slides.
     */
    //private SortedMap gpDocMap;
    //private java.util.List gpDocList;
    
    /** A list that stores the string title of the slides.
     */
    private DefaultListModel listOrder;
    
    /** A list that allows the selection of the slides.
     */
    private JList list;
    
    /** A panel that holds the list of the slides.
     */
    private JPanel listPanel;
    
    /** A status bar for the document.
     */
    private GPStatusBar tpStatusBar;
    
    //private Map graphMap;
    //public ActionListener listener;


    /** 
     * Creates a new instance of TPDocument.
     * @param graphpad A GPGraphpad object.
     * @param filename A string with path to a file.
     * @param graphModelProvider An GraphModelProvider object.
     * @param gpGraph An empty graph object.
     * @param model A graph model.
     * @param undo The undo manager for the graph.
     * @param gpGraphList A List of GPDocument.
     */
    public TPDocument( GPGraphpad graphpad,
                       String filename,
                       GraphModelProvider graphModelProvider,
                       GPGraph gpGraph,
                       GraphModel model,
                       GraphUndoManager undo,
                       java.util.List gpGraphList ) {
        super( graphpad, filename, graphModelProvider, gpGraph, model, undo );
        
        //gpDocMap = new TreeMap();
        tpStatusBar = new GPStatusBar();
        listOrder = new DefaultListModel();
        list = new JList();
        listPanel = new JPanel();
//        tpScrollPane = new JScrollPane();

        addSlidesToDocPanel( gpGraphList );
    }
    
    /*protected Component createCenterComponent() {
        //tpDocPanel = new JPanel( true );
        tpDocPanel = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT );
        return tpDocPanel;
    }*/


    /**
     * Initalizes the slides and add them to the list panel.
     * @param gpGraphList A list of GPDocument to initialize the slides.
     */
    private void addSlidesToDocPanel( java.util.List gpGraphList ) {
        this.add(BorderLayout.SOUTH, tpStatusBar);

        int i = 1;
        for( Iterator iter = gpGraphList.iterator() ; iter.hasNext() ; ) {
            GraphModel model = graphModelProvider.createCleanGraphModel();
            GPGraph gp = (GPGraph)iter.next();
            gp.setSelectNewCells( true );
            gp.setInvokesStopCellEditing( true );
            gp.setMarqueeHandler( graphpad.getMarqueeHandler() );

//            String str = new String( "Slide " + i );
//            gpDocMap.put( str, new GPDocument( graphpad, null, graphModelProvider, gp, model, null ) );
//            listOrder.addElement( str );
            listOrder.addElement( new GPDocument( graphpad, null, graphModelProvider, gp, model, null ) );
            i++;
        }

        list = new JList( listOrder );
        list.setCellRenderer( new TPListCellRenderer( this ) );
        list.setDragEnabled( true );
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        //if (this.getTransferHandler() != null)
        //	System.out.println (this.getTransferHandler()) ;
        //list.setTransferHandler( new TPStringListTransferHandler( this ) );
        list.setTransferHandler( graphpad.getTPGPDocTransferHandler() );
        listPanel.add( list );

        splitPane.setLeftComponent( listPanel );
        splitPane.setRightComponent( scrollPane );

        if( list.getModel().getSize() > 0 ) {
            list.setSelectedIndex( 0 );
        }   
    }
     
    /**
     * Change and show the slide selected from the slide list.
     * @param key The string of the slide selected.
     */
    /*public void changeCurrentSlide( String key ) {
        GPDocument doc = (GPDocument)gpDocMap.get( key );
        graph = doc.getGraph();
        graphUndoManager = doc.getGraphUndoManager();
        scrollPane.setViewportView( doc.getGraph() );
        tpStatusBar.setMessage( "Slide " + ( listOrder.indexOf( key ) + 1 ) +
                                " of " + listOrder.size() );
    }*/
    public void changeCurrentSlide( int index ) {
        //GPDocument doc = (GPDocument)gpDocMap.get( key );
        GPDocument doc = (GPDocument)listOrder.get( index );
        graph = doc.getGraph();
        graphUndoManager = doc.getGraphUndoManager();
        scrollPane.setViewportView( doc.getGraph() );
        tpStatusBar.setMessage( "Slide " + ( index + 1 ) +
                                " of " + listOrder.size() );
    }
    
    //to be optimized
    /**
     * Reorder the string of the slides to correspond to its position in
     * the list.
     */
    /*public void reorderSlides() {
        SortedMap newDocMap = new TreeMap();
        DefaultListModel newLM = new DefaultListModel();
        
        for( int i = 0 ; i < listOrder.size() ; i++ ) {
            String newStr = new String( "Slide " + ( i + 1 ) );
            String oldStr = (String)listOrder.get( i );
            newDocMap.put( newStr, (GPDocument)gpDocMap.get( oldStr ) );
            newLM.addElement( newStr );
        }
        gpDocMap = newDocMap;
        listOrder = newLM;
        list.setModel( listOrder );
    }*/
    
    /**
     * Returns the map of the GPDocument(s) in the document.
     * The GPDocument represents the slides in the document.
     * @return A map of the GPDocument(s) that represents the slides.
     */
    /*public SortedMap getGPDocumentMap() {
        return gpDocMap;
    }*/
    
    public DefaultListModel getListModel() {
        return listOrder;
    }
    
    /**
     * Add a new empty slide to the document.  The slide will
     * be append to the end of the list.
     */
    public void addNewSlide() {
        int slideNum = list.getModel().getSize() + 1;
        
        GraphModel model = graphModelProvider.createCleanGraphModel();
        GPGraph gp = graphModelProvider.createCleanGraph( model );

        gp.setSelectNewCells( true );
        gp.setInvokesStopCellEditing( true );
        gp.setMarqueeHandler( graphpad.getMarqueeHandler() );

        String str = new String( "Slide " + slideNum );
        //gpDocMap.put( str, new GPDocument( graphpad, null, graphModelProvider, gp, model, null ) );
        //listOrder.addElement( str );
        listOrder.addElement( new GPDocument( graphpad, null, graphModelProvider, gp, model, null ) );
        list.setSelectedIndex( slideNum - 1 );
    }
    
    /**
     * Remove the current selected slide from the document.
     * The remain slides will be rename and reorder to relect
     * the change.
     */
    public void removeCurrentSlide() {
        int index = list.getSelectedIndex();
        //String str = (String)listOrder.remove( index );
        //gpDocMap.remove( str );
        listOrder.remove( index );
        
        //reorderSlides();
    
        if( listOrder.size() > index ) {
            list.setSelectedIndex( index );
        } else if( listOrder.size() > 0 ){
            list.setSelectedIndex( index - 1 );
        } else {
            scrollPane.setViewportView( null );
        }    
    }
}

