/*
 * GPDocTransferHandler.java
 *
 * Created on April 3, 2004, 10:46 AM
 */

import java.awt.datatransfer.*;
import java.io.*;
import javax.swing.*;

/** 
 * A list transfer handler for tpdocument.
 *
 * @author TerpPresent
 * @author vxn
 * @version 3.0 TerpPresent
 */
public class TPGPDocTransferHandler extends TransferHandler {
    
    DataFlavor tpFlavor = null;
    boolean samedoc = true;

    /** A tpdocument. */
//    private TPDocument doc = null;
    private GPGraphpad graphpad = null;
    
    /** A list for the source. */
    JList source = null;
    
    /** An array for the indexes. */
    //private int[] indices = null;
    private int index = -1;
    
    /** Location where items were added. */
    private int addIndex = -1;
    
    /** Number of items added. */
    private int addCount = 0;
    
    /**
     * Creates a new instance of TPStringListTransferHandler.
     * @param doc The tpdocument for this handler.
     */
    public TPGPDocTransferHandler( GPGraphpad graphpad ) {// TPDocument doc ) {
        super();
//        this.doc = doc;
        this.graphpad = graphpad;
        try {
            tpFlavor = new DataFlavor( DataFlavor.javaJVMLocalObjectMimeType +
                                ";class=GPGraph" ); 
        } catch( ClassNotFoundException cnfe ) {
            System.err.println( "TPDocTransferHandler: Unable to create dataflavor." );
        }
    }
    
    /**
     * Return the boolean value if the component can be imported.
     * @param comp The component to be imported.
     * @param flavors An array of the importable flavors.
     * @return The boolean value if the component can be imported.
     */
    public boolean canImport( JComponent comp, DataFlavor[] flavors ) {
//        System.out.println( "can import" );
        if( tpFlavor == null ) {
            return false;
        }
        
        for( int i = 0 ; i < flavors.length ; i++ ) {
            if( flavors[ i ].equals( tpFlavor ) ) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Create a transferable for the component to be tranfered.
     * @param comp The component to be transfered.
     * @return The transferable for the component to be transfered.
     */
    protected Transferable createTransferable( JComponent comp ) {
//        System.out.println( "create transferable" );
        source = (JList)comp;
        //indices = source.getSelectedIndices();
        index = source.getSelectedIndex();
        //Object[] values = source.getSelectedValues();
        GPDocument doc = (GPDocument)source.getSelectedValue();
        
//        System.out.println( "about to create tr" );
        GPDocTransferable tr = new GPDocTransferable( doc.getGraph().getArchiveableState() );
//        System.out.println( "tr created" );
        return tr;
        //return new GPDocTransferable( value );
    }
    
    /**
     * Perform clean up operation once the transfer is completed.
     * @param source The source component transfered.
     * @param data The transferable for the component.
     * @param int The int representing the transfer action.
     */
    protected void exportDone( JComponent source, Transferable data, int action ) {
//        System.out.println( "export done" );
        if( action == MOVE && index != -1 ) {//indices != null ) {
            DefaultListModel model  = (DefaultListModel)(((JList)source).getModel());
            //If we are moving items around in the same list, we
            //need to adjust the indices accordingly, since those
            //after the insertion point have moved.
            if( addCount > 0 ) {
                /*for( int i = 0 ; i < indices.length ; i++ ) {
                    if( indices[i] > addIndex ) {
                        indices[i] += addCount;
                    }
                }*/
//                System.out.println( "here 1" );
                if( index > addIndex && samedoc == true ) {
//                    System.out.println( "here 2" );
                    index += addCount;
                }
                if( !samedoc ) {
//                    System.out.println( "not same doc ******" );
                }

                model.remove( index );
            }
            /*for( int i = indices.length - 1 ; i >= 0 ; i-- ) {
                model.remove(indices[ i ]);
            }*/
            if( index < model.size() ) {
                ((TPDocument)graphpad.getCurrentDocument()).changeCurrentSlide( index );
            } else if( model.size() > 0 ) {  //only moving one at a time
                ((TPDocument)graphpad.getCurrentDocument()).changeCurrentSlide( index - 1 );
            }
        }


        //indices = null;
        index = -1;
        addCount = 0;
        addIndex = -1;
        samedoc = true;
//        doc.reorderSlides();
    }
    
    /**
     * Imports the component.
     * @param comp The component to be transfered.
     * @param t The transferable for the component.
     * @return The boolean value if the transfer is successful.
     */
    public boolean importData( JComponent comp, Transferable t ) {
//        System.out.println( "import data" );
    	if ( canImport( comp, t.getTransferDataFlavors() ) ) {
    		try {
    			//String str = ( String )t.getTransferData( DataFlavor.stringFlavor );
                    Object obj = t.getTransferData( tpFlavor );
    			//myImport( comp, str );
                    myImport( comp, obj );
    			return true;
    		} catch ( UnsupportedFlavorException usfe ) {
    			System.err.println(
    			"UnsupportedFlavorException in TPStringListTransferHandler.importData()" );
    		} catch ( IOException ioe ) {
    			System.err.println(
    			"IOException in TPStringListTransferHandler.importData()" );
    		}
    	}
    	return false;
    }
    
    /**
     * Return the type of transfer action.
     * @param comp The component to be transfered.
     * @return The int value for the type of transfer action.
     */
    public int getSourceActions( JComponent comp ) {
 //       System.out.println( "get source actions" );
        return MOVE;
    }
    
    /**
     * Take the incoming string and wherever there is a
     * newline, break it into a separate item in the list.
     * @param comp The component to be transfered.
     * @param str The string to be tokenized.
     */
    private void myImport( JComponent comp, Object obj ) {//String str ) {
//        System.out.println( "my import" );
        JList target = (JList)comp;
        DefaultListModel listModel = (DefaultListModel)target.getModel();
        int index = target.getSelectedIndex();
        
        //Prevent the user from dropping data back on itself.
        //For example, if the user is moving items #4,#5,#6 and #7 and
        //attempts to insert the items after item #5, this would
        //be problematic when removing the original items.
        //So this is not allowed.
        if( source != null && source.equals( target ) ) {
            /*if( indices != null && index >= indices[ 0 ] - 1 &&
                index <= indices[ indices.length - 1 ] ) {
                indices = null;*/
//            System.out.println( "same list" );
            
            if( this.index == index ) {
//                System.out.println( "same list and index" );
                return;
            }
        }
//        System.out.println( "moveing" );
        
        if( !source.equals( target ) ) {
//         System.out.println( "setting false" );
            samedoc = false;
        }
        
        int max = listModel.getSize();
        if( index < 0 ) {
            index = max;
        } else {
            index++;
            if( index > max ) {
                index = max;
            }
        }
        addIndex = index;
        
       if( obj != null ) 
       {            
           //String [] values = str.split( "\n" );           
           //addCount = values.length;           
           addCount = 1;
           /*for( int i = 0 ; i < values.length ; i++ )
               listModel.add( index++, values[ i ]);*/
           //listModel.add( index++, obj );
  
           GraphModelProvider graphModelProvider = graphpad.getCurrentGraphModelProvider();
           GraphModel model = graphModelProvider.createCleanGraphModel();
           
           GPGraph gp = graphModelProvider.createCleanGraph( model );
           gp.setArchivedState( obj );
            
           //GPGraph gp = (GPGraph)obj;
           gp.setSelectNewCells( true );
           gp.setInvokesStopCellEditing( true );
           gp.setMarqueeHandler( graphpad.getMarqueeHandler() );
           listModel.add( index++, new GPDocument( graphpad, null, graphModelProvider, gp, model, null ) );
       }
       
       EditPaste.Paste() ; // Paste stuff from the clipboard onto the document.
    }
    
    
    
    public class GPDocTransferable implements Transferable {
        Object data;
        
        public GPDocTransferable( Object obj ) {
            data = obj;
        }
 
        public Object getTransferData( DataFlavor flavor )
                                 throws UnsupportedFlavorException {
            if( !isDataFlavorSupported( flavor ) ) {
                throw new UnsupportedFlavorException( flavor );
            }
            return data;
        }

        public DataFlavor[] getTransferDataFlavors() {
            return new DataFlavor[] { tpFlavor };
        }

        public boolean isDataFlavorSupported( DataFlavor flavor ) {
            if( tpFlavor.equals( flavor ) ) {
                return true;
            }
            return false;
        }
    }
}
