/*
 * TPGraphModelFileFormatSerial.java
 *
 * Created on March 20, 2004, 1:03 AM
 */

import java.io.*;
import java.util.*;
import java.util.zip.*;
import javax.swing.*;
import java.io.Serializable;
import javax.swing.filechooser.FileFilter;

/** 
 * A file format for tpdocument.  The format writes a serialized
 * tpdocument to the output stream.
 *
 * @author TerpPresent
 * @author vxn
 * @version 3.0 TerpPresent
 */
public class TPGraphModelFileFormatSerial implements GraphModelFileFormat, Serializable {
    
    /** Accessory component. A checkbox for the
    *  zipped output or unzipped output.
    * 
    */
    JComponent compZipSelect;
    
    /** File filter for this file format.
    */
    FileFilter fileFilter;
    
    /** A const value for the key at the properties hashtable.
        */
    public static final String COMPRESS_WITH_ZIP = "CompressWithZip";
    
    /** Creates a new instance of TPGraphModelFileFormatSerial */
    public TPGraphModelFileFormatSerial() {
        fileFilter = new FileFilter() {
            /**
            * @see javax.swing.filechooser.FileFilter#accept(File)
            */
            public boolean accept(File f) {
                if (f == null)
                    return false;
                if (f.getName() == null)
                    return false;
                if (f.getName().endsWith(".tpb_ser"))
                    return true;
                
                return false;
            }
            
            /**
            * @see javax.swing.filechooser.FileFilter#getDescription()
            */
            public String getDescription() {
                return "JGraphpad Diagram (*.tpb_ser)";
            }
        };
        compZipSelect = new JCheckBox(Translator.getString("zipCompress"));
    }
    
    /** Returns <tt>tpb_ser</tt>s
        * @return String type, "tpb_ser".
        */
    public String getFileExtension(){
        return "tpb_ser";
    }
    
    /** Returns a file filter for the <tt>tpb_ser</tt> extension.
        *
        * @see com.jgraph.pad.GraphModelFileFormat#getFileFilter()
        * @return FileFilter type, the current fileFilter is returned.
        */
    public FileFilter getFileFilter() {
        return fileFilter;
    }
    
    /** Returns the compZipSelect object.
        *
        * @see #compZipSelect
        * @see com.jgraph.pad.GraphModelFileFormat#getAccessory()
        * @return JComponent type, returns compZipSelect object.
        */
    public JComponent getWriteAccessory() {
        return compZipSelect;
    }
    
    /** Returns null.
        *
        * @see com.jgraph.pad.GraphModelFileFormat#getAccessory()
        * @return JComponent type, returns null.
        */
    public JComponent getReadAccessory() {
        return null;
    }
    
    /** Writes the graph as Object Output stream
        *
        * @see GraphModelFileFormat#write(String, Hashtable, GPGraph, GraphModel)
        * @param filename String type, current filename.
        * @param properties Hashtable type, the properties.
        * @param gpGraph GPGraph type, the gpgraph.
        * @param graphModel GraphModel type.
        * @throws Exception if no file found.
        * @deprecated Deprecated as of V3. This is an empty implementation.
        */
    public void write(
                      String filename,
                      Hashtable properties,
                      GPGraph gpGraph,
                      GraphModel graphModel)
        throws Exception {}

    /** Writes the serialized tpdocument to the output stream.
     *
     * @see GraphModelFileFormat#write(String, Hashtable, GPGraph, GraphModel)
     * @param filename String type, current filename.
     * @param properties Hashtable type, the properties.
     * @param gpGraphList The list of graphs in this graphpad.
     * @param graphModel GraphModel describing the graphs.
     * @throws Exception if no file found.
     */
    public void write( String filename, Hashtable properties, java.util.List gpGraphList, GraphModel graphModel ) throws Exception {
        OutputStream outStream = new FileOutputStream( filename );
        if( properties != null &&
            properties.get( COMPRESS_WITH_ZIP ) != null &&
            ( (Boolean)properties.get( COMPRESS_WITH_ZIP) ).booleanValue() ) {
            outStream = new GZIPOutputStream( outStream );
        }
        ObjectOutputStream out = new ObjectOutputStream( new BufferedOutputStream( outStream )) ;
        out.writeObject( new Integer( gpGraphList.size() ) );
        //out.writeInt( gpGraphList.size() );
        for( Iterator iter = gpGraphList.iterator() ; iter.hasNext(); ) {
            GPGraph gpGraph = (GPGraph)iter.next();
            out.writeObject( gpGraph.getArchiveableState());
            out.flush();
        }
        out.close();
        
    }
    
    /** Puts the value from the checkbox into the properties hashtable
        *
        * @see com.jgraph.pad.GraphModelFileFormat#getWriteConfiguration(JComponent)
        * @param accessory JComponent type.
        * @return Hashtable type, returns properties with JCheckbox selected.
        */
    public Hashtable getWriteProperties(JComponent accessory) {
        Hashtable properties = new Hashtable();
        properties.put(COMPRESS_WITH_ZIP, new Boolean(((JCheckBox)accessory).isSelected() ));
        return properties;
    }
    
    /** Reads the File form the Object input stream.
        * Tempts to load from a zipped input stream.
        * If this procedure fails the method tempts
        * to load without the zipped option.
        *
        * @see com.jgraph.pad.GraphModelFileFormat#read(String, Hashtable, GPGraph)
        * @param filename String type, file name.
        * @param properties Hashtable type, current properties.
        * @param gpGraph GPGraph type, current graph.
        * @return GraphModel type, returns the graphmodel read in.
        * @throws Exception no file found.
        */
    public GraphModel read(
                           String filename,
                           Hashtable properties,
                           GPGraph gpGraph)
        throws Exception {
            
            //empty implementation
            return null;

    }
    
    /** Reads the serialized tpdocument from the output stream.
     *
     * @see GraphModelFileFormat#read(String, Hashtable, GPGraph, GraphModel)
     * @param filename String type, current filename.
     * @param properties Hashtable type, the properties.
     * @param gpGraphList The list of graphs in this graphpad. 
     * @return GraphModel type, returns the graphmodel read in.
     * @throws Exception if no file found.
     */    
    public GraphModel read( String filename, Hashtable properties, java.util.List gpGraphList ) throws Exception { 
    	ObjectInputStream objIn = null;
    	//URL url = new URL ("file:/" + filename) ;
    	InputStream inStream ;
    	
        try{
            inStream = new FileInputStream(filename);
        	//inStream = url.openStream() ;
            inStream = new GZIPInputStream( inStream );
            objIn = new ObjectInputStream( inStream );
        } catch( Exception e ) {
            inStream = new FileInputStream(filename);
        	//inStream = url.openStream() ;
            objIn = new ObjectInputStream( new BufferedInputStream (inStream ));
        }
        GraphModel model = null;
        GraphModelProvider graphModelProvider = 
            GraphModelProviderRegistry.getGraphModelProvider( filename );
        
        //int size = objIn.readInt();
        int size = ((Integer)objIn.readObject()).intValue();
        for( int i = 1 ; i <= size ; i++ ) {
            GPGraph gpGraph = graphModelProvider.createCleanGraph(
                                graphModelProvider.createCleanGraphModel());
            gpGraph.setArchivedState( objIn.readObject() );
            gpGraphList.add( gpGraph );
            model = gpGraph.getModel();
        }
        
        objIn.close();
        return model;
    }
    
    /** Returns null
        *
        * @see com.jgraph.pad.GraphModelFileFormat#getReadConfiguration(JComponent)
        * @param accessory JComponent type, the accessory.
        * @return Hashtable type, returns null.
        */
    public Hashtable getReadProperties(JComponent accessory) {
        return null;
    }

    
}
