/*
 * TPGraphModelFileFormatXML.java
 *
 * Created on March 23, 2004, 1:14 PM
 */

import java.beans.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.io.Serializable;


/** 
 * A file format for tpdocument.  The format writes a
 * tpdocument to an xml output stream.
 *
 * @author TerpPresent
 * @author vxn
 * @version 3.0 TerpPresent
 */
public class TPGraphModelFileFormatXML implements GraphModelFileFormat, Serializable {
    
	/** File filter for this file format.
	 */
	FileFilter fileFilter;
	
	/** Accessory component. A checkbox for the
	 *  zipped output or unzipped output.
	 * 
	 */
	JComponent compZipSelect;
	
	/** A const value for the key at the properties hashtable.
	 */
	public static final String COMPRESS_WITH_ZIP = "CompressWithZip";

    /** Creates a new instance of TPGraphModelFileFormatXML */
    public TPGraphModelFileFormatXML() {
		fileFilter = new FileFilter() {
			/**
			 * @see javax.swing.filechooser.FileFilter#accept(File)
			 */
			public boolean accept(File f) {
				if (f == null)
					return false;
				if (f.getName() == null)
					return false;
				if (f.getName().endsWith(".tpb"))
					return true;

				return false;
			}

			/**
			 * @see javax.swing.filechooser.FileFilter#getDescription()
			 */
			public String getDescription() {
				return "JGraphpad Diagram (*.tpb)";
			}
		};
		compZipSelect = new JCheckBox(Translator.getString("zipCompress"));
	}
	
	/** Returns <tt>tpb</tt>.
         * @return String type, returns "tpb".
         */
	public String getFileExtension(){
		return "tpb";
	}
	

	/** Returns a file filter for the <tt>tpb</tt> extension.
         *
         * @see com.jgraph.pad.GraphModelFileFormat#getFileFilter()
         * @return FileFilter type, returns the fileFilter.
         */
	public FileFilter getFileFilter() {
		return fileFilter;
	}

	/** Returns null.
         * @see com.jgraph.pad.GraphModelFileFormat#getAccessory()
         * @return JComponent type, returns null.
         */
	public JComponent getReadAccessory() {
		return null;
	}

	/** Returns the compZipSelect object.
         *
         * @see #compZipSelect
         * @see com.jgraph.pad.GraphModelFileFormat#getAccessory()
         * @return JComponent type, returns compZipSelect value.
         */
	public JComponent getWriteAccessory() {
		return compZipSelect;
	}

	/** Writes the graph as XML file.
         *
         * @see GraphModelFileFormat#write(File, Hashtable, GPGraph, GraphModel)
         * @param filename String type, current filename.
         * @param properties Hashtable type, the properties.
         * @param gpGraph GPGraph type, the gpgraph.
         * @param graphModel GraphModel type.
         * @throws Exception if no file found.
         * @deprecated Deprecated as of V3. This is an empty implementation.
         */
	public void write(
		String filename,
		Hashtable properties,
		GPGraph gpGraph,
		GraphModel graphModel)
		throws Exception {}

     public void write( String filename, Hashtable properties, List gpGraphList, GraphModel graphModel ) throws Exception {
        OutputStream outStream = new FileOutputStream( filename );
        if( properties != null &&
            properties.get( COMPRESS_WITH_ZIP ) != null &&
            ( (Boolean)properties.get( COMPRESS_WITH_ZIP) ).booleanValue() ) {
            outStream = new GZIPOutputStream( outStream );
        }
        
        XMLEncoder out = new XMLEncoder( new BufferedOutputStream( outStream ) );
        out.writeObject( new Integer( gpGraphList.size() ) );
        for( Iterator iter = gpGraphList.iterator() ; iter.hasNext(); ) {
            GPGraph gpGraph = (GPGraph)iter.next();
            out.writeObject( gpGraph.getArchiveableState() );
            out.flush();
        }
        out.close();
    }

	/**
	 * Puts the value from the checkbox into the properties hashtable.
	 * 
	 * @see GraphModelFileFormat#getWriteConfiguration(JComponent)
         * @param accessory JComponent type.
         * @return Hashtable type, returns properties with JCheckbox selected.
         */
	public Hashtable getWriteProperties(JComponent accessory) {
		Hashtable properties = new Hashtable();
		properties.put(COMPRESS_WITH_ZIP, new Boolean(((JCheckBox)accessory).isSelected() ));
		return properties;
	}

	/**Reads the File form the XML input stream.
	 * Tempts to load from a zipped input stream.
	 * If this procedure fails the method tempts
	 * to load without the zipped option.
	 * 
	 * @see com.jgraph.pad.GraphModelFileFormat#read(File, Hashtable, GPGraph, GraphModel)
	 * @param filename String type, file name.
         * @param properties Hashtable type, current properties.
         * @param gpGraph GPGraph type, current graph.
         * @return GraphModel type, returns the graphmodel read in.
         * @throws Exception no file found.
         */
	public GraphModel read(
		String filename,
		Hashtable properties,
		GPGraph gpGraph)
		throws Exception {
                    
                    //empty implementation
                    return null;

        }
 
    /** Reads a tpdocument from the xml input stream.
     *
     * @see GraphModelFileFormat#read(String, Hashtable, GPGraph, GraphModel)
     * @param filename String type, current filename.
     * @param properties Hashtable type, the properties.
     * @param gpGraphList The list of graphs in this graphpad. 
     * @return GraphModel type, returns the graphmodel read in.
     * @throws Exception if no file found.
     */    
      public GraphModel read( String filename, Hashtable properties, List gpGraphList ) throws Exception {
        XMLDecoder in = null;
        try{
            InputStream inStream = new FileInputStream(filename);
            inStream = new GZIPInputStream( inStream );
            in = new XMLDecoder( new BufferedInputStream( inStream ) );
        } catch( Exception e ) {
            InputStream inStream = new FileInputStream(filename);
            in = new XMLDecoder( new BufferedInputStream( inStream ) );
        }
        GraphModel model = null;
        GraphModelProvider graphModelProvider = 
            GraphModelProviderRegistry.getGraphModelProvider( filename );
        
        int size = ((Integer)in.readObject()).intValue();
        for( int i = 1 ; i <= size ; i++ ) {
            GPGraph gpGraph = graphModelProvider.createCleanGraph(
                                graphModelProvider.createCleanGraphModel());
            gpGraph.setArchivedState( in.readObject() );
            gpGraphList.add( gpGraph );
            model = gpGraph.getModel();
        }
        in.close();
        return model;
    }


	/**Returns null.
	 * 
	 * @see com.jgraph.pad.GraphModelFileFormat#getReadConfiguration(JComponent)
	 * @param accessory JComponent type, the accessory.
         * @return Hashtable type, returns null.
         */
	public Hashtable getReadProperties(JComponent accessory) {
		return null;
	}

}
