/*
 * TPGraphModelProvider.java
 *
 * Created on March 23, 2004, 1:03 PM
 */

import java.util.*;
import javax.swing.tree.*;
import java.io.Serializable;

/** 
 * A Graph model provider for the tpdocument.
 *
 * @author TerpPresent
 * @author vxn
 * @version 3.0 TerpPresent
 */
public class TPGraphModelProvider implements GraphModelProvider, Serializable {
    
    /** Reference to the terp present xml file format.
    */
    TPGraphModelFileFormatXML tpGraphModelFileFormatXML; 
    
    /** Reference to the terp present serial file format.
    */
    //TPGraphModelFileFormatSerial tpGraphModelFileFormatSerial; 
    
    
    /** Creates a new instance of TPGraphModelProvider */
    public TPGraphModelProvider() {
        tpGraphModelFileFormatXML = new TPGraphModelFileFormatXML();
        //no longer supported
        //tpGraphModelFileFormatSerial = new TPGraphModelFileFormatSerial();
    }
    
    
    /** Returns <tt>TerpPresent Graph Model</tt>.
        *
        * @see com.jgraph.pad.GraphModelProvider#getPresentationName()
        * @return String type, TerpPresent Graph Model.
        */
    public String getPresentationName() {
        return "TerpPresent Graph Model";
    }
    
    /** Creates the Cells for the TerpPresent graph model.
        *
        * @see DefaultEdge
        * @see EllipseCell
        * @see ImageCell
        * @see DefaultPort
        * @see TextCell
        * @param model GraphModel type, the model.
        * @param type int type.
        * @param userObject Object type, the user object.
        * @param map Map type, the current map.
        * @return Object, returns type of userObject.
        */
    public Object createCell(GraphModel model, int type, Object userObject, Map map) {
        
        switch (type){
            case CELL_EDGE_DEFAULT :
                return new DefaultEdge(userObject);
            case CELL_VERTEX_ELLIPSE:
                return new EllipseCell(userObject);
            case CELL_VERTEX_IMAGE :
                return new ImageCell(userObject);
            case CELL_PORT_DEFAULT:
                return new DefaultPort(userObject);
            case CELL_VERTEX_TEXT:
                return new TextCell(userObject, true);
        }
        return new DefaultGraphCell(userObject);
    }
    
    
    
    /**
        * Creates a clean default graph model.
     * 
     * @see DefaultGraphModel
     * @see com.jgraph.pad.GraphModelProvider#createCleanGraphModel()
     * @return GraphModel type, the new default graph model.
     */
    public GraphModel createCleanGraphModel() {
        return new DefaultGraphModel();
    }
    
    /** Currently, returns false every time.
        * 
        * @see com.jgraph.pad.GraphModelProvider#isMutateAbleTo(Class)
        * @param otherGraphModelClass Class type, the other model class.
        * @return boolean type, false.
        */
    public boolean isMutateAbleTo(Class otherGraphModelClass) {
        return false;
    }
    
    /**
        * Currently, returns null every time.
     * 
     * @see com.jgraph.pad.GraphModelProvider#mutateTo(GraphModel, Class)
     * @param sourceGraphModel GraphModel type, the source graph model.
     * @param otherGraphModelClass Class type, the other graph model class.
     * @return GraphModel type, returns null.
     */
    public GraphModel mutateTo(
                               GraphModel sourceGraphModel,
                               Class otherGraphModelClass) {
        return null;
    }
    
    /**
        * Returns the graph model file format objects for this provider.
     * 
     * @see DefaultGraphModelFileFormatXML
     * @see DefaultGraphModelFileFormatSerial
     * @see com.jgraph.pad.GraphModelProvider#getGraphModelFileFormats()
     * @return GraphModelFileFormat[] type, the graph model file format object for the current provider.
     */
    public GraphModelFileFormat[] getGraphModelFileFormats() {
        //return new GraphModelFileFormat[]{tpGraphModelFileFormatXML, tpGraphModelFileFormatSerial};
        return new GraphModelFileFormat[]{tpGraphModelFileFormatXML};
    }
    
    /**Adds a Port to the model specific vertex object
        * 
        * @see com.jgraph.pad.GraphModelProvider#addPort(Object, Object)
        * @param vertex Object type, the vertex.
        * @param port Object type, the port.
        */
    public void addPort(Object vertex, Object port) {
        if (port instanceof DefaultPort){
            if (vertex instanceof DefaultMutableTreeNode){
                ((DefaultMutableTreeNode)vertex).add((DefaultPort)port);
            }
        }
    }
    
    /**
        * Creates a new clean graph for this model provider.
     * 
     * @see com.jgraph.pad.GraphModelProvider#createCleanGraph()
     * @param model GraphModel type, the model.
     * @return GPGraph type, a new GPgraph based on the model input.
     */
    public GPGraph createCleanGraph(GraphModel model) {
        return new GPGraph(model);
    }
    
}
