/*
 * TPListCellRenderer.java
 *
 * Created on March 9, 2004, 1:03 PM
 */

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

/** 
 * A renderer for the list of slides in TPDocument.
 *
 * @author TerpPresent
 * @author vxn
 * @version 3.0 TerpPresent
 */
public class TPListCellRenderer extends JLabel implements ListCellRenderer {
    
    /** The tpdocument. */
    private TPDocument doc;
    
    /** A border for the renderer. */
    protected Border noFocusBorder;
    
    /**
     * Creates a new instance of TPGraphModelProvider.
     * @param doc The tpdocument for this renderer.
     */
    public TPListCellRenderer( TPDocument doc ) {
        super();
        this.doc = doc;
        if (noFocusBorder == null) {
            noFocusBorder = new EmptyBorder(1, 1, 1, 1);
        }
        setOpaque(true);
        setBorder(noFocusBorder);
        setHorizontalAlignment(CENTER);
        setVerticalAlignment(CENTER);
    }
     
    /**
     * Returns the component to be used in rendering.
     * @param list A list of component in the list.
     * @param value The value of the object selected.
     * @param index The index of the selected object in the list.
     * @param isSelected The boolean value if the object is selected.
     * @param cellHasFocus The boolean value if the cell has focus.
     */
    public Component getListCellRendererComponent( JList list, Object value,
                                                   int index, boolean isSelected,
                                                   boolean cellHasFocus ) {
        if( isSelected ) {
            setBackground( list.getSelectionBackground() );
            //doc.changeCurrentSlide( (String)value );
            doc.changeCurrentSlide( index );
        } else {
            setBackground( list.getBackground() );
        }
        //String str = (String)value;
        //setText( str );
        setText( "Slide " + ( index + 1 ) );
        ImageIcon icon = ImageLoader.getImageIcon(
                                        Translator.getString( "Logo" ) );
        if( icon != null ) {
            setIcon(icon);
        } else {
            System.out.println( "TPListRenderer:  Error in loading icon." );
        }
        setEnabled(list.isEnabled());
        doc.getScrollPane().grabFocus();
        setFont(list.getFont());
        setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);
        return this;
    }
}