/*
 * @(#)ToolsMetric.java 1.2 02.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.event.ActionEvent;

import javax.swing.Icon;


/** Selects the tools metric option.
 * @author sven.luzar
 * @version 2.0 TerpPresent
 * @see com.jgraph.pad.actions.AbstractActionCheckBox
 */
public class ToolsMetric extends AbstractActionCheckBox {

 /** Constructor for ToolsMetric.
         * @param graphpad The parameter refers to the current graph.
         * @see com.jgraph.GPGraphpad
         */
 public ToolsMetric(GPGraphpad graphpad) {
  super(graphpad);
 }

 /** Constructor for ToolsMetric.
         * @param graphpad The parameter refers to the current graph.
         * @param name Specified string.
         * @see com.jgraph.GPGraphpad
         */
 public ToolsMetric(GPGraphpad graphpad, String name) {
  super(graphpad, name);
 }

 /** Constructor for ToolsMetric.
         * @param graphpad The parameter refers to the current graph.
         * @param name Specified string.
         * @param icon Specified icon.
         * @see com.jgraph.GPGraphpad
         */
 public ToolsMetric(GPGraphpad graphpad, String name, Icon icon) {
  super(graphpad, name, icon);
 }

 /** Checks if the metric option is selected.
         * @param actionCommand String that specifies the action.
         * @return True if option is selected, false otherwise.
         */
 public boolean isSelected(String actionCommand) {
  if (getCurrentDocument() == null) return false;
  
  return getCurrentDocument().getColumnRule().isMetric();
 }

 /** Selects the metric option.
         * @param e This parameter is called from selecting the corresponding menu item.
         * @see java.awt.event.ActionEvent
         */
 public void actionPerformed(ActionEvent e) {
  boolean metric = !getCurrentDocument().getColumnRule().isMetric();
  getCurrentDocument().getColumnRule().setIsMetric(metric);
  getCurrentDocument().getRowRule() .setIsMetric(metric);
 }

}
