import java.awt.*;
import java.io.*;
import java.awt.datatransfer.*;

/** Handles UM Clip Objects.
 * @author TerpPresent
 * @version 2.0 TerpPresent
 * @see Transferable, ClipboardOwner
 */
public class UMClipObj implements Transferable, ClipboardOwner
{	
	/** Main only used for standalone testing purposes.
	 */    
	public void main(){}
	
	/** Array containing the supported DataFlavors. Add or remove as necessary.
	 * @see #getTransferDataFlavors()
	 */
	private static final DataFlavor[] flavors = 
		{	
			DataFlavor.stringFlavor,
			DataFlavor.imageFlavor,
			DataFlavor.plainTextFlavor
	};
	
	// The following variables will store the clipboard data
	/** Stores data for plain text flavors.
	 */
	
	private String plainTextFlavorValue;
	/** Stores data for string flavors.
	 */
	private String stringFlavorValue;
	/** Stores data for image flavors.
	 */
	private Image imageFlavorValue;
	
	/** Constructor for UMClipObj class.
	 */
	UMClipObj(){ }
	
	/** Constructor for UMClipObj class.
	 * @param stringText Specified value for clipboard stringFlavorValue.
	 * @param plainText Specified value for clipboard plainTextValue.
	 */
	UMClipObj(String stringText, String plainText){
		this.plainTextFlavorValue = plainText;
		this.stringFlavorValue = stringText;
		this.imageFlavorValue = null;
	}
	
	/** Constructor for UMClipObj class.
	 * @param imageObject Specified image for imageFlavorValue.
	 * @see java.awt.image
	 */
	UMClipObj(Image imageObject, String application)
	{
		this.plainTextFlavorValue = application ;
		this.imageFlavorValue = imageObject;
	}
	
	/** Returns copy of supported DataFlavors array.
	 * @return Returns the array of DataFlavors.
	 */
	public DataFlavor[] getTransferDataFlavors() {
		return (DataFlavor[])flavors.clone();
	}
	
	/** Checks if the data in the clipboard is supported.
	 * @param flavor The data in question.
	 * @return Returns true if data is supported, false otherwise.
	 * @see DataFlavor
	 */
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		for (int i = 0; i < flavors.length; i++) {
			if (flavor.equals(flavors[i])) {
				return true;
			}
		}
		return false;
	}
	
	/** Returns the clipboard data depending on which DataFlavor is requested.
	 * @param flavor The data in question.
	 * @return Returns the appropriate flavor of data.
	 * @throws UnsupportedFlavorException Unsupported data format.
	 * @throws IOException Data not retrievable.
	 */
	public Object getTransferData(DataFlavor flavor)
	throws UnsupportedFlavorException, IOException
	{
		if (flavor.equals(DataFlavor.stringFlavor)) {
			
			// Insert processing for stringFlavor request here
			return (Object)stringFlavorValue;
			
		} else if (flavor.equals(DataFlavor.plainTextFlavor)) {
			
			// Insert processing for plainTextFlavor request here
			return (Object)plainTextFlavorValue;
			
		} else if (flavor.equals(DataFlavor.imageFlavor)) {
			
			// Insert processing for imageFlavor request here
			return (Object)imageFlavorValue;
			
		} else {
			
			System.out.println("Exception: Attempting to access clipboard with unsupported DataFlavor");
			throw new UnsupportedFlavorException(flavor);
			
		}
	}
	
	/** This function will be called if the program loses ownership of the clipboard.
	 * @param clipboard The system clipboard.
	 * @param transferable Transferrable object.
	 */
	public void lostOwnership(Clipboard clipboard,
			Transferable transferable)
	{
		System.out.println ("Lost Clipboard ownership");
	}
	
}


/**** The following clipboard functions can be implemented anywhere ****/
/*********************************************************************************
 */
//private void Copy()
//{
//	
//	
//	String copy_text = new String("this text will be copied to the clipboard");
//	BufferedImage copy_image = <<<some buffered image object>>>
//	
//	UMClipObj clip_obj;
//	clip_obj = new UMClipObj(copy_text, copy_text, copy_image);
//	
//	Clipboard clipboard;
//	clipboard=Toolkit.getDefaultToolkit().getSystemClipboard();
//	
//	try {
//		
//		clipboard.setContents(clip_obj, this);
//		
//	}catch (Exception e){
//		
//	}
//}
//
//
//public void Paste(){
//	
//	String fromClip = null;
//	Transferable transferable = null;
//	Object clipboardContents = null;
//	
//	clipboard=Toolkit.getDefaultToolkit().getSystemClipboard();
//	
//	if (clipboard == null){
//		System.out.println("Clipboard is null");
//		return;
//	}
//	try {
//		clipboardContents = clipboard.getContents(this);
//		
//		if (clipboardContents != null){
//			
//			transferable = clipboard.getContents(this);
//			
//			if (transferable.isDataFlavorSupported( <<< desired DataFlavor >>> )){
//				
//				fromClipboard = transferable.getTransferData( <<< desired DataFlavor >>>);
//				
//			} else
//				System.out.println("Clipboard data is unknown type");
//			
//		}
//	} catch(Exception e){
//		
//	}
//	
//	
//}
/*************************************************************************************/
