/*
 * @(#)Utilities.java	1.2 30.01.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
//import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Window;
import java.util.StringTokenizer;

/**
 * Class contains some basic utilities for this project.
 * 
 * @author sven.luzar
 * @version 2.0 TerpPresent
 *
 */
public class Utilities {

	/** Take the given string and chop it up into a series
         * of strings on whitespace boundries.  This is useful
         * for trying to get an array of strings out of the
         * resource file.
         * @param input this String will be tokenized
         * @return Returns a tokenized string  */
	public static String[] tokenize(String input) {
		if (input == null)
			return new String[]{};
		StringTokenizer t = new StringTokenizer(input);
		String cmd[];

		cmd = new String[t.countTokens()];
		int i = 0;
		while (t.hasMoreTokens()){
			cmd[i] = (String) t.nextToken();
			i++;
		}

		return cmd;
	}

	/** Returns a random number between 0 and max.
         * @param max the maximum number allowable
         * @return a random number  */
	public static int rnd(int max) {
		return (int) (Math.random() * max);
	}

	/** Parses the pattern and tries to
         *  parse each token as a float.
         *
         * @return array with the float value for each token
         * @param pattern the pattern to be parsed
         */
	public static float[] parsePattern(String pattern) {
		StringTokenizer st = new StringTokenizer(pattern, ",");
		float[] f = new float[st.countTokens()];
		if (f.length > 0) {
			int i = 0;
			while (st.hasMoreTokens())
				f[i++] = Float.parseFloat(st.nextToken());
		}
		return f;
	}
	/** Returns the classname without the package.
	 *  Example: If the input class is java.lang.String
	 *  than the return value is String.
	 * 
	 *  @param cl The class to inspect
	 *  @return The classname
	 * 
	 */
	public static String getClassNameWithoutPackage(Class cl){
		// build the name for this action
		// without the package prefix
		String className = cl.getName();
		int pos = className.lastIndexOf('.') + 1;
		if (pos == -1)
			pos = 0;
		String name = className.substring(pos);
		return name;
	}
	
        /** This allows centering of the window.
         * @param frame The window that should be centered
         */        
	public static void center(Window frame) {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = frame.getSize();
		frame.setLocation(
			screenSize.width / 2 - (frameSize.width / 2),
			screenSize.height / 2 - (frameSize.height / 2));
	}
	
}
