/*
 * @(#)ToolsShowStatusbar.java	1.2 02.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.event.ActionEvent;
import java.util.Enumeration;

import javax.swing.AbstractButton;



/**
 * This class is used to add or remove status bars from the application.
 *
 * @author sven.luzar
 * @version 1.0
 */
public class ViewShowStatusbar extends AbstractActionCheckBox {

  /**
   * Creates a new ViewShowStatusbar for the associated graphpad.
   *
   * @param graphpad The graph with which the ViewShowStatusBar is associated.
   */
  public ViewShowStatusbar(GPGraphpad graphpad) {
    super(graphpad);
  }

  /**
   * Determines whether the current status bar is visible.
   *
   * @param actionCommand - string that specifies the action. NOTE: THIS IS NEVER USED IN THE METHOD.
   *
   * @return - true if the status bar is visible, false otherwise
   */
  public boolean isSelected(String actionCommand) {
    return graphpad.getStatusBar().isVisible();
  }

  /**
   * Makes the status bar visible if it is not already visible.
   *
   * @param e - This parameter is called from selecting the corresponding menu item. NOTE: THIS IS NEVER USED IN THE METHOD.
   */
  public void actionPerformed(ActionEvent e) {
    graphpad.getStatusBar().setVisible(
      !graphpad.getStatusBar().isVisible());

  }
  /**
   * Updates all the abstract buttons that may have changed as a result of ViewStatusBar action.
   */
  public void update(){
    Enumeration enum = abstractButtons.elements();
    while (enum.hasMoreElements()) {
      AbstractButton button = (AbstractButton) enum.nextElement();
      button.setSelected(isSelected(button.getActionCommand()));
    }
  };

}
