/*
 * @(#)ToolsShowToolbar.java	1.2 02.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.event.ActionEvent;
import java.util.Enumeration;

import javax.swing.AbstractButton;
import javax.swing.Icon;


/**
 * Class that controls the views of the toolbars in the application.
 *
 * @author sven.luzar
 * @version 2.0 TerpPresent.
 */
public class ViewShowToolbar extends AbstractActionCheckBox {

  /**
   * Creates a new ViewShowToolbar object and associates the current graph with it.
   *
   * @param graphpad The graphpad with which this object is being associated.
   */
  public ViewShowToolbar(GPGraphpad graphpad) {
    super(graphpad);
  }

  /** Creates a new ViewShowToolbar object and associates the current graph with it.
   *
   * @param graphpad The graphpad with which this object is being associated.
   * @param name The key for the name of this action (show toolbar).
   */
  public ViewShowToolbar(GPGraphpad graphpad, String name) {
    super(graphpad, name);
  }

  /**
   * Creates a new ViewShowToolbar object and associates the current graph with it.
   *
   * @param graphpad The graphpad with which this object is being associated.
   * @param name The key for the name of this action (show toolbar).
   * @param icon The icon associated with this action.
   */
  public ViewShowToolbar(GPGraphpad graphpad, String name, Icon icon) {
    super(graphpad, name, icon);
  }

  /**
   * Determines whether the toolbar is selected i.e. is visible.
   *
   * @param actionCommand  String that specifies the action. NOTE: THIS IS NOT USED IN THIS METHOD.
   *
   * @return - True if the toolbar is visible, false otherwise.
   */
  public boolean isSelected(String actionCommand) {
    return graphpad.isToolBarsVisible();
  }

  /**
   * Makes the toolbar visible if it not already viewable.
   *
   * @param e - The event associated with the view toolbar event. NOTE: THIS IS NOT USED IN THIS METHOD.
   */
  public void actionPerformed(ActionEvent e) {
    graphpad.setToolBarsVisible(!graphpad.isToolBarsVisible());
    graphpad.invalidate();
    graphpad.repaint();
    update();
  }

  /**
   * Updates all the abstract buttons that may have changed as a result of the ViewToolbar action.
   */
  public void update(){
    Enumeration enum = abstractButtons.elements();
    while (enum.hasMoreElements()) {
      AbstractButton button = (AbstractButton) enum.nextElement();
      button.setSelected(isSelected(button.getActionCommand()));
    }
  };

}
