/*
 * @(#)WindowWindows.java	1.2 09.02.2003
 *
 * Copyright (C) 2003 sven.luzar
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.event.ActionEvent;
//import java.awt.event.ContainerEvent;
//import java.awt.event.ContainerListener;
import java.beans.PropertyVetoException;
import java.util.Vector;

import javax.swing.JInternalFrame;
import javax.swing.JMenu;
//import javax.swing.JMenuItem;



/** public class
 * @author sven.luzar
 * @version 1.0
 */
public class WindowWindows
  extends AbstractActionList {

  /**
   * A vector representing all the menus available in the graph.
   */
  Vector menus = new Vector();

  /**
   * Creates a new WindowWindows object for the specified graph.
   *
   * @param graphpad The graphpad that is being used currently.
   */
  public WindowWindows(GPGraphpad graphpad) {
    super(graphpad);
  }

  /**
   * Returns an array of objects of length 0.
   *
   * @return An empty array of objects.
   */
  protected Object[] getItems() {
    return new Object[] {
    };
  }

  /**
   * Gets the GPInternalFrame from the ActionEvent, brings it to the front the pad and makes it visible.
   *
   * @param e - The ActionEvent associated with bringing a frame to the front.
   */
  public void actionPerformed(ActionEvent e) {
    GPInternalFrame gpframe = (GPInternalFrame) this.getSelectedItem(e);
    gpframe.toFront();
    try {
      gpframe.setSelected(true);
    } catch (PropertyVetoException pve) {
    }
  }

  /**
   * Constructs a menu based on the menu currently in the graphpad and returns it. It also updates the menus vector.
   *
   * @see #menus
   * @see com.jgraph.pad.actions.AbstractActionList#getMenuBarComponent()
   * @return A JMenu object that represents all the menus currently in the graphpad.
   */
  protected JMenu getMenuBarComponent() {
    JMenu menu = new JMenu(this);
    menus.add(menu);
    return menu;
  }

  /**
   * Returns the actionCommand. NOTE: IT IS SUPPOSED TO RETURN THE PRESENTATION TEXT ASSOCIATED WITH AN actionCOMMAND, BUT
   * IT RETURNS THE actionCOMMAND ITSELF.
   *
   * @param actionCommand String that specifies the action.
   * @return Supposed to return the presentation file name from the document, but returns the actionCommand itself.
   */
  public String getPresentationText(String actionCommand) {
    return actionCommand;
  }

  /**
   * Returns the presentationText for the itemValue. NOTE: In the current implementation, this method ALWAYS RETURNS NULL.
   *
   * @param itemValue The item for which we want to find the presentation text.
   * @return Always returns null
   */
  public String getItemPresentationText(Object itemValue) {
    return null;
  }

  /**
   * Updates the window list and the menu entries after the action has been performed.
   *
   */
  public void update(){
    super.update() ;

    JInternalFrame[] iframes = graphpad.getAllFrames();

    for (int j = 0; j < menus.size(); j++) {
      JMenu menu = (JMenu) menus.get(j);
      menu.removeAll();

      for (int i = 0; i < iframes.length; i++) {
        GPInternalFrame internalFrame = (GPInternalFrame) iframes[i];
        menu.add(
          getMenuComponent(
            internalFrame.getDocument().getFrameTitle(),
            internalFrame));
      }
    }
  }

}
